@@ -683,6 +683,18 @@ public class ClassType extends ObjectType
     flags |= ADD_FIELDS_DONE;
   }
 
+    public void removeField(Field field, Field prev) {
+        if (field != (prev == null ? fields : prev.next))
+            new Error();
+        if (prev == null)
+            fields = field.next;
+        else
+            prev.next = field.next;
+        if (last_field == field)
+            last_field = null;
+        fields_count--;
+    }
+
   Method methods;
   int methods_count;
   Method last_method;
@@ -945,15 +957,22 @@ public class ClassType extends ObjectType
   public Method getDeclaredMethod(String name, Type[] arg_types)
   {
     int needOuterLinkArg = "<init>".equals(name) && hasOuterLink() ? 1 : 0;
+    Method found = null;
     for (Method method = getDeclaredMethods();
 	 method != null;  method = method.next)
       {
 	if (! name.equals(method.getName()))
 	  continue;
 	Type[] method_args = method.getParameterTypes();
+        boolean synthetic =
+            (method.getModifiers() & (Access.SYNTHETIC|Access.BRIDGE)) != 0;
 	if (arg_types == null
             || (arg_types == method_args && needOuterLinkArg==0))
-	  return method;
+          {
+            if (! synthetic)
+              return method;
+            found = method;
+          }
 	int i = arg_types.length;
 	if (i != method_args.length-needOuterLinkArg)
 	  continue;
@@ -969,9 +988,13 @@ public class ClassType extends ObjectType
 	      break;
 	  }
 	if (i < 0)
-	  return method;
+          {
+            if (! synthetic)
+              return method;
+            found = method;
+          }
       }
-    return null;
+    return found;
   }
 
   synchronized Method getDeclaredMethod(String name, boolean mustBeStatic, int argCount)
@@ -1105,6 +1128,18 @@ public class ClassType extends ObjectType
     return result;
   }
 
+    public <T extends java.lang.annotation.Annotation>
+    T getAnnotation(Class<T> clas) {
+        T ann = RuntimeAnnotationsAttr.getAnnotation(this, clas);
+        if (ann != null)
+            return ann;
+        if ((flags & EXISTING_CLASS) != 0 && getReflectClass() != null) {
+            Class<?> c = getReflectClass();
+            return c.getAnnotation(clas);
+        }
+        return null;
+    }
+
   /** Do various fixups after generating code but before we can write it out.
    * This includes assigning constant pool indexes where needed,
    * finalizing labels, etc. */
@@ -1354,6 +1389,21 @@ public class ClassType extends ObjectType
     public int isCompatibleWithValue(Type valueType) {
         if (this == objectType && valueType instanceof ObjectType)
             return 2;
+        if (valueType == Type.nullType)
+            return 2;
+        if (isInterface()) {
+            Type rawType = valueType.getRawType();
+            if (! (rawType instanceof ClassType))
+                return -1;
+            else if (rawType == objectType)
+                return 0;
+            else if (((ClassType) rawType).implementsInterface(this))
+                return 2;
+            else
+                // In theory a subclass of valueType might implement this
+                // interface, but that is not normal practice.
+                return -1;
+        }
         int comp = compare(valueType);
         if (comp >= 0) {
             if (valueType instanceof ClassType
