@@ -451,7 +451,11 @@ public abstract class Type
         // Hack for LangPrimType.charType.
         if (this == charType && valueType.getImplementationType()== this)
             return 2;
-        int comp = compare(valueType);
+        return isCompatibleWithValue(this, valueType);
+    }
+
+    public static int isCompatibleWithValue(Type targetType, Type valueType) {
+        int comp = targetType.compare(valueType);
         return comp >= 0 ? 1 : comp == -3 ? -1 : 0;
     }
 
@@ -553,7 +557,8 @@ public abstract class Type
 
     /** Convert an object to a value of this Type.
      * The result is actually of the implementation type, boxed as appropriate,
-     * so it is suitable for standard reflective operations.
+     * so it is suitable for standard reflective operations,
+     * like the arguments to Field#set or Method#invoke.
      * Throw a ClassCastException when this is not possible.
      */
     public abstract Object coerceFromObject (Object obj);
