@@ -172,7 +172,6 @@ public class Declaration
     if (type.isVoid())
       type = Type.objectType;
     this.type = type;
-    if (var != null) var.setType(type);
     typeExp = QuoteExp.getInstance(type);
   }
 
@@ -221,6 +220,15 @@ public class Declaration
   Variable var;
   public Variable getVariable() { return var; }
 
+    /** If non-zero, indicates this names a 'scan' of a sequence.
+     * The value property is a sequence; the declaration names
+     * "each" of the elements.
+     * This field is the amount of nesting within scan ('...') contexts.
+     */
+    public int getScanNesting() { return scanNesting; }
+    public void setScanNesting(int value) { scanNesting = value; }
+    int scanNesting;
+
   public final boolean isSimple()
   { return (flags & IS_SIMPLE) != 0; }
 
@@ -246,7 +254,9 @@ public class Declaration
    * If IS_FLUID, base points to IS_UNKNOWN Symbol. */
   public Declaration base;
 
-  public Field field;
+    public Field field;
+    public Method getterMethod;
+    public Method setterMethod;
 
   /** If this is a field in some object, load a reference to that object. */
   void loadOwningObject (Declaration owner, Compilation comp)
@@ -292,9 +302,9 @@ public class Declaration
           }
       }
     CodeAttr code = comp.getCode();
-    Type rtype = getType();
-    if (! isIndirectBinding()
-        && (flags & ReferenceExp.DONT_DEREFERENCE) != 0)
+    boolean dontDeref = (flags & ReferenceExp.DONT_DEREFERENCE) != 0;
+    Type rtype = dontDeref ? Compilation.typeLocation : getType();
+    if (! isIndirectBinding() && dontDeref)
       {
         if (field == null)
           throw new Error("internal error: cannot take location of "+this);
@@ -358,6 +368,7 @@ public class Declaration
               }
             else
               code.emitGetStatic(field);
+            code.fixUnsigned(getType());
           }
         else if (isClassField())
           {
@@ -400,8 +411,7 @@ public class Declaration
               var = allocateVariable(code, true);
             code.emitLoad(var);
           }
-        if (isIndirectBinding()
-            && (flags & ReferenceExp.DONT_DEREFERENCE) == 0)
+        if (isIndirectBinding() && ! dontDeref)
           {
             String filename;
             int line;
@@ -446,8 +456,13 @@ public class Declaration
               }
             else
               code.emitInvokeVirtual(getLocationMethod);
-
-            rtype = Type.pointer_type;
+            if (isAlias()
+                || target.getType().getRawType() == Type.objectType)
+                rtype = Type.objectType;
+            else {
+                getType().emitCoerceFromObject(code);
+                rtype = getType();
+            }
           }
       }
     target.compileFromStack(comp, rtype);
@@ -529,6 +544,7 @@ public class Declaration
   public static final int IS_SYNTAX = 0x8000;
   public static final int IS_UNKNOWN = 0x10000;
   public static final int IS_IMPORTED = 0x20000;
+    public static final int IS_CAPTURED = IS_IMPORTED; // Re-use bit.
 
   // This should be a type property, not a variable property, at some point!
   public static final int IS_SINGLE_VALUE = 0x40000;
@@ -578,23 +594,42 @@ public class Declaration
   public static final long ENUM_ACCESS = 0x200000000l;
   public static final long FINAL_ACCESS = 0x400000000l;
   public static final long ABSTRACT_ACCESS = 0x800000000l;
+  public static final long SYNCHRONIZED_ACCESS = 0x1000000000l;
+  public static final long STRICTFP_ACCESS = 0x2000000000l;
   public static final long CLASS_ACCESS_FLAGS =
     PRIVATE_ACCESS|PROTECTED_ACCESS|ENUM_ACCESS|FINAL_ACCESS|ABSTRACT_ACCESS;
   public static final long FIELD_ACCESS_FLAGS = PRIVATE_ACCESS|PROTECTED_ACCESS|
     PUBLIC_ACCESS|PACKAGE_ACCESS|VOLATILE_ACCESS|TRANSIENT_ACCESS|
     ENUM_ACCESS|FINAL_ACCESS;
   public static final long METHOD_ACCESS_FLAGS = PRIVATE_ACCESS
-    |PROTECTED_ACCESS|PUBLIC_ACCESS|PACKAGE_ACCESS|FINAL_ACCESS;
-  public static final long MAYBE_UNINITIALIZED_ACCESS = 0x1000000000l;
+    |PROTECTED_ACCESS|PUBLIC_ACCESS|PACKAGE_ACCESS|FINAL_ACCESS
+    |SYNCHRONIZED_ACCESS|STRICTFP_ACCESS;
+
+    public static final long MAYBE_UNINITIALIZED_ACCESS = 0x4000000000l;
   /** Allocate variable on JVM stack as an optimization.
    * This means load is implemented as a dup instruction.
    * (This is no faster on decent JVMs, but the bytecode is more compact.)
    * Note this may cause an InternalError if this is loaded when the
    * JVM stack has grown since the variable was initialized.
    */
-  public static final long ALLOCATE_ON_STACK = 0x2000000000l;
+  public static final long ALLOCATE_ON_STACK = 0x8000000000l;
 
-  protected long flags = IS_SIMPLE;
+    /** True for a variable inside a pattern, but not the top of the pattern.
+     * E.g. for a pattern [a [b c]] there is a main declaration for the
+     * incoming value - which does not have PATTERN_NESTED set.  There are
+     * three declarations a, b, and c - which do have PATTERN_NESTED set.
+     * In addition there may be helper variables which are anonymous
+     * (i.e. getSymbol() returns null), like the match for the sequence [b c];
+     * these also have PATTERN_NESTED set. */
+    public static final long PATTERN_NESTED = 0x10000000000l;
+
+    /** See parameterForMethod() */
+    public static final long SKIP_FOR_METHOD_PARAMETER = 0x20000000000l;
+    public static final long IS_REST_PARAMETER = 0x40000000000l;
+    public static final long IS_PARAMETER = 0x80000000000l;
+    public static final long DONT_COPY = 0x10000000000l;
+
+    protected long flags = IS_SIMPLE;
 
   public final boolean getFlag (long flag)
   {
@@ -649,6 +684,10 @@ public class Declaration
       flags |= Access.ENUM;
     if (getFlag(FINAL_ACCESS))
       flags |= Access.FINAL;
+    if (getFlag(SYNCHRONIZED_ACCESS))
+      flags |= Access.SYNCHRONIZED;
+    if (getFlag(STRICTFP_ACCESS))
+      flags |= Access.STRICT;
     return flags;
   }
 
@@ -678,7 +717,18 @@ public class Declaration
   public final boolean isNamespaceDecl ()
   {
     return (flags & IS_NAMESPACE_PREFIX) != 0;
-  }   
+  }
+
+    /** Is this a parameter for the generated method?
+     * For example if a lambda's parameter is the pattern {@code [x y]}
+     * then there is an anonymous parameter p0 that is the incoming 2-element
+     * sequence plus named parameters {@code x} and {@code y}.
+     * The later two are parameters for the generated method,
+     * but the anonymous p0 is not.
+     */
+    public final boolean parameterForMethod() {
+        return ! getFlag(SKIP_FOR_METHOD_PARAMETER);
+    }
 
   /** True if the value of the variable is the contents of a Location.
    * @see #INDIRECT_BINDING */
@@ -714,6 +764,10 @@ public class Declaration
       base.setCanRead();
   }
 
+    public final boolean getCanReadOrCall() {
+        return (flags & (CAN_READ|CAN_CALL)) != 0;
+    }
+
   public final boolean getCanCall() { return (flags & CAN_CALL) != 0; }
   public final void setCanCall(boolean called) { setFlag(called, CAN_CALL); }
   public final void setCanCall()
@@ -1143,15 +1197,17 @@ public class Declaration
     int fflags = 0;
     boolean isConstant = getFlag(IS_CONSTANT);
     boolean typeSpecified = getFlag(TYPE_SPECIFIED);
-    if (comp.getModule().getFlag(ModuleExp.INTERACTIVE)
-        && context == comp.getModule()
-        && ! isConstant && ! typeSpecified)
-      setIndirectBinding(true);
     // In immediate mode we may need to access the field from a future
     // command in a different "runtime package" (see JVM spec) because it
     // gets loaded by a different class loader.  So make the field public.
-    if (isPublic() || external_access || comp.immediate)
-      fflags |= Access.PUBLIC;
+    if (isPublic() || external_access || comp.immediate) {
+        fflags |= Access.PUBLIC;
+        // FIXME do setIndirectBinding even if it's constant,
+        // as long as just a define-constant or ! and not a class or alias
+        if (comp.isInteractive() && context == comp.getModule()
+            && ! isConstant)
+            setIndirectBinding(true);
+    }
     if (isStatic()
         // "Dynamic" variables use ThreadLocation, based on the current
         // Environment, so we don't need more than one static field.
@@ -1168,8 +1224,15 @@ public class Declaration
         && (context instanceof ClassExp || context instanceof ModuleExp))
       fflags |= Access.FINAL;
     Type ftype = getType().getImplementationType();
-    if (isIndirectBinding() && ! ftype.isSubtype(Compilation.typeLocation))
-      ftype = Compilation.typeLocation;
+    if (isIndirectBinding() && ! ftype.isSubtype(Compilation.typeLocation)) {
+        if (ftype == null || ftype == Type.objectType)
+            ftype = Compilation.typeLocation;
+        else {
+            if (ftype instanceof PrimType)
+                ftype = ((PrimType) ftype).boxedType();
+            ftype = new ParameterizedType(Compilation.typeLocation, ftype);
+        }
+    }
     if (! ignorable())
       {
         String dname = getName();
@@ -1191,7 +1254,7 @@ public class Declaration
               }
             if (external_access && ! getFlag(Declaration.MODULE_REFERENCE))
               {
-                fname = PRIVATE_PREFIX + fname; 
+                fname = PRIVATE_PREFIX + fname;
                 haveName = false;
               }
            nlength = fname.length();
@@ -1265,6 +1328,7 @@ public class Declaration
       }
     // The EARLY_INIT case is handled in SetExp.compile.
     if (! shouldEarlyInit()
+        && context instanceof ModuleExp
 	&& (isIndirectBinding()
 	    || (value != null && ! (value instanceof ClassExp))))
       {
@@ -1527,7 +1591,7 @@ public class Declaration
       }
   }
 
-  public void noteValueFromLet (LetExp letter)
+  public void noteValueFromLet (ScopeExp letter)
   {
     Expression init = getInitValue();
     if (init != QuoteExp.undefined_exp && values != unknownValueValues)
@@ -1554,19 +1618,17 @@ public class Declaration
         return true;
     }
 
-  private void checkNameDecl (Expression value)
-  {
-    if (nvalues == 1 && values[0].kind == ValueSource.GENERAL_KIND)
-      {
-        Expression old = values[0].base;
-        if (old == value)
-          return;
-        if (old instanceof LambdaExp)
-          ((LambdaExp) old).nameDecl = null;
-      }
-    if (nvalues == 0 && value instanceof LambdaExp)
-      ((LambdaExp) value).nameDecl = this;
-  }
+    private void checkNameDecl(Expression value) {
+        if (nvalues == 1) {
+            Expression old = values[0].getValue(this);
+            if (old == value)
+                return;
+            if (old instanceof LambdaExp)
+                ((LambdaExp) old).nameDecl = null;
+        }
+        if (value instanceof LambdaExp)
+            ((LambdaExp) value).nameDecl = nvalues == 0 ? this : null;
+    }
 
   public static class ValueSource
   {
