@@ -92,8 +92,19 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
       }
     if (! skipFunc)
       exp.func = exp.func.visit(this, ignored);
-    if (exitValue == null && ! skipArgs)
-      exp.args = visitExps(exp.args, ignored);
+    if (exitValue == null && ! skipArgs) {
+        int nargs = exp.args.length;
+        for (int i = 0; i < nargs;  i++) {
+            Expression arg = visit(exp.args[i], null);
+            // It is possible that visiting args[i] may cause prefix
+            // arguments to be inserted (search for IS_CAPTURED)
+            // so we need to adjust for that.
+            int inserted = exp.args.length - nargs;
+            i += inserted;
+            exp.args[i] = arg;
+            nargs += inserted;
+        }
+    }
     if (backJumpPossible > oldBackJumpPossible)
       exp.setFlag(ApplyExp.MAY_CONTAIN_BACK_JUMP);
     return exp;
@@ -181,7 +192,7 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
             Declaration bind = allocUnboundDecl(name, false);
             if (! decl.getFlag(Declaration.IS_DYNAMIC))
               maybeWarnNoDeclarationSeen(name, comp, exp);
-            capture(bind);
+            capture(bind, null);
             decl.base = bind;
           }
       }
@@ -232,68 +243,10 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
     return super.visitLetExp(exp, ignored);
   }
 
-  static Expression checkInlineable (LambdaExp current,
-                                     java.util.Set<LambdaExp> seen)
-  {
-    if (current.returnContinuation == LambdaExp.unknownContinuation)
-      return current.returnContinuation;
-    if (seen.contains(current))
-      return current.returnContinuation;
-    if (current.getCanRead()
-        || current.isClassMethod()
-        || Compilation.avoidInline(current)
-        || current.min_args != current.max_args)
-      {
-        current.returnContinuation = LambdaExp.unknownContinuation;
-        return LambdaExp.unknownContinuation;
-      }
-    seen.add(current);
-    Expression r = current.returnContinuation;
-    if (current.tailCallers != null)
-      {
-        for (LambdaExp p : current.tailCallers)
-          {
-            Expression t = checkInlineable(p, seen);
-            if (t == LambdaExp.unknownContinuation)
-              {
-                if (r == null || r == p.body)
-                  {
-                    r = p.body;
-                    current.inlineHome = p;
-                  }
-                else
-                  {
-                    current.returnContinuation = LambdaExp.unknownContinuation;
-                    return t;
-                  }
-              }
-            else if (r == null)
-              {
-                r = t;
-                if (current.inlineHome == null)
-                  current.inlineHome = current.nestedIn(p) ? p : p.inlineHome;
-              }
-            else if ((t != null && r != t)
-                     || current.getFlag(LambdaExp.CANNOT_INLINE))
-              {
-                current.returnContinuation = LambdaExp.unknownContinuation;
-                return LambdaExp.unknownContinuation;
-              }
-          }
-      }
-    return r;
-  }
-
   protected Expression visitLambdaExp (LambdaExp exp, Void ignored)
   {
-    java.util.Set<LambdaExp> seen = new java.util.LinkedHashSet<LambdaExp>();
-    // Finish the job that was started in FindTailCalls.
-    Expression caller = checkInlineable(exp, seen);
-    if (caller != LambdaExp.unknownContinuation)
-      {
-        exp.setInlineOnly(true);
+    if (exp.getInlineOnly())
         backJumpPossible++;
-      }
     return super.visitLambdaExp(exp, ignored);
   }
 
@@ -312,9 +265,9 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
         return exp;
     }
 
-  public void capture(Declaration decl)
+  public void capture(Declaration decl, ReferenceExp rexp)
   {
-    if (! (decl.getCanRead() || decl.getCanCall()))
+    if (! decl.getCanReadOrCall())
       return;
     if (decl.field != null && decl.field.getStaticFlag())
       return;
@@ -403,20 +356,73 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
     if (decl.base != null)
       {
 	decl.base.setCanRead(true);
-	capture(decl.base);
+	capture(decl.base, null);
       }
-    else if (decl.getCanRead() || decl.getCanCall() || declValue == null)
+    else if (decl.getCanReadOrCall() || declValue == null)
       {
 	if (! decl.isStatic())
 	  {
 	    LambdaExp heapLambda = curLambda;
+
+            // Perform the "lambda lifting" optimization, if possible:
+            //   (define (f x) ... y ...)
+            //   (f a)
+            // to:
+            //   (define (f y$ x) ... y$ ...)
+            //   (f y a)
+            // This can avoid the need for creating a closure.
+            if (rexp != null
+                && decl.nvalues == 1 && ! decl.hasUnknownValue()
+                && ! (decl.getValueRaw() instanceof LambdaExp)
+                // don't confuse call/cc inlining (over-conservative )
+                && ! decl.getFlag(Declaration.DONT_COPY)
+                && ! curLambda.getInlineOnly() // FIXME - for simplicity
+                && ! curLambda.getCanRead() && curLambda.nameDecl != null
+                && ! curLambda.nameDecl.context.isClassGenerated()
+                && curLambda.min_args == curLambda.max_args) {
+                Declaration ndecl = null;
+                for (ndecl = curLambda.firstDecl();  ndecl != null;
+                     ndecl = ndecl.nextDecl()) {
+                    if (ndecl.getFlag(Declaration.IS_CAPTURED)
+                        && ndecl.base == decl)
+                        break;
+                }
+                if (ndecl == null) {
+                    ndecl = new Declaration(decl.getSymbol());
+                    ndecl.base = decl;
+                    ndecl.setFlag(Declaration.IS_CAPTURED);
+                    ndecl.setCanRead(true);
+                    curLambda.add(null, ndecl);
+                    curLambda.min_args++;
+                    curLambda.max_args++;
+                    for (ApplyExp exp = curLambda.nameDecl.firstCall;
+                         exp != null;  exp = exp.nextCall) {
+                        LambdaExp context = exp.context;
+                        Expression[] args = exp.getArgs();
+                        Expression[] nargs = new Expression[args.length+1];
+                        boolean recursive = exp.context == curLambda;
+                        ReferenceExp ref =
+                            new ReferenceExp(recursive ? ndecl : decl);
+                        nargs[0] = ref;
+                        System.arraycopy(args, 0, nargs, 1, args.length);
+                        exp.setArgs(nargs);
+                        LambdaExp saveLambda = currentLambda;
+                        currentLambda = context;
+                        capture(decl, ref);
+                        currentLambda = saveLambda;
+                    }
+                }
+                rexp.setBinding(ndecl);
+                return;
+            }
+
             if (! decl.isFluid())
               heapLambda.setImportsLexVars();
 	    LambdaExp parent = heapLambda.outerLambda();
 	    for (LambdaExp outer = parent;  outer != declLambda && outer != null; )
 	      {
 		heapLambda = outer;
-		if (! decl.getCanRead() && declValue == outer)
+		if (! decl.getCanReadOrCall() && declValue == outer)
 		  break;
 		Declaration heapDecl = heapLambda.nameDecl;
 		if (heapDecl != null
@@ -495,11 +501,11 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
       maybeWarnNoDeclarationSeen(exp.getSymbol(), exp.isProcedureName(),
                                  comp, exp);
 
-    capture(exp.contextDecl(), decl);
+    capture(exp.contextDecl(), decl, exp);
     return exp;
   }
 
-  void capture (Declaration containing, Declaration decl)
+  void capture(Declaration containing, Declaration decl, ReferenceExp exp)
   {
     Expression dvalue;
     if (decl.isAlias() && (dvalue = decl.getValue()) instanceof ReferenceExp)
@@ -509,7 +515,7 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
 	if (orig != null
 	    && (containing == null || ! orig.needsContext()))
 	  {
-	    capture(rexp.contextDecl(), orig);
+            capture(rexp.contextDecl(), orig, null);
 	    return;
 	  }
       }
@@ -518,9 +524,9 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
         decl = decl.base;
       }
     if (containing != null && decl.needsContext())
-      capture(containing);
+        capture(containing, null);
     else
-      capture(decl);
+        capture(decl, exp);
   }
 
   protected Expression visitThisExp (ThisExp exp, Void ignored)
@@ -550,7 +556,7 @@ public class FindCapturedVars extends ExpExpVisitor<Void>
       {
 	if (! exp.isDefining())
 	  decl = Declaration.followAliases(decl);
-	capture(exp.contextDecl(), decl);
+	capture(exp.contextDecl(), decl, null);
       }
     return super.visitSetExp(exp, ignored);
   }
