@@ -78,7 +78,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
                 comp.error('w', "void-valued expression where value is needed",
                        exp0);
             // To avoid cascading warnings.
-            return Compilation.makeCoercion(exp, Type.objectType);
+            return new BeginExp(exp, QuoteExp.nullExp);
         }
         return checkType(exp, required);
     }
@@ -143,15 +143,16 @@ public class InlineCalls extends ExpExpVisitor<Type> {
                         +" is incompatible with required type "
                         +language.formatType(required)),
                        exp);
-            if (required instanceof PrimType) {
-                // Box the value to force a run-time ClassCastException.
-                Type boxed = ((PrimType) required).boxedType();
-                ApplyExp expb =
-                    Compilation.makeCoercion(exp, boxed);
-                expb.setType(required);
-                expb.setFlag(Expression.VALIDATED);
-                return expb;
-            }
+           
+            // Box if needed to force a run-time ClassCastException.
+            if (required instanceof PrimType)
+                required = ((PrimType) required).boxedType();
+            // Creating an explicit coercion avoids cascading error messages.
+            ApplyExp expb =
+                Compilation.makeCoercion(exp, required);
+            expb.setType(required);
+            expb.setFlag(Expression.VALIDATED);
+            return expb;
         }
         return exp;
     }
@@ -242,9 +243,9 @@ public class InlineCalls extends ExpExpVisitor<Type> {
     }
 
     protected Expression visitQuoteExp(QuoteExp exp, Type required) {
-        Object value;
-        if (exp.getRawType() == null && ! exp.isSharedConstant()
-            && (value = exp.getValue()) != null) {
+        Object value = exp.getValue();
+        if (exp.getRawType() == null
+                && ! exp.isSharedConstant() && value != null) {
             Language language = comp.getLanguage();
             Type vtype = language.getTypeFor(value.getClass());
             if (vtype == Type.toStringType)
@@ -261,20 +262,11 @@ public class InlineCalls extends ExpExpVisitor<Type> {
                         Object ival = null;
                         switch (sig1) {
                         case 'B':
-                            if (ivalue.inRange(Byte.MIN_VALUE, Byte.MAX_VALUE))
-                                ival = Byte.valueOf(ivalue.byteValue());
-                            break;
                         case 'S':
-                            if (ivalue.inRange(Short.MIN_VALUE, Short.MAX_VALUE))
-                                ival = Short.valueOf(ivalue.shortValue());
-                            break;
                         case 'I':
-                            if (ivalue.inRange(Integer.MIN_VALUE, Integer.MAX_VALUE))
-                                ival = Integer.valueOf(ivalue.intValue());
-                            break;
                         case 'J':
-                            if (ivalue.inRange(Long.MIN_VALUE, Long.MAX_VALUE))
-                                ival = Long.valueOf(ivalue.longValue());
+                            ival = LangPrimType.convertIntegerLiteral(ivalue,
+                                    (PrimType) primRequired, false);
                             break;
                         case 'F':
                             ival = Float.valueOf(ivalue.floatValue());
@@ -379,7 +371,7 @@ public class InlineCalls extends ExpExpVisitor<Type> {
                     && (dtype == null || dtype == Type.objectType
                         // We could also allow (some) widening conversions.
                         || dtype == rdecl.getType()))
-                    return visitReferenceExp(rval, required);
+                    return visitReferenceExp(new ReferenceExp(rval), required);
             }
             if (dval instanceof ClassExp && processingAnnotations()) {
                 ClassExp cval = (ClassExp) dval;
@@ -604,13 +596,26 @@ public class InlineCalls extends ExpExpVisitor<Type> {
         return exp;
     }
 
-    protected Expression visitLetExp (LetExp exp, Type required)
-    {
-        if (! (exp instanceof CatchClause) && ! (exp instanceof FluidLetExp))
-        {
+    protected Expression visitLetExp(LetExp exp, Type required) {
+        if (! (exp instanceof CatchClause) && ! (exp instanceof FluidLetExp)) {
             for (Declaration decl = exp.firstDecl();  decl != null;
-                 decl = decl.nextDecl())
-            {
+                 decl = decl.nextDecl()) {
+
+                // Minor optimization. Even better would be to replace
+                // the entire LetExp with the Declaration's init expression:
+                // (let ((x expr)) x) ==> expr
+                // However, that runs into various complications with
+                // type-checking, void-value-checking, and correctly setting
+                // CAN_READ on a LambdaExp. This is simpler and (because
+                // we have the value-propagation framework) almost as good.
+                if (exp.body instanceof ReferenceExp) {
+                    ReferenceExp ref = (ReferenceExp) exp.body;
+                    if (ref.getBinding() == decl
+                            && ! ref.getDontDereference()) {
+                        decl.setFlag(Declaration.ALLOCATE_ON_STACK);
+                        ref.setFlag(ReferenceExp.ALLOCATE_ON_STACK_LAST);
+                    }
+                }
                 Expression init = decl.getInitValue();
                 if (init == QuoteExp.undefined_exp
                     && decl.getValueRaw() instanceof LambdaExp)
@@ -639,24 +644,6 @@ public class InlineCalls extends ExpExpVisitor<Type> {
 
         if (exitValue == null)
             exp.body = visit(exp.body, required);
-        if (exp.body instanceof ReferenceExp) {
-            ReferenceExp ref = (ReferenceExp) exp.body;
-            Declaration d = ref.getBinding();
-            if (d != null && d.context == exp && ! ref.getDontDereference()) {
-                if (exp.firstDecl() == d && d.nextDecl() == null) // Single decl
-                {
-                    Expression init = d.getInitValue();
-                    Expression texp = d.getTypeExp();
-                    // Note this optimization does yield worse error messages
-                    // than using CheckedTarget.  FIXME.
-                    if (texp != QuoteExp.classObjectExp)
-                        init = visitApplyOnly(Compilation.makeCoercion(init, texp), null);
-                    return init;
-                }
-                // Can also optimize if n > 1, but have to check if any
-                // other inits can cause side-effects.  Probably not worth it.
-            }
-        }
         visitRemainingDeclaredLambdas(exp);
         return exp;
     }
