@@ -51,13 +51,17 @@ public class LambdaExp extends ScopeExp {
 
     /** The location for function start, before arguments are stored. */
     Label startForInlining;
+    /** Queue of deferred inline class.
+     * The items are (deferred-lambda, Target) pairs.
+     */
+    LinkedList<Object> pendingInlines;
 
     public void capture(Declaration decl) {
         if (decl.isSimple()) {
             if (capturedVars == null
                 && ! decl.isStatic()
-                && ! (this instanceof ModuleExp || this instanceof ClassExp)) {
-                heapFrame = new gnu.bytecode.Variable("heapFrame");
+                && ! isClassGenerated()) {
+                heapFrame = new gnu.bytecode.Variable("$heapFrame");
             }
             decl.setSimple(false);
             if (! decl.isPublic()) {
@@ -67,6 +71,12 @@ public class LambdaExp extends ScopeExp {
         }
     }
 
+    public Declaration addParameter(Object name) {
+        min_args++;
+        max_args++;
+        return super.addDeclaration(name);
+    }
+
     /** A local variable that points to the heap-allocated part of the frame.
      * Each captured variable is a field in the heapFrame.  A procedure has
      * a heapFrame iff if has a parameter or local variable that is
@@ -114,8 +124,10 @@ public class LambdaExp extends ScopeExp {
     /** If non-null, a Declaration whose value is (only) this LambdaExp. */
     public Declaration nameDecl;
 
+    public static final String CLOSURE_ENV_NAME = "$closureEnv";
+
     /** If non-null, this is a Field that is used for implementing lexical closures.
-     * If getName() is "closureEnv", it is our parent's heapFrame,
+     * If getName() is CLOSURE_ENV_NAME, it is our parent's heapFrame,
      * which is an instance of one of our siblings.
      * (Otherwise, we use "this" as the implicit "closureEnv" field.) */
     public Field closureEnvField;
@@ -135,7 +147,7 @@ public class LambdaExp extends ScopeExp {
      * See declareClosureEnv and closureEnvField. */
     Variable closureEnv;
 
-    static final int CAN_READ = 2;
+    static final int CAN_READ = Expression.NEXT_AVAIL_FLAG; // 2
     static final int INLINE_ONLY = 4;
     static final int IMPORTS_LEX_VARS = 8;
     static final int NEEDS_STATIC_LINK = 16;
@@ -150,6 +162,8 @@ public class LambdaExp extends ScopeExp {
     public static final int OVERLOADABLE_FIELD = 2048;
     public static final int ATTEMPT_INLINE = 4096;
     public static final int IN_EXPWALKER = 0x2000;
+    /** Treat as inlined in outer lambda when determining tailcalls. */
+    public static final int PASSES_TAILCALLS = 0x4000;
 
     /** True of emitted method should be public.
      * Needed if PrimProcedure.getMethodFor shold be able to find it.
@@ -163,6 +177,17 @@ public class LambdaExp extends ScopeExp {
     public final void setInlineOnly(boolean inlineOnly)
     { setFlag(inlineOnly, INLINE_ONLY); }
 
+    /** Note this function is inlined in a give context.
+     * This is meant to be used during validate-apply processing,
+     * for procedures that will be inlined in a compile method.
+     */
+    public final void setInlineOnly(Expression returnContinuation,
+                                    LambdaExp caller) {
+        setInlineOnly(true);
+        this.returnContinuation = returnContinuation;
+        this.inlineHome = caller;
+    }
+
     public final boolean getNeedsClosureEnv() {
         return (flags & (NEEDS_STATIC_LINK|IMPORTS_LEX_VARS)) != 0;
     }
@@ -237,11 +262,6 @@ public class LambdaExp extends ScopeExp {
     /** True iff this is the dummy top-level function of a module body. */
     public final boolean isModuleBody() { return this instanceof ModuleExp; }
 
-    /** True if a class is generated for this procedure.  */
-    public final boolean isClassGenerated() {
-        return isModuleBody() || this instanceof ClassExp;
-    }
-
     public boolean isAbstract() {
         return body == QuoteExp.abstractExp;
     }
@@ -435,11 +455,12 @@ public class LambdaExp extends ScopeExp {
      * outer.  Usually the same as (this.outerLambdaNotInline()==outer),
      * except in the case that outer.getInlineOnly(). */
     boolean inlinedIn(LambdaExp outer) {
-        for (LambdaExp exp = this; exp.getInlineOnly(); exp = exp.getCaller()) {
+        for (LambdaExp exp = this; ; exp = exp.getCaller()) {
             if (exp == outer)
                 return true;
+            if (! exp.getInlineOnly())
+                return false;
         }
-        return false;
     }
 
     /** For an INLINE_ONLY function, return the function it gets inlined in. */
@@ -480,7 +501,7 @@ public class LambdaExp extends ScopeExp {
                     closureEnv = declareThis(primMethod.getDeclaringClass());
                 else {
                     Type envType = primMethod.getParameterTypes()[0];
-                    closureEnv = new Variable("closureEnv", envType);
+                    closureEnv = new Variable(CLOSURE_ENV_NAME, envType);
                     Variable prev;
                     if (isInit)
                         prev = declareThis(primMethod.getDeclaringClass());
@@ -490,14 +511,10 @@ public class LambdaExp extends ScopeExp {
                     closureEnv.setParameter(true);
                 }
             } else {
-                Variable parentFrame = parent.heapFrame != null ?  parent.heapFrame
+                closureEnv =
+                    parent.heapFrame != null && parent == outerLambda()
+                    ? parent.heapFrame
                     : parent.closureEnv;
-                if (inlinedIn(parent))
-                    closureEnv = parentFrame;
-                else {
-                    closureEnv = new Variable("closureEnv", parentFrame.getType());
-                    getVarScope().addVariable(closureEnv);
-                }
             }
         }
         return closureEnv;
@@ -558,14 +575,31 @@ public class LambdaExp extends ScopeExp {
 
     public void compileEnd (Compilation comp) {
         gnu.bytecode.CodeAttr code = comp.getCode();
+        HashMap<String,Variable> varMap = new HashMap<String,Variable>();
+
+        Label endLabel = new Label(code);
+        while (pendingInlines != null && ! pendingInlines.isEmpty()) {
+            LambdaExp child = (LambdaExp) pendingInlines.remove();
+            Target ctarget = (Target) pendingInlines.remove();
+            if (child.getInlineOnly()
+                && ! child.getFlag(LambdaExp.METHODS_COMPILED)
+                && child.startForInlining != null) {
+                if (code.reachableHere())
+                    code.emitGoto(endLabel);
+                child.compileAsInlined(comp, ctarget);
+            }
+        }
+        if (endLabel.isUsed())
+            endLabel.define(code);
+        code.getCurrentScope().fixParamNames(varMap);
+        popScope(code);        // Undoes enterScope in allocParameters
+
         if (! getInlineOnly()) {
             if (comp.method.reachableHere()
                 && (getCallConvention() < Compilation.CALL_WITH_TAILCALLS
-                    || isModuleBody() || isClassMethod() || isHandlingTailCalls()))
+                    || isModuleBody() || isClassMethod() || isHandlingTailCalls())) {
                 code.emitReturn();
-            HashMap<String,Variable> varMap = new HashMap<String,Variable>();
-            code.getCurrentScope().fixParamNames(varMap);
-            popScope(code);        // Undoes enterScope in allocParameters
+            }
             code.getCurrentScope().fixParamNames(varMap);
             code.popScope(); // Undoes pushScope in method.initCode.
         }
@@ -762,13 +796,18 @@ public class LambdaExp extends ScopeExp {
                     allocMethod(outerLambda(), comp);
                 compileAsMethod(comp);
                 addApplyMethod(comp, null);
+                Variable savedInstance = comp.moduleInstanceVar;
+                comp.moduleInstanceVar = null;
                 ProcInitializer.emitLoadModuleMethod(this, comp);
+                comp.moduleInstanceVar = savedInstance;
             } else {
                 Field field = compileSetField(comp);
                 if (field.getStaticFlag())
                     code.emitGetStatic(field);
                 else {
                     LambdaExp parent = comp.curLambda;
+                    while (parent.getInlineOnly() && parent.heapFrame == null)
+                        parent = parent.outerLambda();
                     Variable frame
                         = parent.heapFrame != null ? parent.heapFrame
                         : parent.closureEnv;
@@ -781,7 +820,7 @@ public class LambdaExp extends ScopeExp {
     }
 
     public ClassType getHeapFrameType() {
-        if (this instanceof ModuleExp || this instanceof ClassExp)
+        if (isClassGenerated())
             return (ClassType) getType();
         else
             return (ClassType) heapFrame.getType();
@@ -1139,8 +1178,14 @@ public class LambdaExp extends ScopeExp {
     public void allocChildClasses(Compilation comp) {
         Method main = getMainMethod();
 	
-        if (main != null && ! main.getStaticFlag())
+        if (main != null && ! main.getStaticFlag()) {
+            CodeAttr code = comp.getCode();
+            // So this gets declared in the parameter_scope.
+            scope = code.getCurrentScope();
             declareThis(main.getDeclaringClass());
+            thisVariable.allocateLocal(code);
+            scope = null;
+        }
 
         Declaration decl = firstDecl();
         for (;;) {
@@ -1163,13 +1208,13 @@ public class LambdaExp extends ScopeExp {
             Variable var = decl.var;
             // i is the register to use for the current parameter
             if (var != null
-                || (getInlineOnly() && decl.ignorable()))
+                || (getInlineOnly() && decl.ignorable())
+                || ! decl.parameterForMethod())
                 ;
             else if (decl.isSimple () && ! decl.isIndirectBinding()) {
                 // For a simple parameter not captured by an inferior lambda,
                 // just allocate it in the incoming register.
                 var = decl.allocateVariable(null);
-                //var.allocateLocal(code);
             } else {
                 // This variable was captured by an inner lambda.
                 // Its home location is in the heapFrame.
@@ -1205,7 +1250,7 @@ public class LambdaExp extends ScopeExp {
         }
         if (! getNeedsClosureEnv())
             closureEnvType = null;
-        else if (outer instanceof ClassExp || outer instanceof ModuleExp)
+        else if (outer.isClassGenerated())
             closureEnvType = outer.getCompiledClassType(comp);
         else {
             LambdaExp owner = outer;
@@ -1216,6 +1261,22 @@ public class LambdaExp extends ScopeExp {
         addMethodFor(comp, closureEnvType);
     }
 
+    public void pushChild(LambdaExp child) {
+        child.nextSibling = firstChild;
+        firstChild = child;
+    }
+
+    public void reverseChildList() {
+        LambdaExp prev = null, child = firstChild;
+        while (child != null) {
+            LambdaExp next = child.nextSibling;
+            child.nextSibling = prev;
+            prev = child;
+            child = next;
+        }
+        firstChild = prev;
+    }
+
     void allocChildMethods(Compilation comp) {
         for (LambdaExp child = firstChild;  child != null;
              child = child.nextSibling) {
@@ -1223,7 +1284,7 @@ public class LambdaExp extends ScopeExp {
                 ClassExp cl = (ClassExp) child;
                 if (cl.getNeedsClosureEnv())  {
                     ClassType parentFrameType;
-                    if (this instanceof ModuleExp || this instanceof ClassExp)
+                    if (isClassGenerated())
                         parentFrameType = (ClassType) getType();
                     else {
                         Variable parentFrame = this.heapFrame != null
@@ -1241,7 +1302,7 @@ public class LambdaExp extends ScopeExp {
     public void allocFrame(Compilation comp) {
         if (heapFrame != null)  {
             ClassType frameType;
-            if (this instanceof ModuleExp || this instanceof ClassExp)
+            if (isClassGenerated())
                 frameType = getCompiledClassType(comp);
             else  {
                 frameType = new ClassType(comp.generateClassName("frame"));
@@ -1254,17 +1315,11 @@ public class LambdaExp extends ScopeExp {
 
     void allocParameters(Compilation comp) {
         CodeAttr code = comp.getCode();
-
-        int i = 0;
-        int j = 0;
-
-        code.locals.enterScope(getVarScope());
+        Scope sc = getVarScope();
+        code.locals.enterScope(sc);
         int line = getLineNumber();
         if (line > 0)
             code.putLineNumber(getFileName(), line);
-
-        if (heapFrame != null)
-            heapFrame.allocateLocal(code);
     }
 
     static Method searchForKeywordMethod3;
@@ -1309,7 +1364,7 @@ public class LambdaExp extends ScopeExp {
             if (closureEnv != null && ! (this instanceof ModuleExp))
                 staticLinkField = frameType.addField("staticLink",
                                                      closureEnv.getType());
-            if (! (this instanceof ModuleExp) && ! (this instanceof ClassExp)) {
+            if (! isClassGenerated()) {
                 frameType.setEnclosingMember(comp.method);
                 code.emitNew(frameType);
                 code.emitDup(frameType);
@@ -1321,7 +1376,9 @@ public class LambdaExp extends ScopeExp {
                     code.emitLoad(closureEnv);
                     code.emitPutField(staticLinkField);
                 }
+                heapFrame.allocateLocal(code);
                 code.emitStore(heapFrame);
+                code.pushAutoPoppableScope().addVariable(heapFrame);
             }
         }
 
@@ -1490,6 +1547,24 @@ public class LambdaExp extends ScopeExp {
         comp.callContextVar = callContextSave;
     }
 
+    void compileAsInlined(Compilation comp, Target target) {
+        flags |= LambdaExp.METHODS_COMPILED;
+	LambdaExp saveLambda = comp.curLambda;
+	comp.curLambda = this;
+	allocChildClasses(comp);
+	allocParameters(comp);
+        CodeAttr code = comp.getCode();
+        if (startForInlining == null)
+            startForInlining = new Label(code);
+        startForInlining.define(code);
+	ApplyExp.popParams(code, this, null, false);
+	enterFunction(comp);
+	body.compileWithPosition(comp, target);
+	compileEnd(comp);
+	generateApplyMethods(comp);
+	comp.curLambda = saveLambda;
+    }
+
     void compileAsMethod(Compilation comp) {
         if ((flags & METHODS_COMPILED) != 0 || isAbstract() || isNative())
             return;
@@ -1803,7 +1878,8 @@ public class LambdaExp extends ScopeExp {
             i = -1;
         for (; decl != null;  decl = decl.nextDecl()) {
             Special mode;
-            if (i < min_args)
+            if (i < min_args
+                || (i == min_args && decl.getFlag(Declaration.PATTERN_NESTED)))
                 mode = null;
             else if (i < min_args + opt_args)
                 mode = Special.optional;
@@ -1817,9 +1893,7 @@ public class LambdaExp extends ScopeExp {
                 out.print(mode);
                 out.writeSpaceFill();
             }
-            Expression defaultArg = null;
-            if (mode == Special.optional || mode == Special.key)
-                defaultArg = decl.getInitValue();
+            Expression defaultArg = decl.getInitValue();
             if (defaultArg != null)
                 out.print('(');
             decl.printInfo(out);
@@ -1828,7 +1902,8 @@ public class LambdaExp extends ScopeExp {
                 defaultArg.print(out);
                 out.print(')');
             }
-            i++;
+            if (decl.getFlag(Declaration.IS_PARAMETER))
+                i++;
             prevMode = mode;
         }
         out.endLogicalBlock(")");
