@@ -4,12 +4,11 @@
 package gnu.expr;
 import gnu.mapping.*;
 import gnu.bytecode.*;
-import gnu.kawa.io.Path;
+import gnu.kawa.io.*;
 import java.util.*;
 import gnu.kawa.util.AbstractWeakHashTable;
 
 public class ModuleInfo {
-
     /** Name of class that implements module.
      * Must be non-null unless we're currently compiling the module,
      * in which case sourcePath and comp must both be non-null.
@@ -17,6 +16,7 @@ public class ModuleInfo {
     protected String className;
 
     Class moduleClass;
+    private Class oldClass;
 
     static ClassToInfoMap mapClassToInfo = new ClassToInfoMap();
 
@@ -42,8 +42,10 @@ public class ModuleInfo {
         String fileName;
         if (mod != null && (fileName = mod.getFileName()) != null) {
             this.sourcePath = fileName;
-            Path abs = absPath(fileName);
-            this.sourceAbsPath = abs;
+            InPort port = comp.lexer == null ? null : comp.lexer.getPort();
+            if (! (port instanceof CharArrayInPort
+                   || port instanceof TtyInPort))
+                this.sourceAbsPath = absPath(fileName);
         }
     }
 
@@ -60,14 +62,19 @@ public class ModuleInfo {
     int numDependencies;
 
     /** Location of source for module, if known.
-     * This is an absolute URI, absolute filename,
+     * This can be any of an absolute URI, absolute filename,
      * or filename relative to current working directory.
      * Null if source not known; in that case className must be non-null.
      * Avoid using, since "relative to current working directory"
      * is unreliable if the working directory can change.
      */
     public String sourcePath;
+
+    /** Absolute and canonicalized location of source for module, if known.
+     */
     Path sourceAbsPath;
+    /** Absolute and canonicalized location of source for module, if known.
+     */
     String sourceAbsPathname;
 
     public long lastCheckedTime;
@@ -154,6 +161,7 @@ public class ModuleInfo {
                     if (fld == null
                         || decl.isIndirectBinding()
                         || (fld.getFlags() & Access.STATIC) == 0
+                        || (fld.getFlags() & Access.FINAL) == 0
                         || decl.getValueRaw() instanceof QuoteExp)
                         continue;
                     try {
@@ -228,6 +236,10 @@ public class ModuleInfo {
         return moduleClass;
     }
 
+    public Class getOldModuleClass() {
+        return oldClass;
+    }
+
     public void setModuleClass(Class clas) {
         moduleClass = clas;
         className = clas.getName();
@@ -332,8 +344,9 @@ public class ModuleInfo {
     public boolean checkCurrent(ModuleManager manager, long now) {
         if (sourceAbsPath == null)
             return true;
-        if (lastCheckedTime + manager.lastModifiedCacheTime >= now)
-            return moduleClass != null;
+        if (lastCheckedTime + manager.lastModifiedCacheTime >= now
+            && moduleClass != null)
+            return true;
         long lastModifiedTime = sourceAbsPath.getLastModified();
         long oldModifiedTime = this.lastModifiedTime;
         this.lastModifiedTime = lastModifiedTime;
@@ -368,6 +381,7 @@ public class ModuleInfo {
             }
         }
         if (lastModifiedTime > oldModifiedTime) {
+            oldClass = moduleClass;
             moduleClass = null;
             return false;
         }
