@@ -1,4 +1,4 @@
-// Copyright (c) 1999, 2000, 2008  Per M.A. Bothner.
+// Copyright (c) 1999, 2000, 2008, 2015  Per M.A. Bothner.
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.expr;
@@ -36,6 +36,12 @@ public class PrimProcedure extends MethodProc {
      * '\0' means don't expect a target. */
     private char mode;
     private boolean sideEffectFree;
+    /** Do we needs to zero-truncate the returned result?
+     * A Kawa-compiled function will do so before loading the value
+     * on the stack (and hence before returning), but post-return
+     * truncation may be needed if calling a Java method.
+     */     
+    private boolean doFixUnsigned;
 
     /** If non-null, the LambdaExp that this PrimProcedure implements. */
     private LambdaExp source;
@@ -45,7 +51,10 @@ public class PrimProcedure extends MethodProc {
     public final int opcode() { return op_code; }
 
     public Type getReturnType () { return retType; }
-    public void setReturnType (Type retType) { this.retType = retType; }
+    public void setReturnType (Type retType) {
+        this.doFixUnsigned = true;
+        this.retType = retType;
+    }
 
     public boolean isSpecial() { return mode == 'P'; }
 
@@ -342,6 +351,13 @@ public class PrimProcedure extends MethodProc {
       : method.getReturnType();
   }
 
+    public PrimProcedure(Method method, Type retType, Type[] argTypes) {
+        init(method);
+        this.retType = retType;
+        if (argTypes != null)
+            this.argTypes = argTypes;
+    }
+
   public PrimProcedure(Method method, Language language)
   {
     this(method, '\0', language, null);
@@ -393,9 +409,10 @@ public class PrimProcedure extends MethodProc {
         }
     }
 
-    static Type decodeType(Type javaType, String[] annotTypes, int annotIndex,
-                           ParameterizedType parameterizedType,
-                           Language lang) {
+    public static Type decodeType(Type javaType,
+                                  String[] annotTypes, int annotIndex,
+                                  ParameterizedType parameterizedType,
+                                  Language lang) {
         String annotType = annotTypes != null && annotTypes.length > annotIndex
             ? annotTypes[annotIndex] : null;
         return lang.decodeType(javaType, annotType, parameterizedType);
@@ -592,7 +609,7 @@ public class PrimProcedure extends MethodProc {
 	Target target =
 	  source == null ? CheckedTarget.getInstance(argTypeForTarget, name, i+1)
 	  : CheckedTarget.getInstance(argTypeForTarget, source, i);
-	args[startArg+i].compileNotePosition(comp, target, args[startArg+i]);
+	args[startArg+i].compileWithPosition(comp, target);
         if (createVarargsNow) // Only if explicitArrayAsVarArgsAllowed
           {
             // Wrap final argument in array if not already an array:
@@ -634,7 +651,10 @@ public class PrimProcedure extends MethodProc {
     Expression[] args = exp.getArgs();
     if (isConstructor())
       {
-        if (exp.getFlag(ApplyExp.MAY_CONTAIN_BACK_JUMP))
+        if (exp.getFlag(ApplyExp.MAY_CONTAIN_BACK_JUMP)
+            // See https://bugs.openjdk.java.net/browse/JDK-8046233
+            // The fix was released in JDK 8u25.
+            && Compilation.defaultClassFileVersion <= ClassType.JDK_1_8_VERSION)
           {
             // JVM spec for Java6:
             // "There must never be an uninitialized class instance
@@ -729,7 +749,7 @@ public class PrimProcedure extends MethodProc {
     else
       {
         compileInvoke(comp, methodForInvoke, target,
-                      exp.isTailCall(), op_code, retType);
+                      exp.isTailCall(), op_code, retType, doFixUnsigned);
       }
   }
 
@@ -739,7 +759,8 @@ public class PrimProcedure extends MethodProc {
    */
   public static void
   compileInvoke (Compilation comp, Method method, Target target,
-                 boolean isTailCall, int op_code, Type returnType)
+                 boolean isTailCall, int op_code, Type returnType,
+                 boolean doFixUnsigned)
   {
     CodeAttr code = comp.getCode();
     comp.usedClass(method.getDeclaringClass());
@@ -795,6 +816,10 @@ public class PrimProcedure extends MethodProc {
         returnType = Type.objectType;
         code.pushScope();
         Variable saveIndex = code.addLocal(Type.intType);
+
+        // FIXME: It would be better (unless we're in a try block) to put
+        // the cleanupFromContext handler inside a fragment (moved to the
+        // end of the Code attribute). See CheckedTarget#emitCheckedCoerce
         comp.loadCallContext();
         code.emitInvokeVirtual(Compilation.typeCallContext.
                                getDeclaredMethod("startFromContext", 0));
@@ -834,7 +859,8 @@ public class PrimProcedure extends MethodProc {
             } else
                 returnType = method.getReturnType().getRawType();
         }
-
+        if (doFixUnsigned)
+          code.fixUnsigned(returnType);
       target.compileFromStack(comp, returnType);
     }
   }
