@@ -76,6 +76,12 @@ public class ReferenceExp extends AccessExp
     this(binding.getSymbol(), binding);
   }
 
+    public ReferenceExp(ReferenceExp old) {
+        this(old.symbol, old.binding);
+        this.setContextDecl(old.contextDecl());
+        this.flags = old.flags;
+    }
+
   protected boolean mustCompile () { return false; }
 
   public final Object valueIfConstant()
@@ -241,7 +247,7 @@ public class ReferenceExp extends AccessExp
   protected gnu.bytecode.Type calculateType()
   {
     Declaration decl = binding;
-    if (decl == null || decl.isFluid())
+    if (decl == null)
       return Type.pointer_type;
     if (getDontDereference())
       {
