@@ -11,6 +11,21 @@ public class StackTarget extends Target
   Type type;
   public StackTarget(Type type) { this.type = type; }
 
+    /** Target is field or array element that automatically truncates.
+     * Relevant if type is (signed or unsigned) byte or short, and
+     * the given type is an int.  Hence we can avoid the truncation.
+     */
+    protected boolean autoTruncates;
+
+    boolean autoTruncates(Type stackType) {
+        if (autoTruncates && stackType == Type.intType) {
+            char sig1 = type.getSignature().charAt(0);
+            if (sig1 == 'B' || sig1 == 'S' || sig1 == 'C')
+                return true;
+        }
+        return false;
+    }
+
   public Type getType() { return type; }
 
   public static Target getInstance(Type type)
@@ -20,32 +35,46 @@ public class StackTarget extends Target
             : new StackTarget(type));
   }
 
+    public static Target getTruncatingInstance(Type type) {
+        if (type.isVoid())
+            return Target.Ignore;
+        if (type == Type.pointer_type)
+            return Target.pushObject;
+        StackTarget target = new StackTarget(type);
+        target.autoTruncates = true;
+        return target;
+    }
+
     protected StackTarget getClonedInstance(Type type) {
         return new StackTarget(type);
     }
 
     public static void forceLazyIfNeeded(Compilation comp, Type stackType, Type type) {
-	CodeAttr code = comp.getCode();
-	if (LazyType.maybeLazy(stackType) && ! LazyType.maybeLazy(type)) {
-            Type rawType = type.getRawType();
-            if (rawType.compare(stackType.getRawType()) < 0) {
-                Method forceMethod;
-                if (stackType instanceof LazyType) {
-                    forceMethod = LazyType.lazyType.getDeclaredMethod("getValue", 0);
-                } else {
-                    int nargsforce;
-                    if (rawType instanceof ClassType) {
-                        nargsforce = 2;
-                        comp.loadClassRef((ClassType) rawType);
-                    } else
-                        nargsforce = 1;
-                    forceMethod = ClassType.make("gnu.mapping.Promise")
-                        .getDeclaredStaticMethod("force", nargsforce);
-                }
-                code.emitInvoke(forceMethod);
-                stackType = stackType instanceof LazyType ? ((LazyType) stackType).getValueType() : Type.objectType;
+	if (LazyType.maybeLazy(stackType) && ! LazyType.maybeLazy(type))
+            forceLazy(comp, stackType, type);
+    }
+
+    public static Type forceLazy(Compilation comp, Type stackType, Type type) {
+        Type rawType = type.getRawType();
+        if (rawType.compare(stackType.getRawType()) < 0) {
+            Method forceMethod;
+            if (stackType instanceof LazyType) {
+                forceMethod = LazyType.lazyType.getDeclaredMethod("getValue", 0);
+            } else {
+                int nargsforce;
+                if (rawType instanceof ClassType) {
+                    nargsforce = 2;
+                    comp.loadClassRef((ClassType) rawType);
+                } else
+                    nargsforce = 1;
+                forceMethod = ClassType.make("gnu.mapping.Promise")
+                    .getDeclaredStaticMethod("force", nargsforce);
             }
+            CodeAttr code = comp.getCode();
+            code.emitInvoke(forceMethod);
+            stackType = stackType instanceof LazyType ? ((LazyType) stackType).getValueType() : Type.objectType;
         }
+        return stackType;
     }
 
   protected boolean compileFromStack0(Compilation comp, Type stackType)
@@ -67,7 +96,7 @@ public class StackTarget extends Target
       }
     else if (stackType instanceof PrimType && type instanceof PrimType)
       {
-	code.emitConvert(stackType, type);
+          code.emitConvert((PrimType) stackType, (PrimType) type);
 	return true;
       }
 
@@ -118,7 +147,8 @@ public class StackTarget extends Target
             comp.getCode().emitInvokeStatic(wrapMethod);
             comp.getCode().emitCheckcast(ltype.getRawType());
         }
-        else if (! compileFromStack0(comp, stackType))
+        else if (! autoTruncates(stackType)
+                 && ! compileFromStack0(comp, stackType))
             doCoerce(comp);
     }
 
