@@ -60,9 +60,9 @@ public class DivideOp extends ArithOp
       {
 	Object arg2 = args[i];
 	int code2 = Arithmetic.classifyValue(arg2);
-	code = code < code2 ? code2 : code;
+	code = Arithmetic.leastSpecificCode(code, code2);
         int scode = code;
-        if (code < Arithmetic.INTNUM_CODE)
+        if (code < Arithmetic.INTNUM_CODE && code != Arithmetic.UNKNOWN_CODE)
           {
             switch (op)
               {
@@ -71,15 +71,22 @@ public class DivideOp extends ArithOp
                 scode = code = Arithmetic.INTNUM_CODE;
                 break;
               default:
-                if (rounding_mode == Numeric.TRUNCATE
-                    && (code == Arithmetic.INT_CODE || code == Arithmetic.LONG_CODE))
+                  if (rounding_mode == Numeric.TRUNCATE
+                      && code >= Arithmetic.INT_CODE
+                      /* #ifdef JAVA8 */
+                      // && code <= Arithmetic.ULONG_CODE
+                      /* #else */
+                      && code <= Arithmetic.LONG_CODE
+                      /* #endif */
+                      )
                   ; // handled specifically
                 else
                   scode = Arithmetic.INTNUM_CODE;
                 break;
               }
           }
-        if (op == DIVIDE_INEXACT && code <= Arithmetic.REALNUM_CODE)
+        if (op == DIVIDE_INEXACT && code <= Arithmetic.REALNUM_CODE
+                 && code != Arithmetic.UNKNOWN_CODE)
           {
             scode = Arithmetic.REALNUM_CODE;
             if (code != Arithmetic.DOUBLE_CODE && code != Arithmetic.FLOAT_CODE)
@@ -107,9 +114,15 @@ public class DivideOp extends ArithOp
               }
 	    result = Integer.valueOf(i1);
 	    break;
+	  case Arithmetic.UINT_CODE:
 	  case Arithmetic.LONG_CODE:
 	    long l1 = Arithmetic.asLong(result);
 	    long l2 = Arithmetic.asLong(arg2);
+            if (scode == Arithmetic.UINT_CODE)
+              {
+                l1 &= 0xffffffffl;
+                l2 &= 0xffffffffl;
+              }
             switch (op)
               {
               case MODULO:
@@ -119,8 +132,27 @@ public class DivideOp extends ArithOp
                 l1 = l1 / l2;
                 break;
               }
-	    result = Long.valueOf(l1);
+            if (scode == Arithmetic.UINT_CODE)
+                result = UInt.valueOf((int) l1);
+            else
+                result = Long.valueOf(l1);
 	    break;
+          /* #ifdef JAVA8 */
+          // case Arithmetic.ULONG_CODE:
+          //   l1 = Arithmetic.asLong(result);
+          //   l2 = Arithmetic.asLong(arg2);
+          //   switch (op)
+          //     { // FIXME Hacker's Delight has better algorithms than jdk8
+          //     case MODULO:
+          //       l1 = Long.divideUnsigned(l1, l2);
+          //       break;
+          //     default:
+          //       l1 = Long.remainderUnsigned(l1, l2);
+          //       break;
+          //     }
+          //   result = ULong.valueOf(l1);
+          //   break;
+          /* #endif */
 	  case Arithmetic.INTNUM_CODE:
             switch (op)
               {
@@ -194,7 +226,7 @@ public class DivideOp extends ArithOp
               {
               case DIVIDE_GENERIC:
               case DIVIDE_INEXACT:
-                result = DFloNum.make(d1 / d2);
+                result = DFloNum.valueOf(d1 / d2);
                 break;
               case QUOTIENT:
                 result = RealNum.toInt(d1 / d2, getRoundingMode());
@@ -206,7 +238,7 @@ public class DivideOp extends ArithOp
               case MODULO:
                 if (d2 != 0)
                   d1 = d1 - RealNum.toInt(d1 / d2, getRoundingMode()) * d2;
-                result = DFloNum.make(d1);
+                result = DFloNum.valueOf(d1);
                 break;
               }
 	    break;
@@ -242,9 +274,15 @@ public class DivideOp extends ArithOp
               case Arithmetic.INT_CODE:
                 result = Integer.valueOf(result.intValue());
                 break;
+               case Arithmetic.UINT_CODE:
+                result = UInt.valueOf(result.intValue());
+                break;
               case Arithmetic.LONG_CODE:
                 result = Long.valueOf(result.longValue());
                 break;
+              case Arithmetic.ULONG_CODE:
+                result = ULong.valueOf(result.longValue());
+                break;
               case Arithmetic.FLOAT_CODE:
                 result = Float.valueOf(result.floatValue());
                 break;
