@@ -5,7 +5,8 @@ import java.text.Format;
 import java.text.FieldPosition;
 import java.text.ParseException;
 import gnu.math.*;
-import gnu.mapping.OutPort;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.PrettyWriter;
 
 /** A representation of a parsed Common Lisp-style format. */
 
@@ -15,6 +16,10 @@ public class LispFormat extends CompoundFormat
   public static final String paramFromCount = "<from count>";
   public static final String paramUnspecified = "<unspecified>";
 
+    static final DelimitSubstitutionFormat delimitSubstitutionInstance
+        = DelimitSubstitutionFormat
+        .getInstance(ObjectFormat.getInstance(false));
+
   public LispFormat(char[] format, int offset, int length)
     throws ParseException
   {
@@ -162,20 +167,17 @@ public class LispFormat extends CompoundFormat
             fmt = dfmt.resolve(null, 0);
 	    break;
 	  case 'A':  case 'S':  case 'W':
-	  case 'Y':  // SRFI-48 "yuppify" (pretty-print)
-	    // We don't distinguish between ~S and ~W.  FIXME.
-	    fmt = ObjectFormat.getInstance(ch != 'A');
-	    if (numParams > 0)
-	      {
-		minWidth = getParam(stack, speci);
-		int colInc = getParam(stack, speci+1);
-		int minPad = getParam(stack, speci+2);
-		padChar = getParam(stack, speci+3);
-		fmt = new LispObjectFormat((ReportFormat) fmt,
-					   minWidth, colInc, minPad,
-					   padChar, seenAt ? 0 : 100);
-	      }
-	    break;
+	  case 'Y': {// SRFI-48 "yuppify" (pretty-print)
+              // We don't distinguish between ~S and ~W.  FIXME.
+              minWidth = getParam(stack, speci);
+              int colInc = getParam(stack, speci+1);
+              int minPad = getParam(stack, speci+2);
+              padChar = getParam(stack, speci+3);
+              fmt = new LispObjectFormat(ObjectFormat.getInstance(ch != 'A'),
+                                         minWidth, colInc, minPad,
+                                         padChar, seenAt ? 0 : 100);
+              }
+              break;
 	  case 'C':
 	    charVal = numParams > 0 ? getParam(stack, speci)
 	      : PARAM_FROM_LIST;
@@ -428,6 +430,9 @@ public class LispFormat extends CompoundFormat
 	    fmt = LispNewlineFormat.getInstance(count,
 						PrettyWriter.NEWLINE_LITERAL);
 	    break;
+          case 'Q':
+              fmt = delimitSubstitutionInstance;
+              break;
           default:
 	    throw new ParseException("unrecognized format specifier ~"+ch, i);
           }
@@ -669,7 +674,8 @@ class LispNewlineFormat extends ReportFormat
     = System.getProperty("line.separator", "\n");
 
   /** One of NEWLINE_LITERAL, NEWLINE_LINEAR, NEWLINE_FILL, NEWLINE_MISER
-   * or NEWLINE_MANDATORY.  These are defined in gnu.text.PrettyWriter. */
+   * or NEWLINE_MANDATORY.  These are defined in gnu.kawa.io.PrettyWriter.
+   */
   int kind;
 
   int count;
@@ -745,9 +751,9 @@ class LispObjectFormat extends ReportFormat
   int minPad;
   int padChar;
   int where;
-  ReportFormat base;
+  ObjectFormat base;
 
-  public LispObjectFormat(ReportFormat base,
+  public LispObjectFormat(ObjectFormat base,
 			  int minWidth, int colInc, int minPad, int padChar,
 			  int where)
   {
@@ -770,6 +776,21 @@ class LispObjectFormat extends ReportFormat
     if (this.minPad == LispFormat.PARAM_FROM_LIST)  start++;
     char padChar = getParam(this.padChar, ' ', args, start);
     if (this.padChar == LispFormat.PARAM_FROM_LIST)  start++;
+    if (base.readable && dst instanceof OutPort
+        // PadFormat formats to a temporary StringBuffer (i.e. not a
+        // PrettyWriter) so we don't support sharing anyway.
+        // FIXME in ParFormat.
+        && minWidth == 0) {
+        PrettyWriter pdst = ((OutPort) dst).getPrettyWriter();
+        pdst.initialiseIDHash();
+        pdst.setSharing(true);
+        try {
+            return base.format(args, start, dst, fpos);
+        } finally {
+            pdst.setSharing(false);
+            pdst.finishIDHash();
+        }
+    }
     return gnu.text.PadFormat.format(base, args, start, dst,
 				     padChar, minWidth, colInc, minPad,
 				     where, fpos);
@@ -873,7 +894,7 @@ class LispEscapeFormat extends ReportFormat
   public final static int ESCAPE_ALL = 0xF2;
 }
 
-/** Handle <code>~&lt;...~&gt;</code> - pretty-printing logical block.
+/** Handle {@code ~<...~:>} - pretty-printing logical block.
  * (Justification is not implemented.) */
 
 class LispPrettyFormat extends ReportFormat
