@@ -25,36 +25,24 @@ import java.nio.charset.CharsetDecoder;
   * It would be nice if we could inherit from LineNumberReader.
   * That may be possible in theory, but it is difficult and
   * expensive (because we don't get access to BufferedReader's buffer).
-  *
-  * @author Per Bothner <bothner@cygnus.com>
   */
 
 public class InPort extends Reader implements Printable
 {
-    public static boolean noConsole;
-
-    public static boolean haveConsole() {
-        if (noConsole)
-            return false;
-        /* #ifdef JAVA6 */
-        return System.console() != null;
-        /* #else */
-        // return true;
-        /* #endif */
-    }
-
     public static final String systemInFilename = "/dev/stdin";
     private static InPort systemInPort;
     static {
         Path systemInPath = Path.valueOf(systemInFilename);
-        if (haveConsole())
+        if (CheckConsole.haveConsole())
             systemInPort = new TtyInPort(System.in, systemInPath,
                                          OutPort.outInitial);
         else
             systemInPort = new BinaryInPort(System.in, systemInPath);
     }
-  public static final ThreadLocation inLocation
-    = new ThreadLocation("in-default");
+    public static final String evalPathname = "<eval>";
+    public static final String stringPathname = "<string>";
+  public static final ThreadLocation<gnu.kawa.io.InPort> inLocation
+    = new ThreadLocation<gnu.kawa.io.InPort>("in-default");
   static { inLocation.setGlobal(systemInPort); }
 
   static public InPort inDefault ()
@@ -414,7 +402,8 @@ public class InPort extends Reader implements Printable
             if (pos >= limit) {
                 if (buffer == null)
                     buffer = new char[BUFFER_SIZE];
-                else if (limit == buffer.length)
+                else if (limit == buffer.length
+                         && ! (this instanceof CharArrayInPort)) // KLUDGE FIXME
                     reserve(buffer, 1);
                 if (pos == 0) {
                     if (prev == '\r')
