@@ -8,9 +8,13 @@ import gnu.kawa.io.Path;
 import gnu.kawa.io.URIPath;
 import gnu.kawa.functions.Arithmetic;
 import gnu.kawa.functions.LProcess;
+import gnu.kawa.functions.MakeDynamic;
+import gnu.kawa.reflect.CompileBuildObject;
+import gnu.kawa.reflect.CompileInvoke;
 import gnu.kawa.reflect.Invoke;
 import gnu.kawa.reflect.LazyType;
 import gnu.lists.Blob;
+import gnu.lists.Sequences;
 import gnu.lists.U8Vector;
 import java.util.*;
 
@@ -50,6 +54,8 @@ public class LangObjType extends SpecialObjectType implements TypeValue
   private static final int F64VECTOR_TYPE_CODE = 26;
   private static final int PROCEDURE_TYPE_CODE = 27;
   private static final int PROMISE_TYPE_CODE = 28;
+  private static final int SEQUENCE_TYPE_CODE = 29;
+  private static final int DYNAMIC_TYPE_CODE = 30;
 
   public static final LangObjType pathType =
     new LangObjType("path", "gnu.kawa.io.Path",
@@ -162,13 +168,81 @@ public class LangObjType extends SpecialObjectType implements TypeValue
     new LangObjType("promise", "gnu.mapping.Lazy",
                     PROMISE_TYPE_CODE);
 
+  public static final LangObjType sequenceType =
+    new LangObjType("sequence", "java.util.List",
+                    SEQUENCE_TYPE_CODE);
+
+    public static final LangObjType dynamicType =
+    new LangObjType("dynamic", "java.lang.Object",
+                    DYNAMIC_TYPE_CODE);
+
     LangObjType(String name, String implClass, int typeCode) {
         super(name, ClassType.make(implClass));
         this.typeCode = typeCode;
     }
 
+    public static LangObjType getInstanceFromClass(String name) {
+        if ("gnu.math.IntNum".equals(name))
+            return LangObjType.integerType;
+        if ("gnu.math.DFloNum".equals(name))
+            return LangObjType.dflonumType;
+        if ("gnu.math.RatNum".equals(name))
+            return LangObjType.rationalType;
+        if ("gnu.math.RealNum".equals(name))
+            return LangObjType.realType;
+        if ("gnu.math.Numeric".equals(name))
+            return LangObjType.numericType;
+        if ("gnu.lists.FVector".equals(name))
+            return LangObjType.vectorType;
+        if ("gnu.lists.LList".equals(name))
+            return LangObjType.listType;
+        if ("gnu.kawa.io.Path".equals(name))
+            return LangObjType.pathType;
+        if ("gnu.kawa.io.URIPath".equals(name))
+            return LangObjType.URIType;
+        if ("gnu.kawa.io.FilePath".equals(name))
+            return LangObjType.filepathType;
+        if ("java.lang.Class".equals(name))
+            return LangObjType.typeClass;
+        if ("gnu.bytecode.Type".equals(name))
+            return LangObjType.typeType;
+        if ("gnu.bytecode.ClassType".equals(name))
+            return LangObjType.typeClassType;
+        if ("gnu.lists.F64Vector".equals(name))
+            return LangObjType.f64vectorType;
+        if ("gnu.lists.F32Vector".equals(name))
+            return LangObjType.f32vectorType;
+        if ("gnu.lists.S64Vector".equals(name))
+            return LangObjType.s64vectorType;
+        if ("gnu.lists.S32Vector".equals(name))
+            return LangObjType.s32vectorType;
+        if ("gnu.lists.S16Vector".equals(name))
+            return LangObjType.s16vectorType;
+        if ("gnu.lists.S8Vector".equals(name))
+            return LangObjType.s8vectorType;
+        if ("gnu.lists.U64Vector".equals(name))
+            return LangObjType.u64vectorType;
+        if ("gnu.lists.U32Vector".equals(name))
+            return LangObjType.u32vectorType;
+        if ("gnu.lists.U16Vector".equals(name))
+            return LangObjType.u16vectorType;
+        if ("gnu.lists.U8Vector".equals(name))
+            return LangObjType.u8vectorType;
+        return null;
+    }
+
     @Override
     public int isCompatibleWithValue(Type valueType) {
+        switch (typeCode) {
+        case SEQUENCE_TYPE_CODE:
+            if (valueType instanceof ArrayType)
+                return 1;
+            if (stringType.isCompatibleWithValue(valueType) > 0)
+                return 1;
+            break;
+        case DYNAMIC_TYPE_CODE:
+            return 2;
+        }
         return getImplementationType().isCompatibleWithValue(valueType);
     }
 
@@ -202,6 +276,10 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         if (other == typeClassType)
           return 1;
         break;
+      case SEQUENCE_TYPE_CODE:
+          if (other instanceof ArrayType
+              || isCompatibleWithValue(stringType, other) > 0)
+          return 1;
       }
     return getImplementationType().compare(other);
   }
@@ -428,12 +506,17 @@ public class LangObjType extends SpecialObjectType implements TypeValue
       case LIST_TYPE_CODE:
       case REGEX_TYPE_CODE:
         return null;
+      case SEQUENCE_TYPE_CODE:
+        return ClassType.make("gnu.lists.Sequences").getDeclaredMethod("coerceToSequence", 1);
       default:
-        return ((PrimProcedure) getConstructor()).getMethod();
+        Procedure cons = getConstructor();
+        if (cons == null)
+          return null;
+        return ((PrimProcedure) cons).getMethod();
       }
   }
 
-  Method coercionOrNullMethod()
+  protected Method coercionOrNullMethod()
   {
     ClassType methodDeclaringClass = implementationType;
     String mname;
@@ -484,31 +567,42 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         methodDeclaringClass = implementationType;
         mname = "asIntNumOrNull";
         break;
+      case SEQUENCE_TYPE_CODE:
+        methodDeclaringClass = ClassType.make("gnu.lists.Sequences");
+        mname = "asSequenceOrNull";
+        break;
       default:
         return null;
       }
     return methodDeclaringClass.getDeclaredMethod(mname, 1);
   }
 
-  public void emitTestIf(Variable incoming, Declaration decl, Compilation comp)
-  {
-    CodeAttr code = comp.getCode();
-    if (incoming != null)
-      code.emitLoad(incoming);
-    Method method = coercionOrNullMethod();
-    if (method != null)
-      code.emitInvokeStatic(method);
-    if (decl != null)
-      {
-        code.emitDup();
-        decl.compileStore(comp);
-      }
-    if (method != null)
-      code.emitIfNotNull();
-    else
-      {
-        implementationType.emitIsInstance(code);
-        code.emitIfIntNotZero();
+    public boolean emitCoercionOrNull(CodeAttr code) {
+        Method method = coercionOrNullMethod();
+        if (method == null)
+            return false;
+        code.emitInvokeStatic(method);
+        return true;
+    }
+
+    public void emitTestIf(Variable incoming, Declaration decl, Compilation comp) {
+        CodeAttr code = comp.getCode();
+        if (incoming != null)
+            code.emitLoad(incoming);
+        if (emitCoercionOrNull(code)) {
+            if (decl != null) {
+                code.emitDup();
+                decl.compileStore(comp);
+            }
+            code.emitIfNotNull();
+        } else {
+            implementationType.emitIsInstance(code);
+            code.emitIfIntNotZero();
+            if (decl != null) {
+                code.emitLoad(incoming);
+                emitCoerceFromObject(code);
+                decl.compileStore(comp);
+            }
       }
   }
 
@@ -540,6 +634,8 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         return coerceIntNum(obj);
       case DFLONUM_TYPE_CODE:
         return coerceDFloNum(obj);
+      case SEQUENCE_TYPE_CODE:
+          return Sequences.coerceToSequence(obj);
       case VECTOR_TYPE_CODE:
       case CONST_VECTOR_TYPE_CODE:
       case S8VECTOR_TYPE_CODE:
@@ -564,18 +660,17 @@ public class LangObjType extends SpecialObjectType implements TypeValue
   {
     Type argType = null;
     String cname = null;
+    String mname = "make";
     switch (typeCode)
       {
       case DFLONUM_TYPE_CODE:
         if (stackType instanceof PrimType)
           {
-            if (stackType == Type.intType
-                || stackType == Type.byteType
-                || stackType == Type.shortType
-                || stackType == Type.longType
-                || stackType == Type.floatType)
+            char sig1 = stackType.getSignature().charAt(0);
+            if (sig1 == 'I' || sig1 == 'B' || sig1 == 'S' || sig1 == 'J'
+                || sig1 == 'F')
               {
-                code.emitConvert(stackType, Type.doubleType);
+                code.emitConvert((PrimType) stackType, Type.doubleType);
                 stackType = Type.doubleType;
               }
             if (stackType == Type.doubleType)
@@ -593,15 +688,21 @@ public class LangObjType extends SpecialObjectType implements TypeValue
           {
             if (stackType == Type.intType
                 || stackType == Type.byteType
-                || stackType == Type.shortType)
+                || stackType == Type.shortType
+                || stackType == LangPrimType.unsignedByteType
+                || stackType == LangPrimType.unsignedShortType)
               {
                 cname = "gnu.math.IntNum";
                 argType = Type.int_type;
               }
-            else if (stackType == Type.longType)
+            else if (stackType == Type.longType
+                     || stackType == LangPrimType.unsignedIntType
+                     || stackType == LangPrimType.unsignedLongType)
               {
                 cname = "gnu.math.IntNum";
-                argType = Type.long_type;
+                mname = stackType == Type.longType ? "valueOf"
+                    : "valueOfUnsigned";
+                argType = stackType.getImplementationType();
               }
             else if (typeCode == REAL_TYPE_CODE
                      || typeCode == NUMERIC_TYPE_CODE)
@@ -624,7 +725,7 @@ public class LangObjType extends SpecialObjectType implements TypeValue
       {
 	ClassType clas = ClassType.make(cname);
 	Type[] args = { argType };
-	code.emitInvokeStatic(clas.getDeclaredMethod("make", args));
+	code.emitInvokeStatic(clas.getDeclaredMethod(mname, args));
       }
      else
        super.emitConvertFromPrimitive(stackType, code);
@@ -701,23 +802,113 @@ public class LangObjType extends SpecialObjectType implements TypeValue
         return new PrimProcedure("gnu.kawa.io.URIPath", "makeURI", 1);
       case VECTOR_TYPE_CODE:
         return new PrimProcedure("gnu.lists.FVector", "make", 1);
-      case U8VECTOR_TYPE_CODE:
-        return new PrimProcedure("kawa.lib.bytevectors", "$make$bytevector$"+VARARGS_SUFFIX, 1);
       case LIST_TYPE_CODE:
         return gnu.kawa.functions.MakeList.list;
       case STRING_TYPE_CODE:
         return new PrimProcedure("kawa.lib.strings", "$make$string$"+VARARGS_SUFFIX, 1);
       case REGEX_TYPE_CODE:
         return new PrimProcedure("java.util.regex.Pattern", "compile", 1);
+      case DYNAMIC_TYPE_CODE:
+          return MakeDynamic.instance;
       default:
         return null;
       }
   }
 
-    /* #ifndef JAVA8 */
-    public String encodeType(Language language) { return null; }
-    /* #endif */
+    public String encodeType(Language language) {
+        if (this == sequenceType) return "sequence";
+        return null;
+    }
 
-  public static final ClassType typeLangObjType =
-    ClassType.make("gnu.kawa.lispexpr.LangObjType");
+    public Type getElementType() {
+        switch (typeCode) {
+        case S8VECTOR_TYPE_CODE:  return LangPrimType.byteType; 
+        case S16VECTOR_TYPE_CODE: return LangPrimType.shortType; 
+        case S32VECTOR_TYPE_CODE: return LangPrimType.intType; 
+        case S64VECTOR_TYPE_CODE: return LangPrimType.longType; 
+        case U8VECTOR_TYPE_CODE:  return LangPrimType.unsignedByteType; 
+        case U16VECTOR_TYPE_CODE: return LangPrimType.unsignedShortType; 
+        case U32VECTOR_TYPE_CODE: return LangPrimType.unsignedIntType; 
+        case U64VECTOR_TYPE_CODE: return LangPrimType.unsignedLongType; 
+        case F32VECTOR_TYPE_CODE: return LangPrimType.floatType; 
+        case F64VECTOR_TYPE_CODE: return LangPrimType.doubleType; 
+        }
+        return null;
+    }
+
+    public String elementGetterMethodName() {
+        switch (typeCode) {
+        case S8VECTOR_TYPE_CODE:
+        case U8VECTOR_TYPE_CODE:
+            return "byteAt";
+        case S16VECTOR_TYPE_CODE:
+        case U16VECTOR_TYPE_CODE:
+            return "shortAt";
+        case S32VECTOR_TYPE_CODE:
+        case U32VECTOR_TYPE_CODE:
+            return "intAt";
+        case S64VECTOR_TYPE_CODE:
+        case U64VECTOR_TYPE_CODE:
+            return "longAt";
+        case F32VECTOR_TYPE_CODE:
+            return "floatAt";
+        case F64VECTOR_TYPE_CODE:
+            return "doubleAt";
+        }
+        return null;
+    }
+
+    public String elementSetterMethodName() {
+         String gname = elementGetterMethodName();
+         if (gname == null)
+             return null;
+         return "set" + Character.toUpperCase(gname.charAt(0))
+             + gname.substring(1);
+    }
+
+    public static final ClassType typeLangObjType =
+        ClassType.make("gnu.kawa.lispexpr.LangObjType");
+
+    public CompileBuildObject getBuildObject() {
+        switch (typeCode) {
+        case S8VECTOR_TYPE_CODE:
+        case S16VECTOR_TYPE_CODE:
+        case S32VECTOR_TYPE_CODE:
+        case S64VECTOR_TYPE_CODE:
+        case U8VECTOR_TYPE_CODE:
+        case U16VECTOR_TYPE_CODE:
+        case U32VECTOR_TYPE_CODE:
+        case U64VECTOR_TYPE_CODE:
+        case F32VECTOR_TYPE_CODE:
+        case F64VECTOR_TYPE_CODE:
+            return new SimpleVectorBuilder(this);
+        default:
+            return new CompileBuildObject();
+        }
+    }
+
+    public static class SimpleVectorBuilder extends CompileBuildObject {
+        Type elementType;
+        PrimProcedure addProc;
+        public SimpleVectorBuilder(LangObjType vectorType) {
+            this.elementType = vectorType.getElementType();
+            Method addMethod = ((ClassType) vectorType.getImplementationType())
+                .getMethod("add",
+                           new Type[] { elementType.getImplementationType() });
+            addProc = new PrimProcedure(addMethod, Type.voidType,
+                                        new Type[] {elementType});
+        }
+
+        @Override
+        public boolean useBuilder(int numCode, InlineCalls visitor) {
+            return (numKeywordArgs() == 0 && hasDefaultConstructor())
+                || super.useBuilder(numCode, visitor);
+        }
+
+        @Override
+        public Expression buildAddChild(Declaration target, Expression child) {
+            return new ApplyExp(addProc, new ReferenceExp(target),
+                                child);
+        }
+    }
 }
