@@ -7,7 +7,10 @@ import gnu.expr.*;
 import gnu.kawa.io.BinaryInPort;
 import gnu.kawa.io.InPort;
 import gnu.kawa.util.GeneralHashTable;
+import gnu.bytecode.PrimType;
+import gnu.bytecode.Type;
 import java.util.regex.*;
+import java.lang.reflect.Array;
 
 /** A Lexer for reading S-expressions in generic Lisp-like syntax.
  * This class may have outlived its usefulness: It's mostly just a
@@ -187,13 +190,20 @@ public class LispReader extends Lexer
   protected Object readAndHandleToken(int ch, int startPos, ReadTable rtable)
     throws java.io.IOException, SyntaxException
   {
-    char readCase = getReadCase();
     readToken(ch, rtable);
+    return handleToken(startPos, rtable);
+  }
+
+  protected Object handleToken(int startPos, ReadTable rtable)
+    throws java.io.IOException, SyntaxException
+  {
+    int ch;
+    char readCase = getReadCase();
     int endPos = tokenBufferLength;
     if (! seenEscapes)
       {
         Object value = parseNumber(tokenBuffer, startPos, endPos - startPos,
-                                   '\0', 0, SCM_NUMBERS);
+                                   '\0', 0, SCM_NUMBERS|rtable.extraFlags);
         if (value != null && ! (value instanceof String))
           {
             tokenBufferLength = startPos;
@@ -503,7 +513,8 @@ public class LispReader extends Lexer
 
     /** After reading a value check for following {@code '['} or {@code ':'}.
      */
-    Object handlePostfix (Object value, ReadTable rtable, int line, int column)
+    protected Object handlePostfix(Object value, ReadTable rtable,
+                                   int line, int column)
         throws java.io.IOException, SyntaxException {
         if (value == QuoteExp.voidExp)
             value = Values.empty;
@@ -578,22 +589,22 @@ public class LispReader extends Lexer
   public static final int SCM_NUMBERS = SCM_COMPLEX;
   public static final int SCM_ANGLE = SCM_NUMBERS << 1;
   public static final int SCM_COLATITUDE = SCM_ANGLE << 1;
+  public static final int SCM_LEXPONENT_IS_BIGDECIMAL = SCM_COLATITUDE << 1;
 
-  public static Object parseNumber
-  /* #ifdef use:java.lang.CharSequence */
-  (CharSequence str, int radix)
-  /* #else */
-  // (CharSeq str, int radix)
-  /* #endif */
-  {
-    char[] buf;
-    if (str instanceof FString)
-      buf = ((FString) str).data;
-    else
-      buf = str.toString().toCharArray();
-    int len = str.length();
-    return parseNumber(buf, 0, len, '\0', radix, LispReader.SCM_NUMBERS);
-  }
+    public static Object parseNumber(CharSequence str, int radix) {
+        char[] buf;
+        int len = str.length();
+        int where;
+        if (str instanceof FString
+            && (where = ((FString) str).getSegmentReadOnly(0, len)) >= 0) {
+            buf = ((FString) str).getBuffer();
+        } else {
+            where = 0;
+            buf = str.toString().toCharArray();
+        }
+        return parseNumber(buf, where, len,
+                           '\0', radix, LispReader.SCM_NUMBERS);
+    }
 
   /** Parse a number.
    * @param buffer contains the characters of the number
@@ -726,7 +737,7 @@ public class LispReader extends Lexer
             return "no digits";
         if (pos < end) {
             Object jmag = parseNumber(buffer, pos, end-pos, exactness,
-                                      10, flags);
+                                      radix, flags);
             if (jmag instanceof String)
                 return jmag;
             if (! (jmag instanceof Quaternion))
@@ -757,7 +768,7 @@ public class LispReader extends Lexer
             return "no digits";
         if (pos < end) {
             Object kmag = parseNumber(buffer, pos, end-pos, exactness,
-                                      10, flags);
+                                      radix, flags);
             if (kmag instanceof String)
                 return kmag;
             if (! (kmag instanceof Quaternion))
@@ -942,18 +953,48 @@ public class LispReader extends Lexer
 	  digits_start = decimal_point;
 	if (numerator != null)
 	  return "floating-point number after fraction symbol '/'";
-	String str = new String(buffer, digits_start, pos - digits_start);
-        if (exp_seen >= 0)
-          {
-            exp_char = Character.toLowerCase(buffer[exp_seen]);
-            if (exp_char != 'e')
-              {
-                int prefix = exp_seen - digits_start;
-                str = str.substring(0, prefix)+'e'+str.substring(prefix+1);
-              }
-          }
-	double d = Convert.parseDouble(str);
-	number = new DFloNum(negative ? - d : d);
+        if (exactness == 'e' || exactness == 'E') {
+            int exp = 0;
+            IntNum inumber;
+            if (decimal_point < 0) {
+                inumber = valueOf(buffer, digits_start,
+                                  exp_seen - digits_start,
+                                  radix, negative, lvalue);
+            }
+            else {
+                StringBuilder sbuf = new StringBuilder();
+                if (negative)
+                    sbuf.append('-');
+                sbuf.append(buffer, digits_start, decimal_point-digits_start);
+                decimal_point++;
+                int fracdigits = (exp_seen >= 0 ? exp_seen : pos)
+                    - decimal_point;
+                sbuf.append(buffer, decimal_point, fracdigits);
+                inumber = IntNum.valueOf(sbuf.toString());
+                exp -= fracdigits;
+            }
+            if (exp_seen >= 0) {
+                exp += Integer.parseInt(new String(buffer, exp_seen+1,
+                                                   pos - (exp_seen+1)));
+            }
+            if (exp > 0)
+                number = IntNum.times(inumber, IntNum.power(IntNum.ten(), exp));
+            else if (exp < 0)
+                number = RatNum.make(inumber, IntNum.power(IntNum.ten(), -exp));
+            else
+                number = inumber;
+        } else {
+            String str = new String(buffer, digits_start, pos - digits_start);
+            if (exp_seen >= 0) {
+                exp_char = Character.toLowerCase(buffer[exp_seen]);
+                if (exp_char != 'e') {
+                    int prefix = exp_seen - digits_start;
+                    str = str.substring(0, prefix)+'e'+str.substring(prefix+1);
+                }
+            }
+            double d = Convert.parseDouble(str);
+            number = new DFloNum(negative ? - d : d);
+        }
       }
     else
       {
@@ -998,7 +1039,7 @@ public class LispReader extends Lexer
 	if (ch == '@')
 	  { /* polar notation */
 	    Object angle = parseNumber(buffer, pos, end - pos,
-				       exactness, 10, flags|SCM_ANGLE);
+				       exactness, radix, flags|SCM_ANGLE);
 	    if (angle instanceof String)
 	      return angle;
 	    if (! (angle instanceof RealNum) && ! (angle instanceof RealNum[]))
@@ -1023,7 +1064,7 @@ public class LispReader extends Lexer
         if (ch == '%') {
             /* extended polar notation */
             Object colatitude = parseNumber(buffer, pos, end - pos,
-                                            exactness, 10,
+                                            exactness, radix,
                                             flags|SCM_COLATITUDE);
             if (colatitude instanceof String)
                 return colatitude;
@@ -1061,7 +1102,7 @@ public class LispReader extends Lexer
         if (ch == '&') {
             /* extended polar notation */
             Object longitude = parseNumber(buffer, pos, end - pos,
-                                           exactness, 10, flags);
+                                           exactness, radix, flags);
             if (longitude instanceof String)
                 return longitude;
             if (! (longitude instanceof RealNum))
@@ -1086,7 +1127,7 @@ public class LispReader extends Lexer
 	  {
 	    pos--;
 	    Object imag = parseNumber(buffer, pos, end - pos,
-				      exactness, 10, flags);
+				      exactness, radix, flags);
 	    if (imag instanceof String)
 	      return imag;
 	    if (! (imag instanceof Quaternion))
@@ -1117,7 +1158,7 @@ public class LispReader extends Lexer
             if (prev == 'i' || prev == 'I') {
                 if (pos < end) {
                     Object jmag = parseNumber(buffer, pos, end-pos,
-                                              exactness, 10, flags);
+                                              exactness, radix, flags);
                     if (jmag instanceof String)
                         return jmag;
                     if (! (jmag instanceof Quaternion))
@@ -1135,7 +1176,7 @@ public class LispReader extends Lexer
             if (prev == 'j' || prev == 'J') {
                 if (pos < end) {
                     Object kmag = parseNumber(buffer, pos, end-pos,
-                                              exactness, 10, flags);
+                                              exactness, radix, flags);
                     if (kmag instanceof String)
                         return kmag;
                     if (! (kmag instanceof Quaternion))
@@ -1171,7 +1212,9 @@ public class LispReader extends Lexer
           case 'd':
             return Double.valueOf(d);
           case 'l':
-            return java.math.BigDecimal.valueOf(d);
+            if ((flags & SCM_LEXPONENT_IS_BIGDECIMAL) != 0)
+              return java.math.BigDecimal.valueOf(d);
+            // else fall through
           }
       }
     return number;
@@ -1606,6 +1649,45 @@ public class LispReader extends Lexer
     return readSimpleVector(reader, kind, ch, size);
   }
 
+    private Object convertListToArray(Object list, int len, PrimType elementType) {
+        char sig1 = elementType.getSignature().charAt(0);
+        Object array = Array.newInstance(elementType.getReflectClass(), len);
+        for (int i = 0;  i < len; i++) {
+            Pair pair = (Pair) list;
+            Object value = pair.getCar();
+            if (sig1 == 'B' || sig1 == 'S' || sig1 == 'I' || sig1 == 'J') {
+                String msg = null;
+                if (! (value instanceof IntNum))
+                    msg = "expected integer value";
+                else {
+                    value = LangPrimType.convertIntegerLiteral((IntNum) value, elementType, true);
+                    if (value == null)
+                        msg = "integer "+pair.getCar()+" not in range of "+elementType.getName();
+                }
+                if (msg != null) {
+                    getMessages().error('e', (SourceLocator) pair, msg);
+                    value = LangPrimType.convertIntegerLiteral(IntNum.zero(),
+                                                               elementType, true);
+                }
+            }
+            if (sig1 == 'F' || sig1 == 'D') {
+                RealNum rvalue = RealNum.asRealNumOrNull(value);
+                if (rvalue != null) {
+                    if (sig1 == 'F')
+                        value = Float.valueOf(rvalue.floatValue());
+                    else
+                        value = Double.valueOf(rvalue.doubleValue());
+                } else {
+                    getMessages().error('e', (SourceLocator) pair,
+                                        "expected real value");
+                }
+            }
+            Array.set(array, i, value);
+            list = pair.getCdr();
+        }
+        return array;
+    }
+
   public static SimpleVector
       readSimpleVector(LispReader reader, char kind, int ch, int size)
     throws java.io.IOException, SyntaxException
@@ -1630,24 +1712,54 @@ public class LispReader extends Lexer
       case 'F':
         switch (size)
           {
-          case 32:  return new F32Vector(q);
-          case 64:  return new F64Vector(q);
+          case 32:
+              return new F32Vector((float[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.floatType));
+          case 64:
+              return new F64Vector((double[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.doubleType));
           }
       case 'S':
         switch (size)
           {
-          case  8:  return new S8Vector(q);
-          case 16:  return new S16Vector(q);
-          case 32:  return new S32Vector(q);
-          case 64:  return new S64Vector(q);
-          }
+         case  8:
+              return new S8Vector((byte[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.byteType));
+          case 16:
+              return new S16Vector((short[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.shortType));
+          case 32:
+              return new S32Vector((int[])
+                                   reader.convertListToArray(list, len,
+                                           LangPrimType.intType));
+          case 64:
+               return new S64Vector((long[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.longType));
+         }
       case 'U':
         switch (size)
           {
-          case  8:  return new U8Vector(q);
-          case 16:  return new U16Vector(q);
-          case 32:  return new U32Vector(q);
-          case 64:  return new U64Vector(q);
+         case  8:
+              return new U8Vector((byte[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.unsignedByteType));
+          case 16:
+              return new U16Vector((short[])
+                                  reader.convertListToArray(list, len,
+                                          LangPrimType.unsignedShortType));
+          case 32:
+              return new U32Vector((int[])
+                                   reader.convertListToArray(list, len,
+                                           LangPrimType.unsignedIntType));
+          case 64:
+               return new U64Vector((long[])
+                                 reader.convertListToArray(list, len,
+                                          LangPrimType.unsignedLongType));
           }
       }
     return null;
