@@ -2,6 +2,8 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.lists;
+
+import gnu.kawa.util.HashUtils;
 import java.util.*;
 
 /**
@@ -73,12 +75,7 @@ public abstract class AbstractSequence<E>
 
   public static RuntimeException unsupportedException (String text)
   {
-    /* #ifdef JAVA2 */
     return new UnsupportedOperationException(text);
-    /* #endif */
-    /* #ifndef JAVA2 */
-    // return new RuntimeException(text);
-    /* #endif */
   }
 
   public E set(int index, E element)
@@ -86,6 +83,10 @@ public abstract class AbstractSequence<E>
     throw unsupported("set");
   }
 
+    public void setAt(int index, E element) {
+        throw unsupported("setAt");
+    }
+
   public void fill(E value)
   {
     for (int i = startPos(); (i = nextPos(i)) != 0; )
@@ -171,7 +172,6 @@ public abstract class AbstractSequence<E>
     return indexOf(o) >= 0;
   }
 
-  /* #ifdef JAVA2 */
   /** See java.util.List. */
   public boolean containsAll(Collection<?> c)
   {
@@ -184,7 +184,6 @@ public abstract class AbstractSequence<E>
       }
     return true;
   }
-  /* #endif */
 
   public final Enumeration<E> elements()
   {
@@ -206,7 +205,6 @@ public abstract class AbstractSequence<E>
     return new SeqPosition<E,AbstractSequence<E>>(this, copyPos(ipos));
   }
 
-  /* #ifdef JAVA2 */
   public final Iterator<E> iterator()
   {
     return getIterator();
@@ -221,7 +219,6 @@ public abstract class AbstractSequence<E>
   {
     return getIterator(index);
   }
-  /* #endif */
 
   /** Add a value at a specified Pos.
    * @return the updated Pos, which is after the inserted value..
@@ -246,7 +243,6 @@ public abstract class AbstractSequence<E>
     releasePos(pos);
   }
 
-  /* #ifdef JAVA2 */
   /** See java.util.Collection. */
   public boolean addAll(Collection<? extends E> c)
   {
@@ -266,26 +262,6 @@ public abstract class AbstractSequence<E>
     releasePos(pos);
     return changed;
   }
-  /* #endif */
-  /* #ifndef JAVA2 */
-  // public boolean addAll(Sequence c)
-  // {
-  //   return addAll(size(), c);
-  // }
-
-  // public boolean addAll(int index, Sequence c)
-  // {
-  //   boolean changed = false;
-  //   int pos = createPos(index, false);
-  //   for (int iter = startPos();  (iter = nextPos(iter)) != 0; )
-  //     {
-  //       pos = addPos(pos, getPosPrevious(iter));
-  //       changed = true;
-  //     }
-  //   releasePos(pos);
-  //   return changed;
-  // }
-  /* #endif */
 
   /**
    * Remove one or more elements.
@@ -294,9 +270,9 @@ public abstract class AbstractSequence<E>
    * following (poses, posNumber); if negative the negative of the number
    * of elements to remove before (poses, posNumber).
    * @exception java.lang.IndexOutOfBoundsException
-   *   if (count >= 0 ? (index < 0 || index + count > size())
-   *       : (index + count < 0 || index > size())),
-   *   where index == nextIndex(ipos, xpos).
+   *   if {@code (count >= 0 ? (index < 0 || index + count > size())
+   *       : (index + count < 0 || index > size()))},
+   *   where {@code index == nextIndex(ipos, xpos)}.
    */
   public void removePos(int ipos, int count)
   {
@@ -312,8 +288,8 @@ public abstract class AbstractSequence<E>
    * @param ipos0 start of range, as a poistion
    * @param ipos1 end of range
    * @exception java.lang.IndexOutOfBoundsException
-   *   if nextIndex(ipos0) > nextIndex(ipos1)
-   *   || nextIndex(ipos0) < 0 || nextIndex(ipos1) > size()
+   *   if {@code nextIndex(ipos0) > nextIndex(ipos1)}
+   *   or {@code nextIndex(ipos0) < 0} or {@code nextIndex(ipos1) > size()}.
    */
   protected void removePosRange(int ipos0, int ipos1)
   {
@@ -342,7 +318,6 @@ public abstract class AbstractSequence<E>
     return true;
   }
 
-  /* #ifdef JAVA2 */
   public boolean removeAll(Collection<?> c)
   {
     boolean changed = false;
@@ -372,7 +347,6 @@ public abstract class AbstractSequence<E>
       }
     return changed;
   }
-  /* #endif */
 
   public void clear()
   {
@@ -437,7 +411,9 @@ public abstract class AbstractSequence<E>
   /**
    * Get the offset from the beginning corresponding to a position cookie.
    */
-  protected abstract int nextIndex(int ipos);
+  protected int nextIndex(int ipos) {
+    return ipos == -1 ? size() : ipos >>> 1;
+  }
 
   protected int fromEndIndex(int ipos)
   {
@@ -714,46 +690,38 @@ public abstract class AbstractSequence<E>
     return hash;
   }
 
+    public int boundedHash(int seed, int limit) {
+        int count = 0;
+        int sublimit = limit >> 1;
+        for (int i = startPos(); (i = nextPos(i)) != 0;  )  {
+            if (++count > limit)
+                break;
+            int h = HashUtils.boundedHash(getPosPrevious(i), 0, sublimit);
+            seed = HashUtils.murmur3step(seed, h);
+        }
+        return HashUtils.murmur3finish(seed, count);
+   }
+
   public boolean equals(Object o)
   {
     // Compatible with the Collections specification.
     // FIXME should also depend on class?
-    /* #ifdef JAVA2 */
     if (! (this instanceof java.util.List)
         || ! (o instanceof java.util.List))
       return this == o;
     Iterator<E> it1 = iterator();
     Iterator<E> it2 = ((java.util.List<E>) o).iterator();
-    /* #endif */
-    /* #ifndef JAVA2 */
-    // if (! (this instanceof Sequence) || ! (o instanceof Sequence))
-    //   return this == o;
-    // Enumeration it1 = elements();
-    // Enumeration it2 = ((Sequence) o).elements();
-    /* #endif */
     for (;;)
       {
-        /* #ifdef JAVA2 */
         boolean more1 = it1.hasNext();
         boolean more2 = it2.hasNext();
-        /* #endif */
-        /* #ifndef JAVA2 */
-        // boolean more1 = it1.hasMoreElements();
-        // boolean more2 = it2.hasMoreElements();
-        /* #endif */
         if (more1 != more2)
           return false;
         if (! more1)
           return true;
-	/* #ifdef JAVA2 */
         E e1 = it1.next();
         E e2 = it2.next();
-	/* #endif */
-	/* #ifndef JAVA2 */
-        // Object e1 = it1.nextElement();
-        // Object e2 = it2.nextElement();
-	/* #endif */
-        if (e1 == null)
+	if (e1 == null)
           {
             if (e2 != null)
               return false;
@@ -773,13 +741,11 @@ public abstract class AbstractSequence<E>
     return new SubSequence<E>(this, ipos0, ipos1);
   }
 
-  /* #ifdef JAVA2 */
   public List<E> subList(int fromIx, int toIx)
   {
     return subSequencePos(createPos(fromIx, false),
                           createPos(toIx, true));
   }
-  /* #endif */
 
   /** Copy an element specified by a position pair to a Consumer.
    * @return if hasNext(ipos). */
