@@ -2,12 +2,38 @@
 // This is free software;  for terms and warranty disclaimer see ./COPYING.
 
 package gnu.lists;
+
+import gnu.text.Char;
 import java.io.IOException;
 
 /** Various static utility methods for general strings (CharSeqs). */
 
 public class Strings
 {
+    public static int characterAt(CharSequence cseq, int index) {
+        return characterAt(cseq, 0, cseq.length(), index);
+    }
+    public static int characterAt(CharSequence cseq, int start, int end,
+                                  int index) {
+        if (index < start || index >= end)
+            throw new IndexOutOfBoundsException();
+        char ch1 = cseq.charAt(index);
+        if (ch1 >= 0xD800 && ch1 <= 0xDBFF) {
+            if (index + 1 < end) {
+                char ch2 = cseq.charAt(index+1);
+                if (ch2 >= 0xDC00 && ch2 <= 0xDFFF)
+                    return ((ch1 - 0xD800) << 10) + (ch2 - 0xDC00) + 0x10000;
+            }
+        } else if (ch1 >= 0xDC00 && ch1 <= 0xDFFF) {
+            if (index > start) {
+                char ch0 = cseq.charAt(index-1);
+                if (ch0 >= 0xD800 && ch0 <= 0xDBFF)
+                    return Char.IGNORABLE_CHAR;
+            }
+        }
+        return ch1;
+    }
+
     public static int sizeInCodePoints(CharSequence str) {
         int len = str.length();
         int nsurr = 0;
@@ -125,4 +151,43 @@ public class Strings
             }
         }
     }
+
+    public static CharSequence indirectIndexed(CharSequence base,
+                                               IntSequence indexes) {
+        if (base instanceof FString) {
+            return (FString) ((FString) base).select(indexes);
+        }
+        if (indexes instanceof Range.IntRange) {
+            Range.IntRange range = (Range.IntRange) indexes;
+            if (range.getStepInt() == 1) {
+                int start = range.getStartInt();
+                int bsize = base.length();
+                if (start < 0 || start > bsize)
+                    throw new IndexOutOfBoundsException();
+                if (range.isUnbounded())
+                    return SubCharSeq.valueOf(base, start, bsize);
+                int size = range.size();
+                if (start+size < 0 || start+size > bsize)
+                    throw new IndexOutOfBoundsException();
+                return SubCharSeq.valueOf(base, start, start+size);
+            }
+        }
+        int len = indexes.size();
+        StringBuilder sbuf = new StringBuilder(len);
+        for (int i = 0; i < len; i++)
+            sbuf.append(base.charAt(indexes.intAt(i)));
+        return sbuf.toString();
+    }
+
+    public static String toUtf8(byte[] bytes, int start, int length) {
+        /* #ifdef JAVA7 */  
+        return new String(bytes, start, length, java.nio.charset.StandardCharsets.UTF_8);
+        /* #else */
+        // try {
+        //   return new String(bytes, start, length, "UTF-8");
+        // } catch (java.io.UnsupportedEncodingException ex) {
+        //     throw new RuntimeException(ex);
+        // }
+        /* #endif */
+    }
 }
