@@ -262,7 +262,7 @@ public class TreeList extends AbstractSequence<Object>
    *   to the outer BEGIN_ELEMENT_SHORT/BEGIN_ELEMENT_LONG/BEGIN_DOCUMENT.
    *.  (If these is no parent, then parent_offset==0.)
    *
-   * This should is used when index < BEGIN_ELEMENT_SHORT_INDEX_MAX,
+   * This should is used when {@code index < BEGIN_ELEMENT_SHORT_INDEX_MAX},
    * both end_offset and parent_offset fit in 16 bits,
    * and the element does not straddle the gap.
    */
@@ -417,7 +417,8 @@ public class TreeList extends AbstractSequence<Object>
 
   public void writeObject(Object v)
   {
-    ensureSpace(3);
+    if (gapEnd - gapStart < 3)
+      ensureSpace(3);
     int index = find(v);
     if (index < 0x1000)
       data[gapStart++] = (char) (OBJECT_REF_SHORT | index);
@@ -1565,6 +1566,7 @@ public class TreeList extends AbstractSequence<Object>
       case DOUBLE_FOLLOWS:
 	return Sequence.DOUBLE_VALUE;
       case CHAR_FOLLOWS:
+        return Sequence.CHAR_VALUE;
       case BEGIN_DOCUMENT:
 	return Sequence.DOCUMENT_VALUE;
       case BEGIN_ENTITY:
