@@ -179,9 +179,9 @@ public class SimpleEnvironment extends Environment
   // FIXME rename to define
   NamedLocation addLocation (Symbol name, Object property, int hash, Location loc)
   {
-    if (loc instanceof ThreadLocation
-        && ((ThreadLocation) loc).property == property)
-      loc = ((ThreadLocation) loc).getLocation();
+    if (loc instanceof DynamicLocation
+        && ((DynamicLocation) loc).property == property)
+      loc = ((DynamicLocation) loc).getLocation();
     NamedLocation nloc = lookupDirect(name, property, hash);
     if (loc == nloc)
       return nloc;
