@@ -38,11 +38,19 @@ public class DFloNum extends RealNum implements Externalizable
       value = -0.0;
   }
 
+  @Deprecated
   public static DFloNum make (double value)
   {
     return new DFloNum (value);
   }
 
+    public static DFloNum valueOf(double value) {
+        return new DFloNum(value);
+    }
+
+    private static final DFloNum zero = new DFloNum(0.0);
+    public static DFloNum zero() { return zero; }
+
   public static DFloNum asDFloNumOrNull (Object value)
   {
     if (value instanceof DFloNum)
