@@ -42,15 +42,6 @@ public class IntNum extends RatNum implements Externalizable
     ival = value;
   }
 
-  /** Return a (possibly-shared) IntNum with a given int value. */
-  public static IntNum make (int value)
-  {
-    if (value >= minFixNum && value <= maxFixNum)
-      return smallFixNums[(int) value - minFixNum];
-    else
-      return new IntNum (value);
-  }
-
   public static final IntNum zero ()
   {
     return smallFixNums[- minFixNum];
@@ -72,25 +63,12 @@ public class IntNum extends RatNum implements Externalizable
     return smallFixNums[-1 - minFixNum];
   }
 
-  /** Return a (possibly-shared) IntNum with a given long value. */
-  public static IntNum make (long value)
-  {
-    if (value >= minFixNum && value <= maxFixNum)
-      return smallFixNums[(int)value - minFixNum];
-    int i = (int) value;
-    if ((long)i == value)
-      return new IntNum (i);
-    IntNum result = alloc (2);
-    result.ival = 2;
-    result.words[0] = i;
-    result.words[1] = (int) (value >> 32);
-    return result;
-  }
-
   public static IntNum asIntNumOrNull (Object value)
   {
     if (value instanceof IntNum)
       return (IntNum) value;
+    if (value instanceof UnsignedPrim)
+      return ((UnsignedPrim) value).toIntNum();
     if (value instanceof BigInteger)
       return IntNum.valueOf(value.toString(), 10);
     if (value instanceof Number
@@ -100,39 +78,6 @@ public class IntNum extends RatNum implements Externalizable
     return null;
   }
 
-  /** Make an IntNum from an unsigned 64-bit value. */
-  public static IntNum makeU (long value)
-  {
-    if (value >= 0)
-      return make(value);
-    IntNum result = alloc(3);
-    result.ival = 3;
-    result.words[0] = (int) value;
-    result.words[1] = (int) (value >> 32);
-    result.words[2] = 0;
-    return result;
-  }
-
-  /** Make a canonicalized IntNum from an array of words.
-   * The array may be reused (without copying). */
-  public static IntNum make (int[] words, int len)
-  {
-    if (words == null)
-      return make (len);
-    len = IntNum.wordsNeeded (words, len);
-    if (len <= 1)
-      return len == 0 ? zero () : make (words[0]);
-    IntNum num = new IntNum ();
-    num.words = words;
-    num.ival = len;
-    return num;
-  }
-
-  public static IntNum make (int[] words)
-  {
-    return make(words, words.length);
-  }
-
   /** Allocate a new non-shared IntNum.
    * @param nwords number of words to allocate
    */
@@ -281,8 +226,9 @@ public class IntNum extends RatNum implements Externalizable
   }
 
   /** Calculate how many words are significant in words[0:len-1].
-   * Returns the least value x such that x>0 && words[0:x-1]==words[0:len-1],
-   * when words is viewed as a 2's complement integer.
+   * Returns the least value {@code x} such that
+   * {@code x>0 && words[0:x-1]==words[0:len-1]},
+   * when {@code words} is viewed as a 2's complement integer.
    */
   public static int wordsNeeded (int[] words, int len)
   {
@@ -1105,30 +1051,6 @@ public class IntNum extends RatNum implements Externalizable
     return result.canonicalize ();
   }
 
-  public static int shift (int x, int count)
-  {
-    if (count >= 32)
-      return 0;
-    if (count >= 0)
-      return x << count;
-    count = -count;
-    if (count >= 32)
-        return x < 0 ? -1 : 0;
-    return x >> count;
-  }
-
-  public static long shift (long x, int count)
-  {
-    if (count >= 64)
-      return 0;
-    if (count >= 0)
-      return x << count;
-    count = -count;
-    if (count >= 64)
-        return x < 0 ? -1 : 0;
-    return x >> count;
-  }
-
   /* #ifdef JAVA5 */
   public void format (int radix, StringBuffer buffer)
   {
@@ -1318,6 +1240,73 @@ public class IntNum extends RatNum implements Externalizable
     return IntNum.equals (this, (IntNum) obj);
   }
 
+    /** Return a (possibly-shared) IntNum with a given int value. */
+    public static IntNum make (int value) { return valueOf(value); }
+
+    /** Make an IntNum from an unsigned 64-bit value. */
+    public static IntNum valueOfUnsigned(long value) {
+        if (value >= 0)
+            return valueOf(value);
+        IntNum result = alloc(3);
+        result.ival = 3;
+        result.words[0] = (int) value;
+        result.words[1] = (int) (value >> 32);
+        result.words[2] = 0;
+        return result;
+    }
+
+    /** Make an IntNum from an unsigned 32-bit value. */
+    public static IntNum valueOfUnsigned(int value) {
+        if (value >= 0)
+            return valueOf(value);
+        IntNum result = alloc(2);
+        result.ival = 2;
+        result.words[0] = value;
+        result.words[1] = 0;
+        return result;
+    }
+
+  /** Make a canonicalized IntNum from an array of words.
+   * The array may be reused (without copying). */
+  public static IntNum make (int[] words, int len)
+  {
+    if (words == null)
+      return make (len);
+    len = IntNum.wordsNeeded (words, len);
+    if (len <= 1)
+      return len == 0 ? zero () : make (words[0]);
+    IntNum num = new IntNum ();
+    num.words = words;
+    num.ival = len;
+    return num;
+  }
+
+    public static IntNum make(long value) {
+        return valueOf(value);
+    }
+
+    /** Return a (possibly-shared) IntNum with a given int value. */
+    public static IntNum valueOf(int value) {
+        if (value >= minFixNum && value <= maxFixNum)
+            return smallFixNums[(int) value - minFixNum];
+        else
+            return new IntNum(value);
+    }
+
+    /** Return a (possibly-shared) IntNum with a given long value. */
+    public static IntNum valueOf(long value) {
+        if (value >= minFixNum && value <= maxFixNum)
+            return smallFixNums[(int)value - minFixNum];
+        int i = (int) value;
+        if ((long)i == value)
+            return new IntNum (i);
+        IntNum result = alloc (2);
+        result.ival = 2;
+        result.words[0] = i;
+        result.words[1] = (int) (value >> 32);
+        return result;
+    }
+
   public static IntNum valueOf (char[] buf, int offset, int length,
 				int radix, boolean negative)
   {
@@ -1541,7 +1530,7 @@ public class IntNum extends RatNum implements Externalizable
   }
 
   /** Copy the abolute value of this into an array of words.
-   * Assumes words.length >= (this.words == null ? 1 : this.ival).
+   * Assumes {@code words.length >= (this.words == null ? 1 : this.ival)}.
    * Result is zero-extended, but need not be a valid 2's complement number.
    */
     
