@@ -63,9 +63,10 @@ public class Q2 extends Scheme
     defSntxStFld("<", "gnu.q2.lang.Operator", "LT");
     defSntxStFld(">", "gnu.q2.lang.Operator", "GT");
     defSntxStFld("==", "gnu.q2.lang.Operator", "EQ");
-    defSntxStFld("=<", "gnu.q2.lang.Operator", "LE");
+    defSntxStFld("<=", "gnu.q2.lang.Operator", "LE");
     defSntxStFld(">=", "gnu.q2.lang.Operator", "GE");
     defSntxStFld(":=", "gnu.q2.lang.Operator", "ASSIGN");
+    defSntxStFld("?>", "gnu.q2.lang.Operator", "IF_THEN");
   }
 
   public String getName()
@@ -75,7 +76,7 @@ public class Q2 extends Scheme
 
   public LispReader getLexer(InPort inp, gnu.text.SourceMessages messages)
   {
-    Compilation.defaultCallConvention = Compilation.CALL_WITH_CONSUMER;
+    //Compilation.defaultCallConvention = Compilation.CALL_WITH_CONSUMER;
     Q2Read lexer = new Q2Read(inp, messages);
     return lexer;
   }
@@ -114,11 +115,14 @@ public class Q2 extends Scheme
     //Also want to allow #!/ at start of file as comment start.
     //rdispatch.set('!', ReaderIgnoreRestOfLine.getInstance());
     rt.setFinalColonIsKeyword(true);
+    rt.postfixLookupOperator = ':';
+    rt.set('@', new ReaderQuote(LispLanguage.splice_sym,
+                                ReadTable.NON_TERMINATING_MACRO));
     return rt;
   }
 
   /** Compare two indentation amounts.
-   * An indentation is @{code (numberOfTabs<<16)+numberOfSpaces}.
+   * An indentation is {@code (numberOfTabs<<16)+numberOfSpaces}.
    * A comparison is indeterminate if it depends on tab-width - e.g.
    * the number of tabs in indentation1 is less than indentation2,
    * but it's the reverse when it comes to spaces.
