@@ -26,6 +26,8 @@ public class Char implements Comparable, Externalizable {
     // Leave open the possibility for characters beyond Unicode.
     int value;
 
+    public static final int IGNORABLE_CHAR = 0x1FFFFF;
+
     /** Should only be used for serialization. */
     public Char() {
     }
@@ -67,6 +69,18 @@ public class Char implements Comparable, Externalizable {
             || obj == Sequence.eofValue;
     }
 
+    /** Return character value of argument; otherwise return -2.
+     * Uses same conversion as {@code castToCharacterOrEof}. */
+    public static int checkCharOrEof(Object obj) {
+        if (obj instanceof Char)
+            return ((Char) obj).intValue();
+        if (obj instanceof Character)
+            return ((Character) obj).charValue();
+        if (obj == Sequence.eofValue)
+            return -1;
+        return -2;
+    }
+
     public static void print(int i, Appendable out) {
         try {
             append(i, out);
@@ -77,12 +91,12 @@ public class Char implements Comparable, Externalizable {
     }
 
     public static void append(int i, Appendable out) throws IOException {
-        if (i >= 0x10000) {
+        if (i < 0x10000)
+            out.append((char) i);
+        else if (i != IGNORABLE_CHAR) {
             out.append((char) (((i - 0x10000) >> 10) + 0xD800));
             out.append((char) ((i & 0x3FF) + 0xDC00));
         }
-        else
-            out.append((char) i);
     }
 
     public final char charValue() {
@@ -107,7 +121,7 @@ public class Char implements Comparable, Externalizable {
             ascii[i] = new Char(i);
     }
 
-    public static Char make(int ch) {
+    public static Char valueOf(int ch) {
         if (ch < 128)
             return ascii[ch];
         synchronized (hashTable) {
@@ -115,6 +129,10 @@ public class Char implements Comparable, Externalizable {
         }
     }
 
+    public static Char make(int ch) {
+        return valueOf(ch);
+    }
+
     public static Object makeOrEof(int ch) {
         if (ch < 0)
             return Sequence.eofValue;
@@ -163,6 +181,8 @@ public class Char implements Comparable, Externalizable {
             if (charNames[i].equalsIgnoreCase(name))
                 return charNameValues.charAt(i);
         }
+        if ("ignorable-char".equalsIgnoreCase(name))
+            return IGNORABLE_CHAR;
         int len = name.length();
         if (len > 1 && name.charAt(0) == 'u') {
             int value = 0;
@@ -234,7 +254,9 @@ public class Char implements Comparable, Externalizable {
                 return sbuf.toString();
             }
         }
-        if (ch < ' ' || ch > 0x7F) {
+        if (ch == IGNORABLE_CHAR) {
+            sbuf.append("ignorable-char");
+        } else if (ch < ' ' || ch > 0x7F) {
             sbuf.append('x');
             sbuf.append(Integer.toString(ch, 16));
         } else
@@ -243,32 +265,15 @@ public class Char implements Comparable, Externalizable {
     }
 
     /**
-     * @serialData Writes the char value as a char.
-     *   If the value is {@code > 0xFFFF}, write a pair of surrogate values.
-     *   If the value is is a high surrogate only,
-     *   write it followed by {@code '\0'}.
+     * @serialData Writes the char value as an int.
      */
     public void writeExternal(ObjectOutput out) throws IOException {
-        if (value > 0xD800) {
-            if (value > 0xFFFF) {
-                out.writeChar(((value - 0x10000) >> 10) + 0xD800);
-                value = (value & 0x3FF) + 0xDC00;
-            } else if (value <= 0xDBFF) {
-                out.writeChar(value);
-                value = '\0';
-            }
-        }
-        out.writeChar(value);
+        out.writeInt(value);
     }
 
     public void readExternal(ObjectInput in)
         throws IOException, ClassNotFoundException {
-        value = in.readChar();
-        if (value >= 0xD800 && value <= 0xDBFF) {
-            char next = in.readChar();
-            if (next >= 0xDC00 && next <= 0xDFFF)
-                value = ((value - 0xD800) << 10) + (next - 0xDC00) + 0x10000;
-        }
+        value = in.readInt();
     }
 
     public Object readResolve() throws ObjectStreamException {
