@@ -4,12 +4,14 @@
 package gnu.xml;
 import gnu.lists.*;
 import java.io.*;
-import gnu.text.*;
 import gnu.math.RealNum;
-import gnu.text.PrettyWriter;
-import gnu.mapping.OutPort;
+import gnu.kawa.io.PrettyWriter;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
 import gnu.mapping.ThreadLocation;
 import gnu.mapping.Symbol;
+import gnu.text.Char;
+import gnu.text.Lexer;
 import java.math.BigDecimal;
 import gnu.expr.Keyword;
 import gnu.kawa.xml.XmlNamespace;
@@ -23,7 +25,7 @@ public class XMLPrinter extends OutPort
    * -1: don't add indentation; 0: pretty-print (avoid needless newlines);
    * 1: indent (force). */
   public int printIndent = -1;
-  /** When indentating, should attributes be lined up? */
+  /** When indenting, should attributes be lined up? */
   public boolean indentAttributes;
 
   boolean printXMLdecl = false;
@@ -926,7 +928,7 @@ public class XMLPrinter extends OutPort
     prev = PROC_INST;
   }
 
-  public void consume (SeqPosition position)
+  public void writePosition(SeqPosition position)
   {
     position.sequence.consumeNext(position.ipos, this);
   }
