@@ -2,13 +2,19 @@ package gnu.xquery.testsuite;
 import java.io.*;
 import java.util.*;
 import gnu.lists.*;
-import gnu.text.*;
 import gnu.mapping.*;
 import gnu.expr.*;
 import gnu.xml.*;
+import gnu.kawa.io.CharArrayOutPort;
+import gnu.kawa.io.FilePath;
+import gnu.kawa.io.InPort;
+import gnu.kawa.io.Path;
 import gnu.kawa.xml.*;
 import gnu.mapping.Symbol;
 import gnu.xquery.lang.*;
+import gnu.text.SourceError;
+import gnu.text.SourceMessages;
+import gnu.text.SyntaxException;
 import org.xml.sax.helpers.AttributesImpl;
 import gnu.xquery.util.NodeUtils;
 
@@ -64,7 +70,7 @@ public class RunXQTS extends FilterConsumer
   XMLPrinter xqlog;
 
   String collectionID;
-  Values collectionDocuments;
+  Values.FromTreeList collectionDocuments;
 
   private void summaryReport (int count, String label)
   {
@@ -415,7 +421,7 @@ public class RunXQTS extends FilterConsumer
     else if (tagMatches("collection"))
       {
         collectionID = attributes.getValue("ID");
-        collectionDocuments = new Values();
+        collectionDocuments = new Values.FromTreeList();
         sources.put(collectionID, collectionDocuments);
       }
     inStartTag = false;
