@@ -1,5 +1,6 @@
 package kawa;
 
+import gnu.kawa.io.CheckConsole;
 import gnu.kawa.io.InPort;
 import gnu.kawa.io.OutPort;
 import gnu.mapping.*;
@@ -9,7 +10,7 @@ import javax.swing.*;
 import java.awt.event.*;
 
 /** A Frame containing a Kawa read-eval-print loop.
-  * @author Albert Ting <alt@artisan.com> (original base)
+  * @author Albert Ting alt@artisan.com (original base)
   * @author Per Bothner (extensive changes).
   */
 
@@ -27,7 +28,7 @@ public class GuiConsole extends JFrame
   ReplDocument document;
 
   public static void main(String[] args) {
-    InPort.noConsole = false;
+    CheckConsole.setHaveConsole(true);
     int iArg = repl.processArgs(args, 0, args.length);
     repl.getLanguage();
     repl.setArgs(args, iArg);
