@@ -1,8 +1,10 @@
 package kawa;
 
 import java.io.Reader;
+import gnu.kawa.io.OutPort;
+import gnu.kawa.io.Path;
+import gnu.kawa.io.TtyInPort;
 import gnu.mapping.*;
-import gnu.text.Path;
 
 /** A TtyInPort that reads from a ReplPane.
   */
