@@ -226,9 +226,7 @@ public class Shell
   {
     Language saveLanguage = Language.setSaveCurrent(language);
     Lexer lexer = language.getLexer(inp, messages);
-    // Wrong for the case of '-f' '-':
-    //boolean interactive = inp instanceof TtyInPort;
-    boolean interactive = perr != null;
+    boolean interactive = inp instanceof TtyInPort;
     lexer.setInteractive(interactive);
     CallContext ctx = CallContext.getInstance();
     Consumer saveConsumer = null;
@@ -261,8 +259,10 @@ public class Shell
 	      {
 		Compilation comp = language.parse(lexer, opts, null);
                 boolean sawError;
-                if (interactive)
+                if (interactive) {
                   sawError = messages.checkErrors(perr, 20);
+                  perr.flush();
+                }
                 else if (messages.seenErrors())
                   throw new SyntaxException(messages);
                 else
@@ -378,9 +378,9 @@ public class Shell
       {
 	if (env != orig_env)
 	  Environment.setCurrent(env);
-        if (! (path instanceof FilePath))
+        File zfile = path.toFile();
+        if (zfile == null)
           throw new RuntimeException ("load: "+name+" - not a file path");
-	File zfile = ((FilePath) path).toFile();
 	if (!zfile.exists ())
 	  throw new RuntimeException ("load: "+name+" - not found");
 	if (!zfile.canRead ())
@@ -522,6 +522,12 @@ public class Shell
                                : new VoidConsumer());
                         Throwable ex
                             = run(language, env, src, out, perr, url, messages);
+                        if (ex instanceof SyntaxException
+                            && ((SyntaxException) ex).getMessages() == messages) {
+                            messages.printAll(perr, 20);
+                            perr.flush();
+                            return false;
+                        }
                         if (ex != null)
                             throw ex;
                     } else {
@@ -559,6 +565,7 @@ public class Shell
             ctx.values = Values.noArgs;
             Object inst = ModuleExp.evalModule1(env, comp, url, null);
             messages.printAll(perr, 20);
+            perr.flush();
             if (inst == null || messages.seenErrors())
                 return null;
             return new CompiledModule(comp.getModule(), inst, language);
