@@ -6,6 +6,7 @@ import gnu.bytecode.Type;
 import gnu.kawa.lispexpr.LangObjType;
 import gnu.kawa.lispexpr.LispLanguage;
 import java.util.ArrayList;
+import kawa.standard.object;
 
 /**
  * The Syntax transformer that re-writes the lambda builtin.
@@ -14,6 +15,9 @@ import java.util.ArrayList;
 
 public class Lambda extends Syntax
 {
+
+    public boolean handlePatterns;
+
   public Object optionalKeyword;
   public Object restKeyword;
   public Object keyKeyword;
@@ -73,6 +77,8 @@ public class Lambda extends Syntax
   public void rewriteFormals(LambdaExp lexp, Object formals,
 		      Translator tr, TemplateScope templateScopeRest)
   {
+    if (handlePatterns)
+        tr.pushScope(lexp);
     if (lexp.getSymbol() == null)
       {
         String filename = lexp.getFileName();
@@ -91,7 +97,8 @@ public class Lambda extends Syntax
     ArrayList<Expression> defaultArgs = null;
     ArrayList<Keyword> keywords = null;
     Object mode = null;
-    for (; ;  bindings = pair.getCdr())
+    Object next = null;
+    for (; ;  bindings = next)
       {
 	if (bindings instanceof SyntaxForm)
 	  {
@@ -106,6 +113,7 @@ public class Lambda extends Syntax
 	TemplateScope templateScope = templateScopeRest;
 	pair = (Pair) bindings;
 	Object pair_car = pair.getCar();
+        next = pair.getCdr();
 	if (pair_car instanceof SyntaxForm)
 	  {
 	    SyntaxForm sf = (SyntaxForm) pair_car;
@@ -162,7 +170,8 @@ public class Lambda extends Syntax
             break;
 	  }
         pair_car = tr.namespaceResolve(pair_car);
-	if (pair_car instanceof Symbol)
+	if (pair_car instanceof Symbol
+            && ! (handlePatterns && mode == null))
           {
             name = pair_car;
             if (pair.getCdr() instanceof Pair
@@ -178,9 +187,11 @@ public class Lambda extends Syntax
                 p = (Pair) p.getCdr();
                 typeSpecPair = p;
                 pair = p;
+                next = pair.getCdr();
               }
           }
-	else if (pair_car instanceof Pair)
+	else if (pair_car instanceof Pair
+                 && ! (handlePatterns && mode == null))
 	  {
 	    p = (Pair) pair_car;
 	    pair_car = p.getCar();
@@ -246,15 +257,41 @@ public class Lambda extends Syntax
 				       + name + '\''+" after type "+p.getCar());
 			break;
 		      }
+                    tr.error('w', "deprecated type specifier syntax - use (VAR :: TYPE) rather than (VAR TYPE)");
 		  }
 	      }
 	  }
-	if (name == null)
-	  {
-	    tr.syntaxError ("parameter is neither name nor (name :: type) nor (name default)"+": "+pair);
-            break;
-	  }
-	Declaration decl = new Declaration(name);
+        Declaration decl;
+        if (handlePatterns && mode == null) {
+            p = (Pair) pair;
+            pair_car = p.getCar();
+            boolean extraParens = false;
+            if (pair_car instanceof Pair) {
+                Object pair_car_cdr = ((Pair) pair_car).getCdr();
+                if (pair_car_cdr instanceof Pair
+                    && tr.matches(((Pair) pair_car_cdr).getCar(), "::"))
+                    extraParens = true;
+            }
+            Object[] r = BindDecls.instance.parsePatternCar(extraParens ? (Pair) pair_car : p, 0, lexp, tr);
+            if (! extraParens)
+                next = r[0];
+            else if (r[0] != LList.Empty) {
+                Object savePos1 = tr.pushPositionOf(r[0]);
+                tr.syntaxError("junk at end of specifier for parameter");
+                tr.popPositionOf(savePos1);
+            }
+            decl = (Declaration) r[1];
+            if (decl == null)
+                decl = new Declaration("<error>");
+            name = decl == null ? null : decl.getSymbol();
+        } else {
+            if (name == null) {
+                tr.syntaxError ("parameter is neither name nor (name :: type) nor (name default)"+": "+pair);
+                break;
+            }
+            decl = new Declaration(name);
+        }
+        decl.setFlag(Declaration.IS_PARAMETER);
 	if (mode == optionalKeyword || mode == keyKeyword)
           {
             decl.setInitValue(new LangExp(defaultValue));
@@ -271,10 +308,15 @@ public class Lambda extends Syntax
             decl.setType(new LangExp(typeSpecPair), null);
 	    decl.setFlag(Declaration.TYPE_SPECIFIED);
 	  }
-	else if (mode == restKeyword)
-	  decl.setType(LangObjType.listType);
+        if (mode == restKeyword)
+	  {
+	    decl.setFlag(Declaration.IS_REST_PARAMETER);
+	    if (! decl.getFlag(Declaration.TYPE_SPECIFIED))
+	      decl.setType(LangObjType.listType);
+	  }
         decl.setFlag(Declaration.IS_SINGLE_VALUE);
-	addParam(decl, templateScope, lexp, tr);
+        if (! handlePatterns || mode != null)
+            addParam(decl, templateScope, lexp, tr);
 	tr.popPositionOf(savePos);
       }
     if (bindings instanceof SyntaxForm)
@@ -295,7 +337,9 @@ public class Lambda extends Syntax
             rest_args = 1;
             Declaration decl = new Declaration(bindings);
             decl.setType(LangObjType.listType);
-            decl.setFlag(Declaration.IS_SINGLE_VALUE);
+            decl.setFlag(Declaration.IS_SINGLE_VALUE
+                         |Declaration.IS_PARAMETER
+                         |Declaration.IS_REST_PARAMETER);
             decl.noteValueUnknown();
             addParam(decl, templateScopeRest, lexp, tr);
           }
@@ -324,21 +368,22 @@ public class Lambda extends Syntax
       lexp.keywords = keywords.toArray(new Keyword[keywords.size()]);
   }
 
-  protected static void addParam (Declaration decl, ScopeExp templateScope,
-				  LambdaExp lexp, Translator tr)
+  protected void addParam (Declaration decl, ScopeExp templateScope,
+                           LambdaExp lexp, Translator tr)
   {
     if (templateScope != null)
       decl = tr.makeRenamedAlias(decl, templateScope);
     lexp.addDeclaration(decl);
     if (templateScope != null)
       decl.context = templateScope;
+    if (handlePatterns)
+        tr.push(decl);
   }
 
   public Object rewriteAttrs(LambdaExp lexp, Object body, Translator tr)
   {
-    String accessFlagName = null;
     String allocationFlagName = null;
-    int accessFlag = 0;
+    long accessFlag = 0;
     int allocationFlag = 0;
     SyntaxForm syntax0 = null;
     for (;;)
@@ -389,40 +434,10 @@ public class Lambda extends Syntax
 	  }
 	else if (attrName == kawa.standard.object.accessKeyword)
 	  {
-	    Expression attrExpr = tr.rewrite_car(pair2, syntax1);
-	    if (! (attrExpr instanceof QuoteExp)
-		|| ! ((attrValue = ((QuoteExp) attrExpr).getValue()) instanceof SimpleSymbol
-                      /* #ifdef use:java.lang.CharSequence */
-		      || attrValue instanceof CharSequence
-                      /* #else */
-		      // || attrValue instanceof String
-                      // || attrValue instanceof CharSeq
-                      /* #endif */
-                      ))
-	      tr.error('e', "access: value not a constant symbol or string");
-	    else if (lexp.nameDecl == null)
-	      tr.error('e', "access: not allowed for anonymous function");
-	    else
-	      {
-		String value = attrValue.toString();
-		if ("private".equals(value))
-		  accessFlag = Declaration.PRIVATE_ACCESS;
-		else if ("protected".equals(value))
-		  accessFlag = Declaration.PROTECTED_ACCESS;
-		else if ("public".equals(value))
-		  accessFlag = Declaration.PUBLIC_ACCESS;
-		else if ("package".equals(value))
-		  accessFlag = Declaration.PACKAGE_ACCESS;
-		else
-		  tr.error('e', "unknown access specifier");
-		if (accessFlagName != null && value != null)
-		  {
-		    tr.error('e', "duplicate access specifiers - "
-			     + accessFlagName + " and "
-			     + value);
-		  }
-		accessFlagName = value;
-	      }
+            accessFlag = object.addAccessFlags(pair2.getCar(),
+                                               accessFlag,
+                                               Declaration.METHOD_ACCESS_FLAGS,
+                                               "method", tr);
 	  }
 	else if (attrName == kawa.standard.object.allocationKeyword)
 	  {
@@ -532,7 +547,8 @@ public class Lambda extends Syntax
         && tr.getModule().getFlag(ModuleExp.SUPERTYPE_SPECIFIED))
       tr.curMethodLambda = lexp;
     ScopeExp curs = tr.currentScope();
-    tr.pushScope(lexp);
+    if (! handlePatterns)
+        tr.pushScope(lexp);
     if (lexp.nameDecl != null)
       rewriteAnnotations(lexp.nameDecl, tr);
     Declaration prev = null;
@@ -560,6 +576,8 @@ public class Lambda extends Syntax
           }
 	prev = cur;
 
+        if (cur.getFlag(Declaration.IS_PARAMETER)) {
+
         if (arg_i >= lexp.min_args
             && (arg_i < lexp.min_args + opt_args
                 || lexp.max_args >= 0
@@ -568,8 +586,10 @@ public class Lambda extends Syntax
             cur.setInitValue(tr.rewrite(cur.getInitValue()));
           }
         arg_i++;
+        }
 
-        tr.lexical.push(cur);
+        if (! handlePatterns)
+            tr.lexical.push(cur);
       }
 
     if (lexp.isClassMethod()
@@ -597,12 +617,15 @@ public class Lambda extends Syntax
         lexp.setCoercedReturnValue(texp, tr.getLanguage());
       }
     else if (lexp.body instanceof BeginExp
+        && body instanceof Pair
+        && ((Pair) body).getCar() instanceof Symbol
         && (len = (exps = ((BeginExp) lexp.body).getExpressions()).length) > 1
         && (exps[0] instanceof ReferenceExp
             || ((val = exps[0].valueIfConstant()) instanceof Type
                 || val instanceof Class)))
       {
 	// Handle '<TYPENAME> BODY':
+        tr.error('w', "deprecated return-type specifier - use '::TYPE'");
         Expression rexp = exps[0];
         len--;
         if (len == 1)
