@@ -395,21 +395,6 @@ public class Translator extends Compilation
 	  {
             Declaration saveContext = macroContext;
             Syntax syntax = check_if_Syntax (decl);
-            if (syntax == null)
-            {
-                Expression dval = Declaration.followAliases(decl).getValue();
-                Object val = dval == null ? null : dval.valueIfConstant();
-                SimpleSymbol eqmacSymbol = Symbol.valueOf("equivalent-syntax");
-                Object syn;
-                if (dval instanceof LambdaExp)
-                    syn = ((LambdaExp) dval).getProperty(eqmacSymbol, null);
-                else if (val instanceof Procedure)
-                    syn = ((Procedure) val).getProperty(eqmacSymbol, null);
-                else
-                    syn = null;
-                if (syn instanceof Procedure)
-                    syntax = new Macro(null, (Procedure) syn);
-            }
             if (syntax != null)
               {
                 Expression e = apply_rewrite (syntax, p);
@@ -446,7 +431,8 @@ public class Translator extends Compilation
     int first_keyword = -1;
     int last_keyword = -1;
     int firstSpliceArg = -1;
-    for (int i = 0; i < cdr_length;)
+    int i = 0;
+    while (cdr != LList.Empty)
       {
 	if (cdr instanceof SyntaxForm)
 	  {
@@ -460,21 +446,41 @@ public class Translator extends Compilation
         Object save_pos = pushPositionOf(cdr);
 	Pair cdr_pair = (Pair) cdr;
         Object cdr_car = cdr_pair.getCar();
+        Object cdr_cdr = cdr_pair.getCdr();
         Expression arg;
         if (cdr_car instanceof Keyword) {
-            if (first_keyword < 0)
+            if (first_keyword < 0) {
                 first_keyword = i;
-            else if (keywordsAreSelfEvaluating())
+                last_keyword = i - 2; // To suppress incorrect warnings
+            }
+            if (keywordsAreSelfEvaluating())
                 ;
             else if (i == last_keyword + 1 || i + 1 == cdr_length)
-                error('w', "missing keyword value");
+                error('w', "missing value after unquoted keyword");
             else if (i != last_keyword + 2)
                 error('w', "keyword separated from other keyword arguments");
             last_keyword = i;
             arg = QuoteExp.getInstance(cdr_car, this);
             arg.setFlag(QuoteExp.IS_KEYWORD);
-        }
-        else {
+        } else if (cdr_cdr instanceof Pair
+                   // FIXME should check binding for ... is builtin ...
+                   && ((Pair) cdr_cdr).getCar() == LispLanguage.dots3_sym) { 
+            LambdaExp dotsLambda = new LambdaExp();
+            pushScanContext(dotsLambda);
+            dotsLambda.body = rewrite_car(cdr_pair, false);
+            List<Expression> seqs = currentScanContext.sequences;
+            int nseqs = seqs.size();
+            Expression[] subargs = new Expression[nseqs + 1];
+            subargs[0] = dotsLambda;
+            for (int j = 0;  j < nseqs; j++)
+                subargs[j+1] = seqs.get(j);
+            arg = new ApplyExp(Scheme.map, subargs);
+            arg = new ApplyExp(MakeSplice.quoteInstance, arg);
+            popScanContext();
+            cdr_cdr = ((Pair) cdr_cdr).getCdr();
+            if (firstSpliceArg < 0)
+                firstSpliceArg = i + (applyFunction != null ? 1 : 0);
+        } else {
             if (cdr_car instanceof Pair
                 && ((Pair) cdr_car).getCar() == LispLanguage.splice_sym) {
                 arg = rewrite_car((Pair) ((Pair) cdr_car).getCdr(), false);
@@ -488,7 +494,7 @@ public class Translator extends Compilation
         i++;
 
         vec.addElement(arg);
-	cdr = cdr_pair.getCdr();
+	cdr = cdr_cdr;
         popPositionOf(save_pos);
       }
 
@@ -520,12 +526,6 @@ public class Translator extends Compilation
         return CompileNamedPart.makeExp(part1, part2);
     }
 
-    /** A language-dependent "apply" function for generic application.
-     */
-    public Expression applyFunction(Expression func) {
-        return null;
-    }
-
   public Namespace namespaceResolvePrefix (Expression context)
   {
     if (context instanceof ReferenceExp)
@@ -774,9 +774,11 @@ public class Translator extends Compilation
                     LambdaExp caller = (LambdaExp) scope;
                     ClassExp cexp = (ClassExp) scope.getOuter();
                     ClassType ctype = (ClassType) cexp.getClassType();
-                    // BUG: lookupMember doesn't work if ctype
-                    // is a class that hasn't been compiled yet,
-                    // such that ClassExp#declareParts hasn't been called.
+                    // If ctype is a class that hasn't been compiled yet (and
+                    // ClassExp#declareParts hasn't been called yet)
+                    // then we may may get a tentative Field created
+                    // by ClassExp#createFields.
+                    // BUG: This doesn't work for not-yet compiled methods.
                     Member part = SlotGet.lookupMember(ctype, dname, ctype);
                     boolean contextStatic
                         = (caller == cexp.clinitMethod
@@ -912,9 +914,20 @@ public class Translator extends Compilation
             if (decl == null && function
                 && nameToLookup==LispLanguage.lookup_sym)
                 decl = getNamedPartDecl;
+            int scanNesting = decl == null ? 0 : decl.getScanNesting();
             ReferenceExp rexp = new ReferenceExp (nameToLookup, decl);
             rexp.setContextDecl(cdecl);
             rexp.setLine(this);
+            if (scanNesting > 0) {
+                if (getScanContext() == null)
+                    error('e', "using scan variable "+decl.getName()+" while not in scan context");
+                else {
+                    Declaration paramDecl =
+                        currentScanContext.getLambda().addParameter(null);
+                    currentScanContext.addSeqExpression(rexp);
+                    return new ReferenceExp(paramDecl);
+                }
+            }
             if (function && separate)
                 rexp.setFlag(ReferenceExp.PREFER_BINDING2);
             return rexp;
@@ -924,6 +937,10 @@ public class Translator extends Compilation
             return (Expression) exp;
         else if (exp == Special.abstractSpecial)
             return QuoteExp.abstractExp;
+        else if (exp == Boolean.TRUE)
+            return QuoteExp.trueExp;
+        else if (exp == Boolean.FALSE)
+            return QuoteExp.falseExp;
         else if (exp == Special.nativeSpecial)
             return QuoteExp.nativeExp;
         else {
@@ -1565,7 +1582,7 @@ public class Translator extends Compilation
             Object savePos = pushPositionOf(pair);
 	    scanForm(pair.getCar(), defs);
             popPositionOf(savePos);
-            if (getState() == Compilation.PROLOG_PARSED)
+            if (getState() == Compilation.PROLOG_PARSED && pendingForm != null)
               {
                 // We've seen a require form during the initial pass when
                 // we're looking module names.  Defer the require and any
@@ -1607,7 +1624,7 @@ public class Translator extends Compilation
   }
 
   /**
-   * Re-write a Scheme <body> in S-expression format into internal form.
+   * Re-write a Scheme 'body' in S-expression format into internal form.
    */
 
   public Expression rewrite_body (Object exp)
@@ -1616,8 +1633,8 @@ public class Translator extends Compilation
     // This is confusing, at the least.  FIXME.
     Object saved = pushPositionOf(exp);
     LetExp defs = new LetExp();
-    int renamedAliasOldSize =
-        renamedAliasStack == null ? 0 : renamedAliasStack.size();
+    defs.setFlag(LetExp.IS_BODY_SCOPE);
+    int renamedAliasOldSize = renamedAliasCount();
     Pair first = formStack.last;
     defs.setOuter(current_scope);
     current_scope = defs;
@@ -1636,9 +1653,8 @@ public class Translator extends Compilation
               }
           }
         rewriteBody(list);
-        int renamedAliasNewSize =
-            renamedAliasStack == null ? 0 : renamedAliasStack.size();
-        popRenamedAlias((renamedAliasNewSize - renamedAliasOldSize) >> 1);
+        int renamedAliasNewSize = renamedAliasCount();
+        popRenamedAlias(renamedAliasNewSize - renamedAliasOldSize);
 	Expression body = makeBody(first, null);
 	setLineOf(body);
 	if (ndecls == 0)
@@ -1698,7 +1714,7 @@ public class Translator extends Compilation
 
   public boolean appendBodyValues () { return false; }
 
-  /** Combine a <body> consisting of a list of expression. */
+  /** Combine a 'body' consisting of a list of expression. */
   public Expression makeBody(Expression[] exps)
   {
     if (appendBodyValues())
@@ -1836,8 +1852,6 @@ public class Translator extends Compilation
       }
     pendingImports = null;
 
-    processAccesses();
-
     setModule(mexp);
     Compilation save_comp = Compilation.setSaveCurrent(this);
     try
@@ -1845,6 +1859,9 @@ public class Translator extends Compilation
         Pair firstForm = formStack.getHead();
         rewriteBody((LList) formStack.popTail(firstForm));
 	mexp.body = makeBody(firstForm, mexp);
+
+        processAccesses();
+
         // In immediate mode need to preserve Declaration for current "session".
         if (! immediate)
 	  lexical.pop(mexp);
@@ -1896,17 +1913,17 @@ public class Translator extends Compilation
   }
 
   /** Push an alias for a declaration in a scope.
-   * If the name of <code>decl</code> came from a syntax template
-   * whose immediate scope is <code>templateScope</code>,
+   * If the name of {@code decl}> came from a syntax template
+   * whose immediate scope is {@code templateScope},
    * then the same syntax template may contain local variable references
-   * that are also in the same <code>templateScope</code>.
+   * that are also in the same {@code templateScope}.
    * Such variable references will <em>not</em> look in the current
-   * "physical" scope, where we just created <code>decl</code>, but
-   * will instead search the "lexical" <code>templateScope</scope>.
-   * So that such references can resolve to <code>decl</code>, we
-   * create an alias in <code>templateScope</code> that points
-   * to <code>decl</code>.  We record that we did this in the
-   * <code>renamedAliasStack</code>, so we can remove the alias later.
+   * "physical" scope, where we just created {@code decl}, but
+   * will instead search the "lexical" {@code templateScope}.
+   * So that such references can resolve to {@code decl}, we
+   * create an alias in {@code templateScope} that points
+   * to {@code decl}.  We record that we did this in the
+   * {@code renamedAliasStack}, so we can remove the alias later.
    */
   public void pushRenamedAlias (Declaration alias)
   {
@@ -1923,6 +1940,10 @@ public class Translator extends Compilation
     renamedAliasStack.push(alias);
   }
 
+    public int renamedAliasCount() {
+        return renamedAliasStack == null ? 0 : renamedAliasStack.size() >> 1;
+    }
+
   /** Remove one or more aliases created by <code>pushRenamedAlias</code>. */
   public void popRenamedAlias (int count)
   {
@@ -1939,15 +1960,25 @@ public class Translator extends Compilation
       }
   }
 
+    public Declaration define(Object name, ScopeExp defs) {
+        return define(name, (TemplateScope) null, defs);
+    }
+
     public Declaration define(Object name, SyntaxForm nameSyntax,
                               ScopeExp defs) {
-        ScopeExp scope = nameSyntax != null ? nameSyntax.getScope()
+        return define(name, nameSyntax == null ? null : nameSyntax.getScope(),
+                      defs);
+    }
+
+    public Declaration define(Object name, TemplateScope templateScope,
+                              ScopeExp defs) {
+        ScopeExp scope = templateScope != null ? templateScope
             : currentScope();
         boolean aliasNeeded = scope != defs;
         Object declName = aliasNeeded
             ? Symbol.makeUninterned(name.toString())
             : name;
-        Declaration decl = defs.getDefine(declName, 'w', this);
+        Declaration decl = defs.getDefine(declName, this);
         if (aliasNeeded) {
             Declaration alias = makeRenamedAlias(name, decl, scope);
             if (defs instanceof LetExp)
@@ -2049,4 +2080,30 @@ public class Translator extends Compilation
             this.values = values;
         }
     }
+
+    private ScanContext currentScanContext;
+
+    public ScanContext getScanContext() { return currentScanContext; }
+    public void setScanContext(ScanContext ctx) { currentScanContext = ctx; }
+
+    public void pushScanContext (LambdaExp lambda) {
+        ScanContext newContext = new ScanContext();
+        newContext.outer = currentScanContext;
+        currentScanContext = newContext;
+        newContext.lambda = lambda;
+    }
+    public void popScanContext() {
+        currentScanContext = currentScanContext.outer;
+    }
+    public static class ScanContext {
+        ScanContext outer;
+        ArrayList<Expression> sequences = new ArrayList<Expression>();
+        LambdaExp lambda;
+
+        public LambdaExp getLambda() { return lambda; }
+
+        public void addSeqExpression(Expression exp) {
+            sequences.add(exp);
+        }
+    }
 }
