@@ -20,10 +20,10 @@
 (define (null? x) :: <boolean>
   (eq? (gnu.mapping.Promise:force x) '()))
 
-(define (set-car! (p <pair>) x)
+(define (set-car! (p ::pair) x)
   (set! p:car x))
 
-(define (set-cdr! (p <pair>) x)
+(define (set-cdr! (p ::pair) x)
   (set! p:cdr x))
 
 (define-procedure car
@@ -92,8 +92,9 @@
 (define-cxr cdddar (cdr cdr cdr car))
 (define-cxr cddddr (cdr cdr cdr cdr))
 
-(define (length list :: <list>) :: <int>
-  (invoke-static <list> 'length list))
+(define (length list ::sequence) :: int
+  validate-apply: "kawa.lib.compile_misc:lengthValidateApply"
+  (list:size))
 
 (define (reverse (list :: <list>)) :: <list>
   (let loop ((arg list) (result '()))
@@ -136,25 +137,22 @@
 
 (define (memq x list)
   (let lp ((lst list))
-    (and (instance? lst <pair>)
-	 (let ((p :: pair lst))
-	   (if (eq? x p:car) lst
-	       (lp p:cdr))))))
+    (and (? p::pair lst)
+         (if (eq? x p:car) lst
+             (lp p:cdr)))))
 
 (define (memv x list)
   (let lp ((lst list))
-    (and (instance? lst <pair>)
-	 (let ((p :: pair lst))
-	   (if (eqv? x p:car) lst
-	       (lp p:cdr))))))
+    (and (? p::pair lst)
+         (if (eqv? x p:car) lst
+             (lp p:cdr)))))
 
 ;;;  The optional test argument is an srfi-1 extension.
 (define (member x list #!optional (test :: <procedure> equal?))
   (let lp ((lst list))
-    (and (instance? lst <pair>)
-	 (let ((p :: pair lst))
-	   (if (test x p:car) lst
-	       (lp p:cdr))))))
+    (and (? p::pair lst)
+         (if (test x p:car) lst
+             (lp p:cdr)))))
 
 (define (assq x list)
   (let lp ((list list))
