@@ -71,18 +71,20 @@
 (define (port? x)
   (or (input-port? x) (output-port? x)))
 
-(define (input-port-open? (port ::input-port))
-  (port:isOpen))
+(define (input-port-open? port)
+  (if (gnu.kawa.io.OutPort? port) #f
+      ((->gnu.kawa.io.InPort port):isOpen)))
 
-(define (output-port-open? (port ::output-port))
-  (port:isOpen))
+(define (output-port-open? port)
+  (if (gnu.kawa.io.InPort? port) #f
+      ((->gnu.kawa.io.OutPort port):isOpen)))
 
-(define-alias-parameter current-input-port <input-port>
-  (static-field <input-port> 'inLocation))
-(define-alias-parameter current-output-port <output-port>
-  (static-field <output-port> 'outLocation))
-(define-alias-parameter current-error-port <output-port>
-  (static-field <output-port> 'errLocation))
+(define-alias-parameter current-input-port input-port
+  (static-field input-port 'inLocation))
+(define-alias-parameter current-output-port output-port
+  (static-field output-port 'outLocation))
+(define-alias-parameter current-error-port output-port
+  (static-field output-port 'errLocation))
 
 (define-constant current-path ::gnu.mapping.LocationProc
   (gnu.mapping.LocationProc:makeNamed
@@ -128,7 +130,7 @@
 (define (open-output-string) :: <string-output-port>
   (<string-output-port>))
 
-(define (get-output-string (output-port  <string-output-port>))
+(define (get-output-string (output-port :: <string-output-port>))
   (<gnu.lists.FString> (output-port:toCharArray)))
 
 (define (open-input-bytevector (bvector ::bytevector))
