@@ -2,14 +2,15 @@
 
 (module-export
  define-syntax define define-private define-constant define-early-constant
- report-syntax-error syntax->expression syntax-body->expression
+ define-variable report-syntax-error syntax->expression syntax-body->expression
  if try-catch letrec)
 
-(import (rename (only (kawa standard define) defineRaw) (defineRaw %define)))
-(import (rename (only (kawa standard define_syntax) define_syntax)
-                (define_syntax %define-syntax)))
-(import (rename (only (kawa standard let) let) (let %let)))
-(import (rename (only (kawa standard set_b) set) (set set!)))
+(import (only (kawa standard define) (defineRaw %define)))
+(import (only (kawa standard define_syntax) (define_syntax %define-syntax)))
+(import (only (kawa standard let) (let %let)))
+(import (only (kawa standard set_b) (set set!)))
+(import (only (kawa standard begin) begin))
+(import (only (kawa standard SchemeCompilation) mlambda))
 
 (%define-syntax define-syntax
   (syntax-rules ($lookup$)
@@ -56,8 +57,6 @@
      (%define ($lookup$ part1 'part2) 9 type value))
     ((define-constant ($lookup$ part1 'part2) value)
      (%define ($lookup$ part1 'part2) 8 #!null value))
-    ((define-constant (name . formals) . body)
-     (%define name 10 #t formals . body))
     ((define-constant name :: type value)
      (%define name 9 type value))
     ((define-constant name value)
@@ -74,6 +73,21 @@
     ((define-early-constant name value)
      (%define name 24 #!null value))))
 
+(%define-syntax define-variable
+  (syntax-rules (:: $lookup$)
+    ((define-variable name :: type)
+     (define-variable name :: type #!undefined))
+    ((define-variable name)
+     (define-variable name #!undefined))
+    ((define-variable ($lookup$ part1 'part2) :: type value)
+     (%define ($lookup$ part1 'part2) 33 type value))
+    ((define-variable ($lookup$ part1 'part2) value)
+     (%define ($lookup$ part1 'part2) 32 #!null value))
+    ((define-variable name :: type value)
+     (%define name 33 type value))
+    ((define-variable name value)
+     (%define name 32 #!null value))))
+
 (%define report-syntax-error 2 #!null (id #!rest (msg :: <Object[]>))
   (invoke-static <kawa.standard.syntax_error> 'error id msg))
 
@@ -87,25 +101,56 @@
     ((syntax-body->expression x)
      (kawa.lang.SyntaxForms:rewriteBody x))))
 
+(define-syntax %if-and-x
+ (lambda (x)
+   (syntax-case x (? :: and)
+    ((_ k then)
+     (gnu.expr.ExitExp (syntax->expression #'then) #'k))
+    ((_ k then test1 . rest)
+     #`(if test1
+         (%if-and-x k then . rest))))))
+
 (define-rewrite-syntax if
   (lambda (x)
-    (syntax-case x ()
-		 ((_ test then)
-		  (make <gnu.expr.IfExp>
-		    (syntax->expression (syntax test))
-		    (syntax->expression (syntax then))
-		    #!null))
-		 ((_ test then else)
-		  (make <gnu.expr.IfExp>
-		    (syntax->expression (syntax test))
-		    (syntax->expression (syntax then))
-		    (syntax->expression (syntax else))))
-		 ((_ e1 e2 e3 . rest)
-		  (report-syntax-error #'rest
-				"too many expressions for 'if'"))
-		 ((_ . rest)
-		  (report-syntax-error #'rest
-				"too few expressions for 'if'")))))
+    (syntax-case x (? :: and)
+      ((_ (and . tests) then else)
+       (%let ((bl (gnu.expr.BlockExp)))
+             (bl:setRunFinallyBlocks #f)
+             (bl:setBody
+              (gnu.expr.BeginExp
+               (syntax->expression #`(%if-and-x #,bl then . tests))
+               (syntax->expression #`else)))
+             bl))
+      ((_ (? . rest) then)
+       #'(if (? . rest) then #!void))
+      ((_ (and . rest) then)
+       #'(if (and . rest) then #!void))
+      ((_ (? pattern :: type init) then else)
+       #`(#,gnu.kawa.reflect.TypeSwitch:typeSwitch
+          init
+          (mlambda (pattern :: type) then)
+          (lambda (unused) else)))
+      ((_ (? pattern init) then else)
+       #`(#,gnu.kawa.reflect.TypeSwitch:typeSwitch
+          init
+          (mlambda (pattern) then)
+          (lambda (unused) else)))
+      ((_ test then)
+       (make <gnu.expr.IfExp>
+         (syntax->expression (syntax test))
+         (syntax->expression (syntax then))
+         #!null))
+      ((_ test then else)
+       (make <gnu.expr.IfExp>
+         (syntax->expression (syntax test))
+         (syntax->expression (syntax then))
+         (syntax->expression (syntax else))))
+      ((_ e1 e2 e3 . rest)
+       (report-syntax-error #'rest
+                            "too many expressions for 'if'"))
+      ((_ . rest)
+       (report-syntax-error #'rest
+                            "too few expressions for 'if'")))))
 
 (define-rewrite-syntax try-catch
   (lambda (x)
@@ -121,6 +166,8 @@
     (%let ((out-bindings '()) (out-inits '()))
       (syntax-case form ()
 	((_ bindings . body)
+         ;; Simulate a (letrec ((process-binding (lambda ...) ...)) ...)
+         ;; which we can't use in here, because that is what we're defining.
 	 (%let ((process-binding #!undefined))
 	       (set! process-binding
 		     (lambda (b)
