@@ -1,4 +1,3 @@
 (module-name (scheme write))
 (require kawa.lib.ports)
-(export
- display write write-shared write-simple)
+(include "write-exports")
