@@ -1,7 +1,7 @@
 ;;; Definitions for some standard syntax.
 
 (module-export cond and or let let* do delay lazy delay-force
-               else ... => _ else unquote unquote-splicing
+               else ... ? => _ else unquote unquote-splicing
                syntax->datum datum->syntax with-syntax
 	       syntax-object->datum datum->syntax-object ; deprecated
 	       generate-temporaries define-procedure
@@ -16,6 +16,13 @@
 (import (only (kawa standard Scheme) not))
 (import (only (kawa standard begin) begin))
 
+(define-syntax ?
+  (syntax-rules ()
+    ((_ id :: type)
+     (syntax-error "missing init expression for '?'"))
+    ((_ . rest)
+     (syntax-error "'?' is only allowed in a conditional e.g. 'if' or 'and'"))))
+
 (define-syntax =>
   (syntax-rules ()
     ((_ . rest)
@@ -96,8 +103,7 @@
 	       ((_) (%lang-boolean #t))
 	       ((_ test) (syntax test))
 	       ((_ test1 . test2)
-		#`(%let ((x test1))
-			  (if x (and . test2) x)))))
+		(syntax (if test1 (and . test2) #f)))))
 ;;; OR
 
 (define-syntax (or f)
@@ -113,9 +119,9 @@
 (define-syntax %let-lambda1
   (syntax-rules (::)
 		((%let-lambda1 ((var type init) . in) out body)
-		 (%let-lambda1 in ((var type) . out) body))
+		 (%let-lambda1 in ((var :: type) . out) body))
 		((%let-lambda1 ((var :: type init) . in) out body)
-		 (%let-lambda1 in ((var type) . out) body))
+		 (%let-lambda1 in ((var :: type) . out) body))
 		((%let-lambda1 ((var init) . in) out body)
 		 (%let-lambda1 in (var . out) body))
 		((%let-lambda1 () out body)
