@@ -82,7 +82,7 @@
 (define-compare string<=? string <= %string-compare2)
 (define-compare string>=? string >= %string-compare2)
 
-(define (substring (str <string>) (start <int>) (end <int>))
+(define (substring str::<string> start::<int> end::<int>)
   :: <string>
   (with-start-end str (start end) (istart iend)
                   (gnu.lists.FString str istart (- iend istart))))
@@ -100,7 +100,7 @@
            (loop (make <pair> (string-cursor-ref str prev) result) prev))))))
 
 (define (list->string (lst ::list)) ::string
-  (let* ((len ::int (length lst))
+  (let* ((len ::int (lst:size))
 	 (result ::string (make gnu.lists.FString len)))
     (do ((i ::int 0 (+ i 1)))
 	((>= i len) result)
@@ -142,14 +142,23 @@
 (define (string-fill! str ::abstract-string ch ::character
                       #!optional
                       (start ::int 0)
-                      (end ::int (string-length str)))
+                      (end ::int -1))
   ::void
-  (let ((width ::int (if (> (as int ch) #xffff) 2 1)))
-    (do ((to-do ::int (- end start) (- to-do 1))
-         (pos ::int (java.lang.Character:offsetByCodePoints str 0 start)
-              (+ pos width)))
-        ((<= to-do 0))
-      (str:setCharacterAt pos (as int ch)))))
+  (! cstart ::int (java.lang.Character:offsetByCodePoints str 0 start))
+  (! send (if (>= end 0) end (gnu.lists.Strings:sizeInCodePoints str)))
+  (cond ((? fstr::gnu.lists.FString str)
+         (gnu.lists.FString? str)
+         (let ((cend (if (< end 0) (fstr:length)
+                         (java.lang.Character:offsetByCodePoints str 0 end))))
+           (fstr:delete cstart cend)
+           (fstr:insertRepeated cstart (as int ch) (- send start))))
+        (else
+         ;; Can be O(n^2) if a bad mix of 1-char and 2-char characters.
+         (let ((width ::int (if (> (as int ch) #xffff) 2 1)))
+           (do ((to-do ::int (- send start) (- to-do 1))
+                (pos ::int cstart (+ pos width)))
+               ((<= to-do 0))
+             (str:setCharacterAt pos (as int ch)))))))
 
 (define (string-upcase! str ::abstract-string) ::string
   (gnu.lists.Strings:makeUpperCase str)
