@@ -2,7 +2,7 @@
 	       begin-for-syntax define-for-syntax
                when unless try-finally synchronized
                let-values let*-values case-lambda define-values
-               receive define-alias-parameter
+               define-alias-parameter $bracket-list$
                $string$ $string-with-default-format$ $format$ $sprintf$
                $string-with-delimiter-marks$ define-simple-constructor)
 
@@ -76,29 +76,39 @@
 ;; This code is in the public domain.
 
 (define-syntax let-values
-  (syntax-rules ()
-    ((let-values (?binding ...) ?body0 ?body1 ...)
-     (let-values "bind" (?binding ...) () (begin ?body0 ?body1 ...)))
-    ((let-values "bind" () ?tmps ?body)
-     (let ?tmps ?body))
+  (lambda (form)
+    (syntax-case form ()
+      ;; Optimize the common/simple case.  Not only is macro expansion faster,
+      ;; but (more importantly) the resulting Expression tree is simpler.
+      ((let-values ((formals init)) . body)
+       #'(call-with-values
+             (lambda() init)
+           (lambda formals . body)))
+      ((let-values (?binding ...) ?body0 ?body1 ...)
+       #'(let-values "bind" (?binding ...) () (begin ?body0 ?body1 ...)))
+      ((let-values "bind" () ?tmps ?body)
+       #'(let ?tmps ?body))
     
-    ((let-values "bind" ((?b0 ?e0) ?binding ...) ?tmps ?body)
-     (let-values "mktmp" ?b0 ?e0 () (?binding ...) ?tmps ?body))
+      ((let-values "bind" ((?b0 ?e0) ?binding ...) ?tmps ?body)
+       #'(let-values "mktmp" ?b0 ?e0 () (?binding ...) ?tmps ?body))
     
-    ((let-values "mktmp" () ?e0 ?args ?bindings ?tmps ?body)
-     (call-with-values 
-       (lambda () ?e0)
-       (lambda ?args
-         (let-values "bind" ?bindings ?tmps ?body))))
+      ((let-values "mktmp" () ?e0 ?args ?bindings ?tmps ?body)
+       #'(call-with-values 
+           (lambda () ?e0)
+           (lambda ?args
+             (let-values "bind" ?bindings ?tmps ?body))))
     
-    ((let-values "mktmp" (?a . ?b) ?e0 (?arg ...) ?bindings (?tmp ...) ?body)
-     (let-values "mktmp" ?b ?e0 (?arg ... x) ?bindings (?tmp ... (?a x)) ?body))
+      ((let-values "mktmp" (?a . ?b) ?e0 (?arg ...) ?bindings (?tmp ...) ?body)
+       (let ((xvar (datum->syntax #'x (syntax->datum #'?a))))
+         #`(let-values "mktmp" ?b ?e0 (?arg ... #,xvar)
+                       ?bindings (?tmp ... (?a #,xvar)) ?body)))
     
-    ((let-values "mktmp" ?a ?e0 (?arg ...) ?bindings (?tmp ...) ?body)
-     (call-with-values
-       (lambda () ?e0)
-       (lambda (?arg ... . x)
-         (let-values "bind" ?bindings (?tmp ... (?a x)) ?body))))))
+      ((let-values "mktmp" ?a ?e0 (?arg ...) ?bindings (?tmp ...) ?body)
+       (let ((xvar (datum->syntax #'x (syntax->datum #'?a))))
+         #`(call-with-values
+             (lambda () ?e0)
+             (lambda (?arg ... . #,xvar)
+               (let-values "bind" ?bindings (?tmp ... (?a #,xvar)) ?body))))))))
 
 (define-syntax let*-values
   (syntax-rules ()
@@ -163,32 +173,11 @@
        (call-with-values (lambda () expr)
          list)))))
 
-;; RECEIVE implementation from http://srfi.schemers.org/srfi-8/srfi-8.html
-;; Copyright (C) John David Stone (1999). All Rights Reserved.
-;; This document and translations of it may be copied and furnished to
-;; others, and derivative works that comment on or otherwise explain
-;; it or assist in its implementation may be prepared, copied,
-;; published and distributed, in whole or in part, without restriction
-;; of any kind, provided that the above copyright notice and this
-;; paragraph are included on all such copies and derivative
-;; works. However, this document itself may not be modified in any
-;; way, such as by removing the copyright notice or references to the
-;; Scheme Request For Implementation process or editors, except as
-;; needed for the purpose of developing SRFIs in which case the
-;; procedures for copyrights defined in the SRFI process must be
-;; followed, or as required to translate it into languages other than
-;; English.
-(define-syntax receive
-  (syntax-rules ()
-    ((receive formals expression body ...)
-     (call-with-values (lambda () expression)
-                       (lambda formals body ...)))))
-
 (define-syntax define-alias-parameter
   (syntax-rules ()
     ((define-alias-parameter name type location)
      (begin
-       (define-constant name :: <gnu.mapping.LocationProc>
+       (define-constant name :: gnu.mapping.LocationProc[type]
 	 (gnu.mapping.LocationProc:makeNamed 'name location))
        (gnu.mapping.LocationProc:pushConverter
 	name
@@ -287,6 +276,15 @@
     ((_ fmt . args)
      (gnu.kawa.functions.Format:sprintfToString fmt . args))))
 
+(define-syntax $bracket-list$
+  (syntax-rules (<: <=: >: >=:)
+    ((_ x <: y) (invoke-static gnu.lists.Range 'valueOfLT x y))
+    ((_ x <=: y) (invoke-static gnu.lists.Range 'valueOfLE x y))
+    ((_ x >: y) (invoke-static gnu.lists.Range 'valueOfGT x y))
+    ((_ x >=: y) (invoke-static gnu.lists.Range 'valueOfGE x y))
+    ((_ x <:) (invoke-static gnu.lists.Range 'valueOfUnbounded x))
+    ((_ . args) (constant-vector . args))))
+
 (cond-expand
  (kawa
   ;; (%symbol->construct 'sym) returns '$construct$:sym
