@@ -9,7 +9,7 @@
 (define (vector-length x :: <vector>) :: <int>
   (invoke x 'size))
 
-(define (vector-set! (vector <vector>) (k <int>) obj) :: <void>
+(define (vector-set! (vector ::vector) (k ::int) obj) :: <void>
   (invoke vector 'set k obj))
 
 (define-procedure vector-ref
@@ -78,57 +78,65 @@
 (define (vector-fill! (vec :: vector) fill
                       #!optional (start ::int 0) (end ::int (vec:size)))
   :: void
-  (vec:fill fill start end))
+  (vec:fill start end fill))
 
 ;;; vector-map and vector-for-each are mandated by R6RS. These
 ;;; implementations are optimized for the one-vector case, and permit
 ;;; vectors of varying length, using the shortest vector as the
 ;;; limiting factor.
-(define (vector-map (f :: procedure) (vec :: vector)
-                    #!rest (vecs :: vector[]))
+(define (vector-map (proc :: procedure) vec1 #!rest (vecs :: object[]))
   :: vector
-  (define (vector-map-one (f :: procedure) (vec :: vector)) :: vector
-    (let* ((len :: int (vector-length vec))
+  validate-apply: "kawa.lib.compile_map:vectorMapValidateApply"
+  (define (vector-map-simple proc::procedure vec::java.util.List) :: vector
+    (let* ((len :: int (vec:size))
            (r :: vector (make-vector len)))
       (do ((i :: int 0 (+ i 1)))
           ((= i len) r)
-        (vector-set! r i (f (vector-ref vec i))))))
-  (define (vector-map-generic (f :: procedure)
-                              (vec :: vector)
-                              #!rest (vecs :: vector[]))
-    :: vector
-    (let loop ((ls :: list '())
-                (len :: int (vector-length vec))
-                (i :: int (- vecs:length 1)))
-      (if (>= i 0)
-          (loop (cons (vecs i) ls)
-                (min len (vector-length (vecs i)))
-                (- i 1))
-          (do ((r :: vector (make-vector len))
-               (i :: int 0 (+ i 1)))
-              ((= i len) r)
-            (vector-set! r i
-                         (apply f (vector-ref vec i)
-                                (map (lambda (v) (vector-ref v i))
-                                     ls)))))))
-  (if (= 0 vecs:length)
-      (vector-map-one f vec)
-      (vector-map-generic f vec vecs)))
+        (vector-set! r i (proc (vec:get i))))))
+  (define (vector-map-one proc vec) :: vector
+    (if (and (? fp::procedure proc) (? vlist::gnu.lists.SimpleVector vec))
+        (vector-map-simple fp vlist)
+        (let* ((r :: vector (make-vector 0))
+               (it (gnu.lists.Sequences:getIterator vec)))
+          (do ((i :: int 0 (+ i 1))) ((not (it:hasNext)) r)
+            (vector-set! r i (proc (it:next)))))))
 
-(define (vector-for-each (f :: procedure) (vec :: vector)
-                         #!rest (vecs :: vector[]))
+  (if (= 0 vecs:length)
+      (vector-map-one proc vec1)
+      (let* ((nargs (+ vecs:length 1))
+             (iterators (java.util.Iterator[] length: nargs))
+             (elements (object[] length: nargs))
+             (result (make-vector 0)))
+        (do ((i ::int 0 (+ i 1))) ((= i nargs))
+          (set! (iterators i)
+                (gnu.lists.Sequences:getIterator
+                 (if (= i 0) vec1 (vecs (- i 1))))))
+        (let loop1 ()
+          (let loop2 ((i ::int 0))
+            (cond ((= i nargs)
+                   (result:add (proc @elements))
+                   (loop1))
+                  (((iterators i):hasNext)
+                   (set! (elements i) ((iterators i):next))
+                   (loop2 (+ i 1)))
+                  (else
+                   result)))))))
+
+(define (vector-for-each (f :: procedure) (vec :: java.util.List)
+                         #!rest (vecs :: java.util.List[]))
   :: void
-  (define (vector-for-each-one (f :: procedure) (vec :: vector))
+  validate-apply: "kawa.lib.compile_map:vectorForEachValidateApply"
+  (define (vector-for-each-one (f :: procedure) (vec :: java.util.List))
     :: void
-    (let ((len :: int (vector-length vec)))
+    (let ((len :: int (vec:size)))
       (do ((i :: int 0 (+ i 1)))
           ((= i len))
-        (f (vector-ref vec i)))))
+        (f (vec:get i)))))
   (define (vector-for-each-generic (f :: procedure)
-                                   (vec :: vector)
-                                   #!rest (vecs :: vector[]))
+                                   (vec :: java.util.List)
+                                   #!rest (vecs :: java.util.List[]))
     (let loop ((ls :: list '())
-               (len :: int (vector-length vec))
+               (len :: int (vec:size))
                (i :: int (- vecs:length 1)))
       (if (>= i 0)
           (loop (cons (vecs i) ls)
@@ -136,8 +144,8 @@
                 (- i 1))
           (do ((i :: int 0 (+ i 1)))
               ((= i len))
-            (apply f (vector-ref vec i)
-                   (map (lambda (v) (vector-ref v i))
+            (apply f (vec:get i)
+                   (map (lambda (v::java.util.List) (v:get i))
                         ls))))))
   (if (= 0 vecs:length)
       (vector-for-each-one f vec)
