@@ -32,7 +32,7 @@ public class ImportFromLibrary extends Syntax
         { "2", "and-let*", "gnu.kawa.slib.srfi2" },
         { "5", "let", MISSING },
         { "6", "basic-string-ports", BUILTIN },
-        { "8", "receive", BUILTIN },
+        { "8", "receive", "gnu.kawa.slib.receive" },
         { "9", "records", BUILTIN },
         { "11", "let-values", BUILTIN },
         { "13", "strings", "gnu.kawa.slib.srfi13" },
@@ -49,6 +49,8 @@ public class ImportFromLibrary extends Syntax
         { "28", "basic-format-strings", BUILTIN },
         { "29", "localization", MISSING },
         { "31", "rec", MISSING },
+        { "35", "conditions", "gnu.kawa.slib.conditions" },
+        { "37", "args-fold", "gnu.kawa.slib.srfi37" },
         { "38", "with-shared-structure", MISSING },
         { "39", "parameters", BUILTIN },
         // Note the default for (srfi :41) should be "streams".  We put that last,
@@ -99,7 +101,7 @@ public class ImportFromLibrary extends Syntax
 
     public static String checkSrfi(String lname, Translator tr) {
         if (lname.startsWith("srfi.")) {
-            String demangled = Compilation.demangleName(lname.substring(5));
+            String demangled = Compilation.demangleSymbolic(lname.substring(5));
             int dot = demangled.indexOf('.');
             String srfiName;
             StringBuilder badNameBuffer = null;
@@ -157,6 +159,13 @@ public class ImportFromLibrary extends Syntax
     }
 
     void scanImportSet(Object imports, ScopeExp defs, Translator tr, require.DeclSetMapper mapper) {
+        if (imports instanceof SimpleSymbol) {
+            String sname = imports.toString();
+            handleImport(sname, null,
+                         Compilation.mangleQualifiedName(sname),
+                         defs, tr, mapper);
+            return;
+        }
         int specLength = Translator.listLength(imports);
         if (specLength <= 0) {
             Object save1 = tr.pushPositionOf(imports);
@@ -180,9 +189,65 @@ public class ImportFromLibrary extends Syntax
         else if (first == librarySymbol && specLength == 2
                  && cdrPair.getCar() instanceof Pair)
             pimport = (Pair) cdrPair.getCar();
-        if (specLength >= 2 && kind != '\0'
-            // A keyword such as 'only must be followed by an <import set>.
-            && cdrPair.getCar() instanceof LList) {
+        else if (first == classSymbol && specLength >= 2
+                 && cdrPair.getCar() instanceof SimpleSymbol) {
+            Map<Symbol, Expression> decls =
+                new LinkedHashMap<Symbol, Expression>();
+            SimpleSymbol name1 = (SimpleSymbol) cdrPair.getCar();
+            String str1 = name1.getName();
+            rest = cdrPair.getCdr();
+            if (rest == LList.Empty) {
+                tr.error('e', "class-prefix must be followed by class-names");
+            }
+            while (rest != LList.Empty) {
+                cdrPair = (Pair) rest;
+                Object part2 = cdrPair.getCar();
+                String cname = null;
+                SimpleSymbol dname = null;
+                if (part2 instanceof SimpleSymbol) {
+                    dname = (SimpleSymbol) part2;
+                    String str2 = dname.getName();
+                    cname = name1+"."+str2;
+                } else if (part2 instanceof Pair
+                           && Translator.listLength(part2) == 2) {
+                    Pair rpair1 = (Pair) part2;
+                    Pair rpair2 = (Pair) rpair1.getCdr();
+                    Object rname1 = rpair1.getCar();
+                    Object rname2 = rpair2.getCar();
+                    if (rname1 instanceof SimpleSymbol
+                        && rname2 instanceof SimpleSymbol) {
+                        cname = name1 + "." + ((SimpleSymbol) rname1).getName();
+                        dname = (SimpleSymbol) rname2;
+                    }
+                }
+                if (dname == null) {
+                    tr.error('e', "imported class-name must be NAME or (NAME NEW-NAME)");
+                } else {
+                    try {
+                        Class clas = ObjectType.getContextClass(cname);
+                        decls.put(dname, tr.makeQuoteExp(clas));
+                    } catch (ClassNotFoundException ex) {
+                        tr.error('e', "no class found named "+cname);
+                    }
+                }
+                rest = cdrPair.getCdr();
+            }
+            if (mapper != null)
+                decls = mapper.map(decls, tr);
+            for (Map.Entry<Symbol,Expression> entry : decls.entrySet()) {
+                Symbol aname = entry.getKey();
+                Declaration decl = tr.define(aname, defs);
+                decl.setAlias(true);
+                decl.setFlag(Declaration.IS_CONSTANT|Declaration.EARLY_INIT);
+                SetExp sexp = new SetExp(decl, entry.getValue());
+                tr.setLineOf(sexp);
+                decl.noteValueFromSet(sexp);
+                sexp.setDefining (true);
+                tr.formStack.push(sexp);
+            }
+            return;
+        }
+        if (specLength >= 2 && kind != '\0') {
             ImportSetMapper nmapper
                 = new ImportSetMapper(kind, cdrPair.getCdr(), specLength-2);
             nmapper.chain = mapper;
@@ -214,7 +279,7 @@ public class ImportFromLibrary extends Syntax
                 if (sbuf.length() > 0)
                     sbuf.append('/');
                 String part = car.toString();
-                cbuf.append(Compilation.mangleNameIfNeeded(part));
+                cbuf.append(Compilation.mangleClassName(part));
                 sbuf.append(part);
             }
             libref = cdr;
@@ -250,12 +315,12 @@ public class ImportFromLibrary extends Syntax
                 existingClass = ObjectType.getContextClass(tname);
                 break;
             } catch (Exception ex) {
+            } catch (NoClassDefFoundError ex) {
             }
         }
 
-        String currentFileName = tr.getFileName();
-        Path currentSource = currentFileName == null ? null
-            : Path.valueOf(currentFileName).getAbsolute();
+        ModuleInfo curinfo = tr.getMinfo();
+        Path currentSource = tr.getSourceAbsPath();
         String currentExtension = currentSource == null ? null
             : currentSource.getExtension();
         if (currentExtension == null) {
@@ -274,12 +339,11 @@ public class ImportFromLibrary extends Syntax
             isAbsolute = false;
         }
 
-        ModuleInfo curinfo = tr.getMinfo();
         String currentClassName = curinfo.getClassName();
 
         // Is the current module a file - as opposed to (say) a tty?
-        boolean currentIsFile = currentSource instanceof FilePath
-            && ((FilePath) currentSource).toFileRaw().isFile();
+        boolean currentIsFile = currentSource != null
+            && currentSource.isPlainFile();
         Path currentRoot = currentIsFile ? currentSource.getDirectory()
             : Path.currentPath();
         if (currentIsFile
@@ -358,8 +422,10 @@ public class ImportFromLibrary extends Syntax
                     pbuf.append(implicitSource.substring(prefixLength));
                     pbuf.append(pathElement.substring(star+1));
                 } else {
-                    pbuf.append(pathElement);
-                    pbuf.append('/');
+                    if (! ".".equals(pathElement)) {
+                        pbuf.append(pathElement);
+                        pbuf.append('/');
+                    }
                     if (explicitSource != null)
                         pbuf.append(explicitSource);
                     else {
@@ -372,13 +438,13 @@ public class ImportFromLibrary extends Syntax
                 }
                 pathStr = pbuf.toString();
             }
-            Path path = currentRoot.resolve(pathStr);
+            Path path = currentRoot.resolve(pathStr).getCanonical();
             // Might be more efficient to first check the ModuleManager,
             // before asking the file-system.  FIXME
             long lastModifiedTime = path.getLastModified();
             if (lastModifiedTime != 0) {
                 if (minfo != null) {
-                    String pstring = path.getCanonical().toString();
+                    String pstring = path.toString();
                     Path infoPath = minfo.getSourceAbsPath();
                     if (infoPath == null
                         || ! (pstring.equals(infoPath.toString()))) {
@@ -410,7 +476,7 @@ public class ImportFromLibrary extends Syntax
     }
 
     public Expression rewriteForm(Pair form, Translator tr) {
-        return null;
+        return tr.syntaxError(getName()+" is only allowed in a <body>");
     }
 
     static class ImportSetMapper implements require.DeclSetMapper {
@@ -425,32 +491,47 @@ public class ImportFromLibrary extends Syntax
             this.listLength = listLength;
         }
 
-        public Map<Symbol, Declaration> map(Map<Symbol, Declaration> decls, Compilation comp) {
+        public Map<Symbol, Expression> map(Map<Symbol, Expression> decls, Compilation comp) {
             Translator tr = (Translator) comp;
             Object lst = this.list;
-            Map<Symbol,Declaration> nmap = decls;
+            Map<Symbol,Expression> nmap = decls;
 
             switch (kind) {
             case 'E': // 'except; list has the form (name ...)
             case 'O': // 'only; list has the form (name ...)
                 if (kind == 'O')
-                    nmap = new LinkedHashMap<Symbol,Declaration>();
+                    nmap = new LinkedHashMap<Symbol,Expression>();
                 while (lst instanceof Pair) {
                     Pair pair = (Pair) lst;
                     Object save1 = tr.pushPositionOf(pair);
                     Object name = Translator.stripSyntax(pair.getCar());
+                    Symbol oldsym = null;
+                    Symbol newsym = null;
                     if (name instanceof Symbol) {
-                        Symbol sym = (Symbol) name;
-                        Declaration old = decls.get(sym);
-                        if (old == null)
-                            tr.error('e', "unknown symbol in import set: "+sym);
-                        else if (kind == 'E')
-                            nmap.remove(sym);
-                        else
-                            nmap.put(sym, old);
+                        oldsym = (Symbol) name;
+                        newsym = oldsym;
+                    } else if (kind == 'O' && name instanceof Pair
+                           && Translator.listLength(name) == 2) {
+                        Pair rpair1 = (Pair) name;
+                        Object rname1 = rpair1.getCar();
+                        Object rname2 = ((Pair) rpair1.getCdr()).getCar();
+                        if (rname1 instanceof Symbol
+                            && rname2 instanceof Symbol) {
+                            oldsym = (Symbol) rname1;
+                            newsym = (Symbol) rname2;
+                        }
                     }
-                    else
+                    if (oldsym == null)
                         tr.error('e', "non-symbol in name list");
+                    else {
+                        Expression old = decls.get(oldsym);
+                        if (old == null)
+                            tr.error('e', "unknown symbol in import set: "+oldsym);
+                        else if (kind == 'E')
+                            nmap.remove(oldsym);
+                        else
+                            nmap.put(newsym, old);
+                    }
                     tr.popPositionOf(save1);
                     lst = pair.getCdr();
                 }
@@ -458,7 +539,7 @@ public class ImportFromLibrary extends Syntax
 
             case 'R': // 'rename; list has the form: ((oldname newname) ...)
                 Symbol[] pendingSymbols = new Symbol[listLength];
-                Declaration[] pendingDecls = new Declaration[listLength];
+                Expression[] pendingDecls = new Expression[listLength];
                 int npending = 0;
                 while (lst instanceof Pair) {
                     Pair pair = (Pair) lst;
@@ -473,12 +554,12 @@ public class ImportFromLibrary extends Syntax
                             && newname instanceof Symbol) {
                             Symbol oldSymbol = (Symbol) oldname;
                             Symbol newSymbol = (Symbol) newname;
-                            Declaration oldDecl = decls.remove(oldSymbol);
-                            if (oldDecl == null)
+                            Expression oldValue = decls.remove(oldSymbol);
+                            if (oldValue == null)
                                 tr.error('e', "missing binding "+oldSymbol);
                             else {
                                 pendingSymbols[npending] = newSymbol;
-                                pendingDecls[npending] = oldDecl;
+                                pendingDecls[npending] = oldValue;
                                 npending++;
                             }
                         }
@@ -492,25 +573,25 @@ public class ImportFromLibrary extends Syntax
                 }
                 for (int i = 0;  i < npending;  i++) {
                     Symbol newSymbol = pendingSymbols[i];
-                    Declaration decl = pendingDecls[i];
+                    Expression decl = pendingDecls[i];
                     if (decls.put(newSymbol, decl) != null)
                         tr.error('e', "duplicate binding for "+newSymbol);
                 }
                 break;
 
             case 'P':  // 'prefix; list has the form: (name-prefix)
-                nmap = new LinkedHashMap<Symbol,Declaration>();
+                nmap = new LinkedHashMap<Symbol,Expression>();
                 if (listLength != 1
                     || ! (((Pair) list).getCar() instanceof SimpleSymbol))
                     tr.error('e', "bad syntax for prefix import specifier");
                 else {
                     String prefix
                         = ((SimpleSymbol) ((Pair) list).getCar()).getName();
-                    for (Map.Entry<Symbol,Declaration> entry : decls.entrySet()) {
+                    for (Map.Entry<Symbol,Expression> entry : decls.entrySet()) {
                         Symbol aname = entry.getKey();
-                        Declaration decl = entry.getValue();
+                        Expression old = entry.getValue();
                         Symbol nname = Symbol.valueOf(prefix+aname);
-                        nmap.put(nname, decl);
+                        nmap.put(nname, old);
                     }
                 }
                 break;
@@ -550,6 +631,7 @@ public class ImportFromLibrary extends Syntax
         return Include.getSearchPath(searchPath, "kawa.import.path", ".");
     }
 
+    public static final SimpleSymbol classSymbol = Symbol.valueOf("class");
     public static final SimpleSymbol exceptSymbol = Symbol.valueOf("except");
     public static final SimpleSymbol librarySymbol = Symbol.valueOf("library");
     public static final SimpleSymbol onlySymbol = Symbol.valueOf("only");
