@@ -12,103 +12,84 @@ import gnu.kawa.functions.*;
 import gnu.kawa.io.CharArrayInPort;
 import gnu.kawa.io.InPort;
 import gnu.kawa.reflect.LazyType;
+import gnu.kawa.reflect.MultValuesType;
 import gnu.kawa.servlet.HttpRequestContext;
 
-public class Scheme extends LispLanguage
-{
-  public static final Environment nullEnvironment;
-  public static final Environment r4Environment;
-  public static final Environment r5Environment;
-  public static final Environment r6Environment;
-  protected static final SimpleEnvironment kawaEnvironment;
+public class Scheme extends LispLanguage {
+    public static final int FOLLOW_R5RS = 5;
+    public static final int FOLLOW_R6RS = 6;
+    public static final int FOLLOW_R7RS = 7;
 
-  public static final LangPrimType booleanType;
+    int standardToFollow;
 
-  public static final int FOLLOW_R5RS = 5;
-  public static final int FOLLOW_R6RS = 6;
-  public static final int FOLLOW_R7RS = 7;
+    public int getStandardToFollow() { return standardToFollow; }
 
-  public static final Scheme instance;
-  private static Scheme r5rsInstance;
-  private static Scheme r6rsInstance;
-  private static Scheme r7rsInstance;
-  int standardToFollow;
+    public static final Environment nullEnvironment =
+        Environment.make("null-environment");
+    public static final Environment r4Environment =
+        Environment.make("r4rs-environment", nullEnvironment);
+    public static final Environment r5Environment =
+        Environment.make("r5rs-environment", r4Environment);
+    public static final Environment r6Environment =
+        Environment.make("r6rs-environment", r5Environment);
+    protected static final SimpleEnvironment kawaEnvironment =
+        Environment.make("kawa-environment", r6Environment);
 
-  public int getStandardToFollow() { return standardToFollow; }
+    public static final Scheme instance = new Scheme(kawaEnvironment);
+    public static final Scheme r5rsInstance = newStandardInstance(FOLLOW_R5RS);
+    public static final Scheme r6rsInstance = newStandardInstance(FOLLOW_R6RS);
+    public static final Scheme r7rsInstance = newStandardInstance(FOLLOW_R7RS);
 
-  public static final gnu.kawa.reflect.InstanceOf instanceOf;
-  public static final Not not;
-  public static final gnu.kawa.functions.Map map;
-  public static final gnu.kawa.functions.Map forEach;
-  public static final gnu.kawa.functions.IsEq isEq;
-  public static final gnu.kawa.functions.IsEqv isEqv;
-  public static final gnu.kawa.functions.IsEqual isEqual;
+    public static final LangPrimType booleanType =
+        new LangPrimType(Type.booleanType, instance);
 
-  public static final NumberCompare numEqu;
-  public static final NumberCompare numGrt;
-  public static final NumberCompare numGEq;
-  public static final NumberCompare numLss;
-  public static final NumberCompare numLEq;
-  public static final NumberPredicate isOdd;
-  public static final NumberPredicate isEven;
+    public static final ApplyToArgs applyToArgs =
+        new ApplyToArgs("apply-to-args", instance);
+    public static final Apply apply =
+        new Apply("apply", applyToArgs);
 
-  public static final Apply apply;
-  public static final ApplyToArgs applyToArgs;
+    public static final gnu.kawa.reflect.InstanceOf instanceOf =
+        new gnu.kawa.reflect.InstanceOf(instance, "instance?");
+    public static final Not not =
+        new Not(instance, "not");
 
-  private static final String[] uniformVectorTags =
+    public static final gnu.kawa.functions.IsEq isEq =
+        new gnu.kawa.functions.IsEq(instance, "eq?");
+    public static final gnu.kawa.functions.IsEqv isEqv =
+        new gnu.kawa.functions.IsEqv(instance, "eqv?", isEq);
+    public static final gnu.kawa.functions.IsEqual isEqual =
+        new gnu.kawa.functions.IsEqual(instance, "equal?");
+   
+    public static final gnu.kawa.functions.Map map =
+        new gnu.kawa.functions.Map(true, applyToArgs, isEq);
+    public static final gnu.kawa.functions.Map forEach =
+        new gnu.kawa.functions.Map(false, applyToArgs, isEq);
+    public static final NumberCompare numEqu =
+        NumberCompare.make(instance, "=", NumberCompare.TRUE_IF_EQU);
+    public static final NumberCompare numGrt =
+        NumberCompare.make(instance, ">", NumberCompare.TRUE_IF_GRT);
+    public static final NumberCompare numGEq =
+        NumberCompare.make(instance, ">=",
+                           NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU);
+    public static final NumberCompare numLss =
+        NumberCompare.make(instance, "<", NumberCompare.TRUE_IF_LSS);
+    public static final NumberCompare numLEq =
+        NumberCompare.make(instance, "<=",
+                           NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
+    public static final NumberPredicate isOdd =
+        new NumberPredicate(instance, "odd?", NumberPredicate.ODD);
+    public static final NumberPredicate isEven =
+        new NumberPredicate(instance, "even?", NumberPredicate.EVEN);
+
+    private static final String[] uniformVectorTags =
     {"s8", "s16", "s32", "s64", "u8", "u16", "u32", "u64", "f32", "f64" };
 
     public static final String emptyStringLeft = new String();
     public static final String emptyStringRight = new String();
 
-  static {
-    // (null-environment)
-    nullEnvironment = Environment.make("null-environment");
-    r4Environment = Environment.make("r4rs-environment", nullEnvironment);
-    r5Environment = Environment.make("r5rs-environment", r4Environment);
-    r6Environment = Environment.make("r6rs-environment", r5Environment);
-    kawaEnvironment = Environment.make("kawa-environment", r6Environment);
-
-    instance = new Scheme(kawaEnvironment);
-    instanceOf = new gnu.kawa.reflect.InstanceOf(instance, "instance?");
-    not = new Not(instance, "not");
-    applyToArgs = new ApplyToArgs("apply-to-args", instance);
-    apply = new Apply("apply", applyToArgs);
-    isEq = new gnu.kawa.functions.IsEq(instance, "eq?");
-    isEqv = new gnu.kawa.functions.IsEqv(instance, "eqv?", isEq);
-    isEqual = new gnu.kawa.functions.IsEqual(instance, "equal?");
-    map = new gnu.kawa.functions.Map(true, applyToArgs, isEq);
-    forEach = new gnu.kawa.functions.Map(false, applyToArgs, isEq);
-    numEqu = NumberCompare.make(instance, "=",
-                                NumberCompare.TRUE_IF_EQU);
-    numGrt = NumberCompare.make(instance, ">",
-                                NumberCompare.TRUE_IF_GRT);
-    numGEq = NumberCompare.make(instance, ">=",
-                                NumberCompare.TRUE_IF_GRT|NumberCompare.TRUE_IF_EQU);
-    numLss = NumberCompare.make(instance, "<",
-                                NumberCompare.TRUE_IF_LSS);
-    numLEq = NumberCompare.make(instance, "<=",
-                                NumberCompare.TRUE_IF_LSS|NumberCompare.TRUE_IF_EQU);
-    isOdd = new NumberPredicate(instance, "odd?", NumberPredicate.ODD);
-    isEven = new NumberPredicate(instance, "even?", NumberPredicate.EVEN);
-
-    instance.initScheme();
-
-    booleanType = new LangPrimType(Type.booleanType, instance);
-            
-    int withServlets = HttpRequestContext.importServletDefinitions;
-    if (withServlets > 0)
-      {
-        try
-          {
-            instance.loadClass(withServlets > 1 ? "gnu.kawa.servlet.servlets"
-                               : "gnu.kawa.servlet.HTTP");
-          }
-        catch (Exception ex)
-          {
-          }
-      }
-  }
+    static {
+        instance.initScheme();
+    }
 
   public static Scheme getInstance()
   {
@@ -122,26 +103,17 @@ public class Scheme extends LispLanguage
     return instance;
   }
 
-  public static synchronized Scheme getR5rsInstance()
-  {
-    if (r5rsInstance == null)
-      r5rsInstance = newStandardInstance(FOLLOW_R5RS);
-    return r5rsInstance;
-  }
+    public static synchronized Scheme getR5rsInstance() {
+        return r5rsInstance;
+    }
 
-  public static synchronized Scheme getR6rsInstance()
-  {
-    if (r6rsInstance == null)
-      r6rsInstance = newStandardInstance(FOLLOW_R6RS);
-    return r6rsInstance;
-  }
+    public static synchronized Scheme getR6rsInstance() {
+        return r6rsInstance;
+    }
 
-  public static synchronized Scheme getR7rsInstance()
-  {
-    if (r7rsInstance == null)
-      r7rsInstance = newStandardInstance(FOLLOW_R7RS);
-    return r7rsInstance;
-  }
+    public static synchronized Scheme getR7rsInstance() {
+        return r7rsInstance;
+    }
 
   public static Environment builtin ()
   {
@@ -154,6 +126,7 @@ public class Scheme extends LispLanguage
 
       defSntxStFld("lambda", "kawa.standard.SchemeCompilation", "lambda");
       defSntxStFld("$bracket-apply$", "gnu.kawa.lispexpr.BracketApply", "instance");
+      defSntxStFld("$bracket-list$", "kawa.lib.syntax");
       defSntxStFld("$string$", "kawa.lib.syntax");
       defSntxStFld("$string-with-default-format$", "kawa.lib.syntax");
       defSntxStFld("$format$", "kawa.lib.syntax");
@@ -173,6 +146,7 @@ public class Scheme extends LispLanguage
       //-- Section 4.1  -- complete
       defSntxStFld(LispLanguage.quote_str, "kawa.lang.Quote", "plainQuote");
       defSntxStFld("define", "kawa.lib.prim_syntax");
+      defSntxStFld("!", "kawa.standard.MatchDef", "matchDef");
 
       defSntxStFld("if", "kawa.lib.prim_syntax");
       defSntxStFld ("set!", "kawa.standard.set_b", "set");
@@ -181,6 +155,7 @@ public class Scheme extends LispLanguage
       defSntxStFld("cond", "kawa.lib.std_syntax");
       defSntxStFld("...", "kawa.lib.std_syntax");
       defSntxStFld("_", "kawa.lib.std_syntax");
+      defSntxStFld("?", "kawa.lib.std_syntax");
       defSntxStFld("=>", "kawa.lib.std_syntax");
       defSntxStFld("else", "kawa.lib.std_syntax");
       defSntxStFld("unquote", "kawa.lib.std_syntax");
@@ -351,6 +326,12 @@ public class Scheme extends LispLanguage
       defProcStFld("asin", "kawa.lib.numbers");
       defProcStFld("acos", "kawa.lib.numbers");
       defProcStFld("atan", "kawa.lib.numbers");
+      defProcStFld("sinh", "kawa.lib.numbers");
+      defProcStFld("cosh", "kawa.lib.numbers");
+      defProcStFld("tanh", "kawa.lib.numbers");
+      defProcStFld("asinh", "kawa.lib.numbers");
+      defProcStFld("acosh", "kawa.lib.numbers");
+      defProcStFld("atanh", "kawa.lib.numbers");
       defProcStFld("sqrt", "kawa.lib.numbers");
       defProcStFld("square", "kawa.lib.numbers");
       defProcStFld("expt", "kawa.standard.expt");
@@ -544,13 +525,13 @@ public class Scheme extends LispLanguage
       environ = r5Environment;
 
       defProcStFld("values", "kawa.lib.misc");
-      defProcStFld("call-with-values", "kawa.standard.call_with_values",
+      defProcStFld("call-with-values", "gnu.kawa.functions.CallWithValues",
 		   "callWithValues");
       defSntxStFld("let-values", "kawa.lib.syntax");
       defSntxStFld("let*-values", "kawa.lib.syntax");
       defSntxStFld("define-values", "kawa.lib.syntax");
       defSntxStFld("case-lambda", "kawa.lib.syntax");
-      defSntxStFld("receive", "kawa.lib.syntax");
+      defSntxStFld("receive", "gnu.kawa.slib.receive");
       defProcStFld("eval", "kawa.lib.scheme.eval");
       defProcStFld("repl", "kawa.standard.SchemeCompilation", "repl");
       defProcStFld("scheme-report-environment", "kawa.lib.misc");
@@ -844,8 +825,7 @@ public class Scheme extends LispLanguage
                    "define_alias");
       defSntxStFld("define-private-alias", "kawa.standard.define_alias",
                    "define_private_alias");
-      defSntxStFld("define-variable", "kawa.standard.define_variable",
-                   "define_variable");
+      defSntxStFld("define-variable", "kawa.lib.prim_syntax");
       defSntxStFld("define-member-alias", "kawa.standard.define_member_alias",
                    "define_member_alias");
       defSntxStFld("define-enum", "gnu.kawa.slib.enums");
@@ -908,7 +888,7 @@ public class Scheme extends LispLanguage
 
       defSntxStFld("cut", "gnu.kawa.slib.cut");
       defSntxStFld("cute", "gnu.kawa.slib.cut");
-
+      defSntxStFld("scan", "kawa.standard.Scan", "scan");
       defSntxStFld("cond-expand", "kawa.standard.IfFeature", "condExpand");
 
       defAliasStFld("*print-base*", "gnu.kawa.functions.DisplayFormat",
@@ -927,12 +907,15 @@ public class Scheme extends LispLanguage
       defAliasStFld("$entity$", "gnu.kawa.lispexpr.LispLanguage", "entityNamespace");
       defAliasStFld("$construct$", "gnu.kawa.lispexpr.LispLanguage", "constructNamespace");
 
+      defAliasStFld("dynamic", "gnu.kawa.lispexpr.LangObjType", "dynamicType");
       defAliasStFld("path", "gnu.kawa.lispexpr.LangObjType", "pathType");
       defAliasStFld("filepath", "gnu.kawa.lispexpr.LangObjType", "filepathType");
       defAliasStFld("URI", "gnu.kawa.lispexpr.LangObjType", "URIType");
       defProcStFld("resolve-uri", "kawa.lib.files");
 
-      defAliasStFld("$bracket-list$", "gnu.kawa.lispexpr.LangObjType", "constVectorType");
+      defAliasStFld("sequence", "gnu.kawa.lispexpr.LangObjType", "sequenceType");
+      defAliasStFld("constant-vector",
+                    "gnu.kawa.lispexpr.LangObjType", "constVectorType");
       defAliasStFld("vector", "gnu.kawa.lispexpr.LangObjType", "vectorType");
       defAliasStFld("string", "gnu.kawa.lispexpr.LangObjType", "stringType");
       defAliasStFld("list", "gnu.kawa.lispexpr.LangObjType", "listType");
@@ -967,7 +950,16 @@ public class Scheme extends LispLanguage
       defProcStFld("annotation", "gnu.kawa.reflect.MakeAnnotation", "instance");
 
       kawaEnvironment.setLocked();
-  }
+
+        int withServlets = HttpRequestContext.importServletDefinitions;
+        if (withServlets > 0) {
+            try {
+                loadClass(withServlets > 1 ? "gnu.kawa.servlet.servlets"
+                          : "gnu.kawa.servlet.HTTP");
+            } catch (Exception ex) {
+            }
+        }
+    }
 
   public Scheme ()
   {
@@ -1093,6 +1085,7 @@ public class Scheme extends LispLanguage
         if (types == null) {
             types = new HashMap<String, Type>(128); // Bit more wiggle room
             types.put("boolean", booleanType);
+            types.put("parameter", Compilation.typeLocationProc);
             types.putAll(super.getTypeMap());
             for (int i = uniformVectorTags.length; --i >= 0;) {
                 String tag = uniformVectorTags[i];
@@ -1110,6 +1103,20 @@ public class Scheme extends LispLanguage
             return formatType(ltype.getRawType())
                 +'['+formatType(ltype.getValueType())+']';
         }
+        if (type instanceof MultValuesType) {
+            MultValuesType mtype = (MultValuesType) type;
+            StringBuilder sbuf = new StringBuilder();
+            sbuf.append("values[");
+            int n = mtype.getValueCount();
+            for (int i = 0; i < n; i++) {
+                if (i > 0)
+                    sbuf.append(' ');
+                Type etype = mtype.getValueType(i);
+                sbuf.append(etype == null ? "unspecified" : formatType(etype));
+            }
+            sbuf.append(']');
+            return sbuf.toString();
+        }
         if (typeToStringMap == null)  {
             typeToStringMap = new HashMap<Type,String>();
             // Invert the map returned by getTypeMap.
@@ -1123,7 +1130,7 @@ public class Scheme extends LispLanguage
     }
 
  /** Convert expression to a Type.
-   * Allow "TYPE" or 'TYPE or <TYPE>.
+   * Allow {@code "TYPE"} or {@code 'TYPE} or {@code <TYPE>}.
    */
   public static Type exp2Type (Expression exp)
   {
@@ -1165,6 +1172,7 @@ public class Scheme extends LispLanguage
       {
         tab.postfixLookupOperator = ':';
         tab.setFinalColonIsKeyword(true);
+        tab.extraFlags = LispReader.SCM_LEXPONENT_IS_BIGDECIMAL;
         tab.set('@', new ReaderQuote(LispLanguage.splice_sym,
                                      ReadTable.NON_TERMINATING_MACRO));
       }
