@@ -25,24 +25,24 @@
 (define-namespace Pair2 <gnu.lists.Pair>)
 (define-namespace Pair3 "class:gnu.lists.Pair")
 
-(define (set-car1 (p <pair>) x)  (set! (*:.car p) x))
-(define (set-car2 (p <pair>) x)  (set! (<pair>:.car p) x))
-(define (set-car3 (p <pair>) x)  (set! (gnu.lists.Pair:.car p) x))
-(define (set-car4 (p <pair>) x)  (set! (<gnu.lists.Pair>:.car p) x))
-(define (set-car5 (p <pair>) x)  (set! (Pair1:.car p) x))
-(define (set-car6 (p <pair>) x)  (set! (Pair2:.car p) x))
-(define (set-car7 (p <pair>) x)  (set! (Pair3:.car p) x))
-(define (set-car8 (p <pair>) x)  (set!  p:car x))
+(define (set-car1 (p ::<pair>) x)  (set! (*:.car p) x))
+(define (set-car2 (p ::<pair>) x)  (set! (<pair>:.car p) x))
+(define (set-car3 (p ::<pair>) x)  (set! (gnu.lists.Pair:.car p) x))
+(define (set-car4 (p ::<pair>) x)  (set! (<gnu.lists.Pair>:.car p) x))
+(define (set-car5 (p ::<pair>) x)  (set! (Pair1:.car p) x))
+(define (set-car6 (p ::<pair>) x)  (set! (Pair2:.car p) x))
+(define (set-car7 (p ::<pair>) x)  (set! (Pair3:.car p) x))
+(define (set-car8 (p ::<pair>) x)  (set!  p:car x))
 (define (set-car9 p x)  (set! (Pair1:.car p) x))
 
-(define (get-car1 (p <pair>))  (*:.car p))
-(define (get-car2 (p <pair>))  (<pair>:.car p))
-(define (get-car3 (p <pair>))  (gnu.lists.Pair:.car p))
-(define (get-car4 (p <pair>))  (<gnu.lists.Pair>:.car p))
-(define (get-car5 (p <pair>))  (Pair1:.car p))
-(define (get-car6 (p <pair>))  (Pair2:.car p))
-(define (get-car7 (p <pair>))  (Pair3:.car p))
-(define (get-car8 (p <pair>))  p:car)
+(define (get-car1 (p ::<pair>))  (*:.car p))
+(define (get-car2 (p ::<pair>))  (<pair>:.car p))
+(define (get-car3 (p ::<pair>))  (gnu.lists.Pair:.car p))
+(define (get-car4 (p ::<pair>))  (<gnu.lists.Pair>:.car p))
+(define (get-car5 (p ::<pair>))  (Pair1:.car p))
+(define (get-car6 (p ::<pair>))  (Pair2:.car p))
+(define (get-car7 (p ::<pair>))  (Pair3:.car p))
+(define (get-car8 (p ::<pair>))  p:car)
 (define (get-car9 p)  (Pair3:.car p))
 (define (get-car10 p)  (<pair>:.car p))
 (define (get-car11 p::pair) (car p))
@@ -87,7 +87,7 @@
 (define (new-pair11 x y) (make Pair2:<> x y))
 (define (new-pair12 x y) (make Pair3:<> x y))
 
-(define (is-empty1 (p <pair>)) ;; OK
+(define (is-empty1 (p::<pair>)) ;; OK
   (*:isEmpty p))
 
 (define (make-iarr1 (n :: <int>)) (make <int[]> size: n))
@@ -430,3 +430,106 @@
      ((#\a #\b #\c #\d) 1)
      ((#\e #\f #\g #\h) 2)
      (else 3)))
+
+(define (callWithValues1 x::integer y::integer)
+  (call-with-values (lambda () (floor/ x y))
+    (lambda (a b) (list b a))))
+
+(define (callWithValues2 x::integer y::integer)
+  (call-with-values (lambda () (values (+ x 1) (- y 1)))
+    list))
+
+(define (callWithValues3 x::integer y::integer)
+  (call-with-values (lambda () (floor/ x y))
+    list))
+
+(define (mmemq x list)
+  (let lp ((lst list))
+    (and (? p::pair lst)
+         (if (eq? x p:car) lst
+             (lp p:cdr)))))
+
+(define (greater-equal x y)::boolean
+  (>= x y))
+(define (greater-equal-u32-s32 x::uint y::int)
+  (>= x y))
+(define (greater-equal-u32-s32-generic x::uint y::int)
+  (greater-equal x y))
+(define (greater-equal-u64-s32 x::ulong y::int)
+  (>= x y))
+(define (greater-equal-u64-u64 x::ulong y::ulong)
+  (>= x y))
+(define (greater-equal-u64-u64-generic x::ulong y::ulong)
+  (greater-equal x y))
+
+(define s8a::byte 123)
+(define (increment-s8a) (set! s8a (+ s8a 1)))
+(define (increment-arr-s8 arr::byte[] i::int) (set! (arr i) (+ (arr i) 1)))
+
+(define u8a::ubyte 253)
+(define u16a::ushort #xff35)
+(define (increment-u8a) (set! u8a (+ u8a 1)))
+(define (set-u16a val::int) (set! u16a val))
+(define (add-u8a-u16a)
+  (+ u8a u16a))
+
+(define (rshift-integer x::integer y::int)
+  (bitwise-arithmetic-shift-right x y))
+(define (rshift-s16 x::short y::int)
+  (bitwise-arithmetic-shift-right x y))
+(define (rshift-s32 x::int y::int)
+  (bitwise-arithmetic-shift-right x y))
+(define (rshift-s64 x::long y::int)
+  (bitwise-arithmetic-shift-right x y))
+(define (rshift-u32 x::uint y::int)
+  (bitwise-arithmetic-shift-right x y))
+(define (rshift-u64 x::ulong y::int)
+  (bitwise-arithmetic-shift-right x y))
+
+(define (lshift-integer x::integer y::int)
+  (bitwise-arithmetic-shift-left x y))
+(define (lshift-s16 x::short y::int)
+  (bitwise-arithmetic-shift-left x y))
+(define (lshift-s32 x::int y::int)
+  (bitwise-arithmetic-shift-left x y))
+(define (lshift-s64 x::long y::int)
+  (bitwise-arithmetic-shift-left x y))
+(define (lshift-u32 x::uint y::int)
+  (bitwise-arithmetic-shift-left x y))
+(define (lshift-u64 x::ulong y::int)
+  (bitwise-arithmetic-shift-left x y))
+
+(define (index-s16 i::int)
+  (let ((v #s16(3 5 -12 2)))
+    (v i)))
+(define (make-u8v1 x::int)
+  (u8vector 5 253 x))
+(define (index-u8v1 i::int)
+  (let ((v (make-u8v1 3)))
+    (v i)))
+(define (index-u8i1 i::int) ::int
+  (let ((v (make-u8v1 3)))
+    (v i)))
+(define (index-u8i2 i::int) ::int
+  (index-u8v1 i))
+(define (index-f32 i::int)
+  (let ((v #f32(3.4 1/2 55)))
+    (f32vector-ref v i)))
+
+(define (set-u8vector1 v::u8vector i::int x::long)
+  (u8vector-set! v i x))
+
+(define (set-u8vector2 v::u8vector i::int x::long)
+  (set! (v i) x))
+
+(define (index-seq q::sequence i::int)
+  (q i))
+
+(define (index-str1 x::string i::int)
+  (x i))
+
+(define (index-str2 x::string i::int)
+  ((dynamic x) i))
+
+(define (index-str3 x::string i::int)
+  (index-seq x i))
