@@ -1,9 +1,9 @@
-(test-init "Objects" 143)
+(test-init "Objects" 145)
 
 ;; Force procedure to be applied without being inlined:
 (define-syntax force-eval
   (syntax-rules () ((force-eval proc arg ...)
-                    (apply proc (list arg ...)))))
+                    ((dynamic proc) arg ...))))
 
 ;; Force call to be compiled with (hopefully) inlining:
 (define-syntax force-compile
@@ -44,7 +44,7 @@
        (let*
 	   ((obj (object (<object>)
 			 (fld 10)
-			 ((toString) <String> fld)))
+			 ((toString) ::String fld)))
 	    (val1 (field obj 'fld))
 	    (str1 (as <String> obj)))
 	 (begin
@@ -59,7 +59,7 @@
 	   ((val0 100)
 	    (obj (object (<object>)
 			 (fld val0)
-			 ((toString) <String> fld)))
+			 ((toString) ::String fld)))
 	    (val1 (field obj 'fld))
 	    (str1 (as <String> obj)))
 	 (begin
@@ -83,11 +83,11 @@
 		(letrec ((opt
 			  (lambda (args)
 			    (list obj
-				  (object (<java.lang.Object>
-					   <java.awt.event.ItemListener>)
+				  (object (java.lang.Object
+					   java.awt.event.ItemListener)
 					  ((itemStateChanged
-					    (arg <java.awt.event.ItemEvent>))
-					   <void>
+					    (arg::java.awt.event.ItemEvent))
+					   ::void
 					   (display name) 
 					   (newline)))))))
 		  (opt 3)))))
@@ -101,8 +101,8 @@
 			  (let ((listener
 				 (object (<java.lang.Object>
 					  <java.awt.event.ItemListener>) 
-					 ((itemStateChanged (arg <java.awt.event.ItemEvent>)) 
-					  <void> 
+					 ((itemStateChanged (arg::java.awt.event.ItemEvent)) 
+					  ::void
 					  (display "listener of checkbutton ") 
 					  (display c-name) 
 					  (display arg) 
@@ -117,8 +117,7 @@
 (define (document-filter arg1)
   (lambda (arg2)
     (object ()
-            ((toString)
-             <String>
+            ((toString) ::String
              (format #f "{arg1: ~s arg2: ~s}" arg1 arg2)))))
 
 (test "{arg1: 23 arg2: 12}" 'object-with-closure-3
@@ -231,6 +230,8 @@
 (require <classes1>)
 (require <classes2>)
 
+(test "thunk re-initialized" thunk "thunk re-initialized")
+
 (test 3 slot-ref IsClass2-value 'var1)
 (test 4 slot-ref IsClass2-value 'var2)
 (set! fail-expected "static counter is incorrectly initialized in non-static run method")
@@ -407,3 +408,6 @@
 (test #!null 'before-m2-close m2-object:my-array-7)
 (let ((cl ::java.io.Closeable m2-object)) (cl:close))
 (test #f 'after-m2-close m2-object:my-array-7)
+
+(import (libx))
+(test '(ax in libx) libx-report)
